"""Aplikace pro prevod typovych zkousek ze stare verze GFM do nove (GFM02)"""

import typetests
from os import path

#Kazda typovka zacina radkem, ktery ma jako prvni znak hvezdicku (*)

def create_typetest(klass, lines):
	tt = klass()
	tt.name.value = lines[0][1:].strip()
	tt.description.value = lines[1][1:].strip()
	return tt

def convert_typetest_directional(lines):
	tt = create_typetest(typetests.TypetestDirectional, lines)
	isoclass = ({'Ob': typetests.IsocandelRectangle, 'El': typetests.IsocandelEllipse}[lines[2][:2]])
	for i in xrange(3):
		iso = isoclass()
		iso.intensity.value = float(lines[i*5+3])
		iso.omega_min.value = float(lines[i*5+4])
		iso.omega_max.value = float(lines[i*5+5])
		iso.sigma_min.value = float(lines[i*5+6])
		iso.sigma_max.value = float(lines[i*5+7])
		if iso.omega_min.value != 0 or iso.omega_max.value != 0: # same nuly => nedef. izokandela
			tt.isocandels.insert(0, iso)
	tt.brightness.value = float(lines[18])
	tt.uniformity.value = float(lines[19])
	tt.omega_min.value = float(lines[20])
	tt.sigma_min.value = float(lines[21])
	tt.omega_max.value = float(lines[22])
	tt.sigma_max.value = float(lines[23])
	tt.omega_step.value = float(lines[24])
	tt.sigma_step.value = float(lines[25])
	return tt

def convert_typetest_approach(lines):
	tt = create_typetest(typetests.TypetestApproach, lines)
	isoclass = ({'Ov': typetests.IsocandelOval}[lines[2][:2]])
	for i in xrange(5):
		iso = isoclass()
		iso.white_intensity.value = float(lines[i*5+3])
		iso.red_intensity.value = iso.white_intensity.value / 2.0
		iso.omega_min.value = float(lines[i*5+4])
		iso.omega_max.value = float(lines[i*5+5])
		iso.sigma_min.value = float(lines[i*5+6])
		iso.sigma_max.value = float(lines[i*5+7])
		if iso.omega_min.value != 0 or iso.omega_max.value != 0: # same nuly => nedef. izokandela
			tt.isocandels.insert(0, iso)
	tt.omega_min.value = float(lines[28])
	tt.sigma_min.value = float(lines[29])
	tt.omega_max.value = float(lines[30])
	tt.sigma_max.value = float(lines[31])
	tt.omega_step.value = float(lines[32])
	tt.sigma_step.value = float(lines[33])
	return tt

def convert_typetest_omni(lines):
	tt = create_typetest(typetests.TypetestOmni, lines)
	os, ss = typetests.OmegaSpan(), typetests.SigmaSpan()
	os.sigma.value = 0
	os.omega_min.value = float(lines[2])
	os.omega_max.value = float(lines[3])
	os.omega_step.value = float(lines[4])
	ss.sigma_min.value = float(lines[5])
	ss.sigma_max.value = float(lines[6])
	ss.sigma_step.value = float(lines[7])
	os.intensity.value = float(lines[8])
	ss.intensity.value = float(lines[9])
	if lines[10].find('?') < 0: 
		ss.omega.value = float(lines[10])
		ss.parameter.value = 'value'
	else:
		ss.parameter.value = 'min'
	
	tt.omega_spans.append(os)
	tt.sigma_spans.append(ss)
	return tt

def convert_typetests(infile, outdir, convert_func):
	inf = open(infile, 'r')
	lines = inf.readlines()
	inf.close()
	for idx, line in enumerate(lines):
		if line[0] != '*':
			continue
		tt = convert_func(lines[idx:])
		tt.validate()
		if not tt.validation.valid:
			print tt.name.value
		tt.xml_save(path.join(outdir, tt.name.value+'.xml'))

convert_typetests(r'stare_typovky\definice.tzk', r'stare_typovky\smerova', convert_typetest_directional)
convert_typetests(r'stare_typovky\sestup.tzk', r'stare_typovky\sestupova', convert_typetest_approach)
convert_typetests(r'stare_typovky\vsesmer.tzk', r'stare_typovky\vsesmerova', convert_typetest_omni)
