# -*- encoding: cp1250 -*-

""" prohlizeni chybovych logu """

from glob import glob
import os

from mvc import *
import registry
import error_logging

logs_path = registry.paths.logs

resources = LazyXMLResource('logviewer.xrc')

class LogViewerModel(CompositeModel):
	_models = {
		'filenames': ArrayModel(FilenameModel()),
		'error': error_logging.Error(),
	}

	def refresh_path(self, path):
		fnames = glob(os.path.join(logs_path, '*.xml'))
		fnames.sort()
		self.filenames.copy_from_values(fnames)

# LogViewerDlg ##################################################################

class LogViewerDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_logview', parent)
		m, f = self.model, self.frame

		self.focused_item = IntModel()
		self.focused_item.add_change_observer(self.on_file_changed)
		display_defs = { 'file': ('File', wx.LIST_AUTOSIZE_USEHEADER, list_str()) }
		self.add_view(MulticolArrayView(m.filenames, f, 'List_files', ('file', ), display_defs, self.focused_item))
		self.add_view(StrEditor(m.error.info, f, 'Text_info'))
		self.add_view(StrEditor(m.error.error, f, 'Text_error'))
		self.bind(wx.EVT_BUTTON, self.on_refresh_pressed, 'But_refresh')

	def on_file_changed(self):
		try:
			self.model.error.xml_load(self.model.filenames[self.focused_item.value].value)
		except XMLModelLoadError, e:
			log_error("Chyba pi nahrvn logovho souboru.")

	def on_refresh_pressed(self, evt = None):
		self.model.refresh_path(logs_path)
		self.focused_item.value = len(self.model.filenames) - 1

# LogViewerApp ##############################################################

class LogViewerApp(wx.App):
	def OnInit(self):
		m = LogViewerModel()
		m.refresh_path(logs_path)
		LogViewerDlg.run_dialog(m)
		return True

# main ######################################################################

if __name__ == "__main__":
	app = LogViewerApp(0)
	app.MainLoop()
