#!/usr/bin/env python
# -*- coding: ISO-8859-1 -*-

import wx
from numpy import pi
import matika
#
import pdb

class DrawPanel(wx.Panel):
    def __init__(self, parent, drawFun):
        wx.Panel.__init__(self, parent, -1)
        self.SetBackgroundColour(wx.WHITE)

        self.drawFun = drawFun
        self.Bind(wx.EVT_PAINT, self.OnPaint)

    def OnPaint(self, evt):
        dc = wx.PaintDC(self)
        dc.Clear()
        self.drawFun(dc)


class MyFrame(wx.Frame):
	def __init__(self, parent, id, title, app):
		wx.Frame.__init__(self, parent, id, title, 
			style = wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE,
			size = (360, 360))
		self.app = app
		
		main_sizer = wx.BoxSizer(wx.VERTICAL)
		self.SetSizer(main_sizer)
		main_panel = DrawPanel(self, self.draw_func)
		main_sizer.Add(main_panel, proportion = 1, flag = wx.EXPAND)
		lr_sizer = wx.BoxSizer(wx.HORIZONTAL)
		main_panel.SetSizer(lr_sizer)

		self.SetAutoLayout(True)
		self.Layout()

		self.test_points = {}
		for x in xrange(-78, 78, 5):
			for y in xrange(-78, 78, 5):
				self.test_points[(float(x), float(y))] = None
		self.perform_test()
		
	def draw_func(self, dc):
		for point in self.test_points.iteritems():
			c = point[1]
			if not c: continue
			dc.SetPen(wx.Pen(colour = wx.Colour(c, c, c)))
			dc.DrawPoint(point[0][0] + 90, point[0][1] + 90)

	def perform_test(self):
		m = self.app.table_math
		cnt = 0
		for point in self.test_points.iteritems():
			print "Computing: [%f, %f], %i/%i" % (point[0][0], point[0][1], cnt, len(self.test_points))
			cnt += 1
			try:
				r, t, i = m.marker2table(point[0][0] /180.0 * pi, point[0][1] / 180.0 * pi)
			except:
				continue
			self.test_points[point[0]] = i
	
class MyApp(wx.App):
	def OnInit(self):
		self.table_math = matika.TableMath(f=(0.0, 0.5, 0.5), sensor=(0.0, 1.0, 0.0), look = (0,1,0))
		wx.InitAllImageHandlers()
		myframe = MyFrame(None, -1, "Table View", self)
		self.SetTopWindow(myframe)
		myframe.Show()
		return 1

if __name__ == "__main__":
	my_app = MyApp(0)
	my_app.MainLoop()
