import unittest
import bicubic
import storage

from numpy import array, arange
import misc_num

places = 5 #pro assertAlmostEqual
epsilon = 1.0/(10**places)

class BicubicTestCase(unittest.TestCase):
	def setUp(self):
		pass

	@staticmethod
	def mesh22():
		"""Vytvori a vrati mrizku

		11
		11
		"""
		m = storage.Mesh()
		coords = (0.0, 1.0)
		m.create_from_function(coords, coords, lambda o,s: 1)
		return m
	
	@staticmethod
	def mesh33():
		"""Vytvori a vrati mrizku

		123
		456
		789
		"""
		m = storage.Mesh()
		coords = (-1.0, 0.0, 1.0)
		m.create_from_function(coords, coords, lambda o,s: 3*(s+1) + o+2)
		return m

	@staticmethod
	def mesh44():
		"""Vytvori a vrati mrizku

		1111
		2222
		3333
		4444
		"""
		m = storage.Mesh()
		coords = (-1.0, 0.0, 1.0, 2.0)
		m.create_from_function(coords, coords, lambda o, s: s + 2)
		return m

	@staticmethod
	def mesh55():
		"""Vytvori a vrati mrizku

		01234
		12345
		23456
		34567
		45678
		"""
		m = storage.Mesh()
		coords = (0.0, 1.0, 2.0, 3.0, 4.0)
		m.create_from_function(coords, coords, lambda o, s: o + s)
		return m

	@staticmethod
	def mesh55_random():
		import random
		m = storage.Mesh()
		coords = (0.0, 1.0, 2.0, 3.0, 4.0)
		m.create_from_function(coords, coords, lambda o, s: random.random())
		return m

	"""
	@staticmethod
	def mesh55_random():
		import random
		m = storage.Mesh()
		coords = (0.0, 1.0, 2.0, 3.0, 4.0)
		m.mesh = array(((1,5,2,3,4), (1,4,6,3,5), (1,6,4,2,3), (5,4,6,2,1), (4,3,6,1,5)), 'f')
		m.rows = m.cols = coords
		return m
	"""

	def assertMeshEqual(self, m1, m2):
		for v1, v2 in zip(m1, m2):
			self.assertVecEqual(v1, v2)

	def assertVecEqual(self, v1, v2):
		"""Vraci true pokud jsou dva vektory (1d pole) stejne"""
		self.assertTrue(misc_num.vec_equals(v1,v2,epsilon))
	
	def test_interpolate_mesh(self):
		"""test na BicubicMesh.interpolate_mesh"""
		#Test na identitu
		def test(m, rows, cols, expected_m):
			b = bicubic.BicubicMesh(m.mesh.value, m.cols.value, m.rows.value)
			i_m = b.interpolate_mesh(cols.value, rows.value)
			self.assertMeshEqual(expected_m.value, i_m)

		m = self.mesh22(); test(m, m.rows, m.cols, m.mesh)
		m = self.mesh33(); test(m, m.rows, m.cols, m.mesh)
		m = self.mesh44(); test(m, m.rows, m.cols, m.mesh)
		m = self.mesh55(); test(m, m.rows, m.cols, m.mesh)
		m = self.mesh55_random(); test(m, m.rows, m.cols, m.mesh)

		#Test monotonosti pri monotonim vstupu
		def test(m, rows, cols, expected_m):
			b = bicubic.BicubicMesh(m.mesh.value, m.cols.value, m.rows.value)
			i_m = b.do_interpolate_mesh(cols, rows)
			for ridx in xrange(len(rows) - 1):
				for cidx in xrange(len(cols) - 1):
					self.assertTrue(i_m[ridx][cidx] < i_m[ridx+1][cidx])
					self.assertTrue(i_m[ridx][cidx] < i_m[ridx][cidx+1])

		coords = arange(0, 5, 0.1)
		m = self.mesh55(); test(m, coords, coords, m.mesh)

		coords = arange(0, 5, 2)
		m = self.mesh55(); test(m, coords, coords, m.mesh)

def suite():
	return unittest.makeSuite(BicubicTestCase)

if __name__ == '__main__':
	unittest.main()
