# -*- encoding: cp1250 -*-
"""Cache, v podstate slovnik s omezenym poctem polozek"""

def Cache(limit = None):
	"""Inicializace cache

	limit: max. pocet prvku v cache, prvni se odebiraji nejstarsi
	"""
	if not limit:
		return CacheWithoutLimit()
	else:
		return CacheWithLimit(limit)

class CacheWithoutLimit(dict):
	def get(self, key):
		return self[key]

	def add(self, key, val):
		self[key] = val

class CacheWithLimit(dict):
	"""Cache s omezenim na max. pocet prvku"""

	def __init__(self, limit):
		"""Inicializace cache

		limit: max. pocet prvku v cache, prvni se odebiraji nejstarsi
		"""
		self.items_order = []
		self.limit = limit
		
	def get(self, key):
		"""Vrati prvek s klicem key.

		pokud prvek v cache neni, hazi KeyError
		"""
		return dict.__getitem__(self, key)

	def __setitem__(self, key, val):
		if key in self:
			return
		dict.__setitem__(self, key, val)
		self.items_order.append(key)
		if (not self.limit) or len(self) <= self.limit:
			return
		last_key = self.items_order.pop(0) 
		dict.pop(self, last_key)

	def add(self, key, val):
		"""Prida do cache polozku s klicem key a hodnotou val
		"""
		self.__setitem__(key, val)
