# -*- encoding: cp1250 -*-
"""GUI ke kalibraci pristroju - nulove polohy, cidla luxmetru, ..."""

import wx

from mvc import *
import calibration
import registry
import ckdm_interface
from ckdm_interface import interface, devices, CKDM_Error
import debug
import time

resources = registry.xresources.calibration

#############################################################################
# Kalibrace stolu ###########################################################
#############################################################################

# SensorDlg ##############################################################

class SensorDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_Sensor', parent)

		m, f = self.model, self.frame
		self.add_view(StrEditor(m.description, f, 'Text_description'))
		self.add_view(FloatEditor(m.x, f, 'Text_x', '%.3f'))
		self.add_view(FloatEditor(m.y, f, 'Text_y', '%.3f'))
		self.add_view(FloatEditor(m.z, f, 'Text_z', '%.3f'))

# TableZeroDlg #############################################################

class TableZeroDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_TableZero', parent)

		f = self.frame
		self.add_view(FloatView(devices.rot, f, 'Text_rot_zero', '%.3f'))
		self.add_view(FloatView(devices.tilt, f, 'Text_tilt_zero', '%.3f'))
		self.bind(wx.EVT_BUTTON, self.on_rdcoords, 'But_rdcoords')

	def on_run(self):
		try:
			interface.connect()
			interface.table_reference()
			interface.table_manual_enable()
		except CKDM_Error, e:
                        e.wx_log_error()
			return False

	def on_destroy(self):
		try:
			interface.table_manual_disable()
		except CKDM_Error, e:
                        e.wx_log_error()

	def pre_ok(self):
		try:
			interface.table_set_zero(0, 0)
			interface.table_get_zero()
			self.model.rot_zero.copy_from(devices.rot_zero)
			self.model.tilt_zero.copy_from(devices.tilt_zero)
		except CKDM_Error, e:
                        e.wx_log_error()
			return False

	def on_rdcoords(self, evt = None):
		try:
			interface.table_get_pos()
			interface.table_manual_enable()
		except CKDM_Error, e:
                        e.wx_log_error()

# TableCalibrationFrame #########################################################
			
class TableCalibrationFrame(DocumentFrame):
	def __init__(self, model, parent = None):
		DocumentFrame.__init__(self, model, resources, 'Frame_CalibrationTable', parent)

		m, f = self.model, self.frame
		display_defs = {
			'x': ('x [m]', 50, list_text('%.3f', 'x')),
			'y': ('y [m]', 50, list_text('%.3f', 'y')),
			'z': ('z [m]', 50, list_text('%.3f', 'z')),
			'description': ("Popis", 50, list_text('%s', 'description')),
		}
		self.add_view(
			MulticolArrayEditor(m.sensors, f, 'List_sensors', 
			None, ('x', 'y', 'z', 'description'), display_defs, 
			'But_up', 'But_down', 'But_add', 'But_del', 'But_edit', SensorDlg.run_dialog))

		self.add_view(FloatView(m.rot_zero, f, 'Text_rot_zero', '%.3f'))
		self.add_view(FloatView(m.tilt_zero, f, 'Text_tilt_zero', '%.3f'))

		self.set_menu('MB_CalibrationTable')
		self.bind_file_events(None, 'MI_Open', 'MI_Save', 'MI_SaveAs', 'MI_Quit')
		self.setup_file_dialogs(registry.paths.config, registry.fnames.calibration_table, 
			"Otevt soubor s kalibrac stolu", "Uloit soubor s kalibrac stolu", 
			registry.wildcards.xml)
		self.bind(wx.EVT_BUTTON, self.on_but_zero, 'But_zero')

	def on_but_zero(self, evt):
		if ckdm_interface.version == 'simul':
			TableZeroDlg.run_dialog_modeless(self.model, self.frame)
		else:
			TableZeroDlg.run_dialog(self.model, self.frame)

##############################################################################
# Kalibrace ramene ###########################################################
##############################################################################

# ArmZeroDlg #############################################################

class ArmZeroDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_ArmZero', parent)
		self.add_view(FloatView(devices.arm, self.frame, 'Text_rot_zero', '%.3f'))
		self.bind(wx.EVT_BUTTON, self.on_rdcoords, 'But_rdcoords')
		
	def on_run(self):
		try:
			interface.connect()
			interface.arm_reference()
			interface.arm_manual_enable()
		except CKDM_Error, e:
                        e.wx_log_error()
			return False

	def on_destroy(self):
		try:
			interface.arm_manual_disable()
		except CKDM_Error, e:
                        e.wx_log_error()

	def pre_ok(self):
		try:
			interface.arm_get_zero()
			interface.arm_get_pos()
			zero = devices.arm.value + devices.arm_zero.value
			interface.arm_set_zero(zero)
			self.model.arm_zero.value = zero
		except CKDM_Error, e:
                        e.wx_log_error()
			return False

	def on_rdcoords(self, evt = None):
		try:
			interface.arm_get_pos()
			interface.arm_manual_enable()
		except CKDM_Error, e:
                        e.wx_log_error()

# ArmCalibrationFrame #####################################################

class ArmCalibrationFrame(DocumentFrame):
	def __init__(self, model, parent = None):
		DocumentFrame.__init__(self, model, resources, 'Frame_CalibrationArm', parent)

		m, f = self.model, self.frame

		self.add_view(FloatView(m.arm_zero, f, 'Text_rot_zero', '%.3f'))
		self.add_view(FloatEditor(m.sensor_distance, f, 'Text_sensor_distance', '%.1f'))

		self.set_menu('MB_CalibrationArm')
		self.bind_file_events(None, 'MI_Open', 'MI_Save', 'MI_SaveAs', 'MI_Quit')
		self.setup_file_dialogs(registry.paths.config, registry.fnames.calibration_arm, 
			"Otevt soubor s kalibrac ramene", "Uloit soubor s kalibrac ramene", 
			registry.wildcards.xml)
		self.bind(wx.EVT_BUTTON, self.on_but_zero, 'But_zero')

	def on_but_zero(self, evt):
		if ckdm_interface.version == 'simul':
			ArmZeroDlg.run_dialog_modeless(self.model, self.frame)
		else:
			ArmZeroDlg.run_dialog(self.model, self.frame)

##############################################################################
# Nastaveni luxmetru #########################################################
##############################################################################

# LuxmetersFrame ############################################################

class LuxmetersFrame(DocumentFrame):
	def __init__(self, model, parent = None):
		DocumentFrame.__init__(self, model, resources, 'Frame_Luxmeters', parent)

		m, f = self.model, self.frame

		display_defs = {
			'descr': ('Popis luxmetru', 50, lambda li, m: li.SetText(m.value)),
		}
		self.add_view(
			MulticolArrayEditor(m.luxmeters, f, 'List_luxmeters', 
			None, ('descr',), display_defs, 
			'But_up', 'But_down', 'But_add', 'But_del', 'But_edit', self.edit_lux))

		self.set_menu('MB_Luxmeters')
		self.bind_file_events(None, 'MI_Open', 'MI_Save', 'MI_SaveAs', 'MI_Quit')
		self.setup_file_dialogs(registry.paths.config, registry.fnames.luxmeters, 
			"Otevt soubor s nastavenm luxmetr", "Uloit soubor s nastavenm luxmetr", 
			registry.wildcards.xml)

	def edit_lux(self, model, parent):
		return TextEntryDialog(model, 'Popis luxmetru', 'Zadejte popis luxmetru',
		                       parent = parent).run_modal()

##############################################################################
# Externi fce ################################################################
##############################################################################

def _gen_gui_func(model, klass, frame):
	def gui_func(parent):
		m = getattr(registry, model).value
		if not m:
			m = getattr(calibration, klass)()
		frame(m, parent.frame).run()
	return gui_func

gui_calibration_table = _gen_gui_func('calibration_table', 'CalibrationTable', TableCalibrationFrame)
gui_calibration_arm = _gen_gui_func('calibration_arm', 'CalibrationArm', ArmCalibrationFrame)
gui_luxmeters = _gen_gui_func('luxmeters', 'Luxmeters', LuxmetersFrame)
