# -*- encoding: cp1250 -*-

from mvc import *
import registry
from misc_num import round_to

# zaokrouhlovani ############################################################

def round_rot(rot):
	"Zaokrouhli hodnotu natoceni stolu na platne cislice"
	return round_to(rot, 0.005)

def round_tilt(tilt):
	"Zaokrouhli hodnotu naklopeni stolu na platne cislice"
	return round_to(tilt, 0.005)

def round_arm(arm):
	"Zaokrouhli hodnotu natoceni ramene na platne cislice"
	return round_to(arm, 0.05)

# raise_error ###############################################################

def raise_error(*args):
	log_event('error', '\n'.join(args))
	e = CKDM_Error('\n'.join(args))
	# e.wx_log_error() #Debug
	raise e

# log_event #################################################################

def log_event(type, msg):
	for err_msg in msg.split('\n'):
		le = LogEvent()
		le.time.set_now()
		le.type.value = type
		le.msg.value = err_msg
		devices.log.append(le)
	
# LogEvent ##################################################################

class LogEvent(CompositeModel, Exception):
	"""Zaznam v deniku udalosti"""
	_models = {
		'time': TimeModel(),
		'type': StrModel(),
		'msg': StrModel(),
	}

# CKDM_Error ################################################################

class CKDM_Error(Exception):
	def __init__(self, msg):
		Exception.__init__(self)
		self.msg = msg

	def wx_log_error(self):
		for err_msg in self.msg.split('\n'):
			log_error(err_msg)

	def __str__(self):
		return self.msg

# DevicesModel ##############################################################

class DevicesModel(CompositeModel):
	"""Popisuje aktualni stav pripojenych zarizeni.

	Ma jedinou instanci - devices. Pres tuto instanci by se melo k zarizenim
	pristupovat. Zajimave atributy jsou read-only, pri provedeni nekterych
	funkci CKDM_Interface se do teto instance promitnou pripadne zmeny.
	"""
	_models = {
		'ckdm_id': StrModel(),

		'table_id': StrModel(),
		'table_initialized': BoolModel(False),
		'table_referenced': BoolModel(False),
		'rot': FloatModel(0.0),
		'tilt': FloatModel(0.0),
		'rot_zero': FloatModel(0.0),
		'tilt_zero': FloatModel(0.0),
		'table_manual_enable': BoolModel(False),

		'arm_id': StrModel(),
		'arm': FloatModel(0.0),
		'arm_zero': FloatModel(0.0),
		'arm_manual_enable': BoolModel(False),

		'luxmeter_id': StrModel(),
		'luxmeter_initialized': BoolModel(False),
		'luxmeter': FloatModel(0.0),
		'luxmeter_manual_enable': BoolModel(False),

		'log': ArrayModel(LogEvent()),
	}

devices = DevicesModel()
