# -*- encoding: cp1250 -*-

import wx

from mvc import *
from registry import xresources
from ckdm_interface import devices, interface
from misc_num import sin, cos, sqrt

resources = xresources.ckdm_interface

# DevicesFrame ##############################################################

class DevicesFrame(Frame):
	def __init__(self, model, parent = None):
		Frame.__init__(self, model, resources, 'Frame_Devices', parent, statusbar = False)

		m, f = self.model, self.frame

		# ckdm
		self.add_view(StaticTextView(m.ckdm_id, f, 'SText_ckdm_id'))

		# stul
		self.add_view(StaticTextView(m.table_id, f, 'SText_table_id'))
		self.add_view(StaticTextBoolView(m.table_manual_enable, f, 
			'SText_table_manual_enabled', "povoleno", "zakzno"))
		self.add_view(FloatView(m.rot, f, 'Text_rot'))
		self.add_view(FloatView(m.rot_zero, f, 'Text_rot_zero'))
		self.add_view(FloatView(m.tilt, f, 'Text_tilt'))
		self.add_view(FloatView(m.tilt_zero, f, 'Text_tilt_zero'))

		# rameno
		self.add_view(StaticTextView(m.arm_id, f, 'SText_arm_id'))
		self.add_view(StaticTextBoolView(m.arm_manual_enable, f, 
			'SText_arm_manual_enabled', "povoleno", "zakzno"))
		self.add_view(FloatView(m.arm, f, 'Text_arm'))
		self.add_view(FloatView(m.arm_zero, f, 'Text_arm_zero'))

		# luxmetr
		self.add_view(StaticTextView(m.luxmeter_id, f, 'SText_luxmeter_id'))
		self.add_view(FloatView(m.luxmeter, f, 'Text_luxmeter'))

		# denik
		display_defs = {
			'time': ('as', 100, list_time('time')),
			'type': ('typ', 80, list_text('%s', 'type')),
			'msg': ('zprva', 50, list_text('%s', 'msg')),
		}
		log_view = MulticolArrayView(m.log, f, 'List_log', ('time', 'type', 'msg'), display_defs)
		self.add_view(log_view)

		# vsechna tlacitka

		rot_step = 0.005
		tilt_step = 0.005
		arm_step = 0.05

		self.buttons_defs = {
				'But_rot_incstep':   (rot_step,  0.0,   0.0),
				'But_rot_incdeg':    (1.0,   0.0,   0.0),
				'But_rot_decstep':   (-rot_step, 0.0,   0.0),
				'But_rot_decdeg':    (-1.0,  0.0,   0.0),
				'But_tilt_incstep':  (0.0,  tilt_step,  0.0),
				'But_tilt_incdeg':   (0.0,  1.0,   0.0),
				'But_tilt_decstep':  (0.0,  -tilt_step, 0.0),
				'But_tilt_decdeg':   (0.0,  -1.0,  0.0),
				'But_arm_incstep': (0.0,  0.0,  arm_step),
				'But_arm_incdeg':  (0.0,  0.0,  1.0),
				'But_arm_decstep': (0.0,  0.0,  -arm_step),
				'But_arm_decdeg':  (0.0,  0.0,  -1.0),
			}
		for but_id in self.buttons_defs.iterkeys():
			self.bind(wx.EVT_BUTTON, self.on_but_move, but_id)
		self.bind(wx.EVT_BUTTON, self.close, 'But_Close')

		devices.table_manual_enable.add_change_observer(self.on_device_enable_changed)
		devices.arm_manual_enable.add_change_observer(self.on_device_enable_changed)

		self.on_device_enable_changed()

	def on_destroy(self):
		devices.table_manual_enable.del_change_observer(self.on_device_enable_changed)
		devices.arm_manual_enable.del_change_observer(self.on_device_enable_changed)

	def on_device_enable_changed(self):
		table_manual_enable = devices.table_manual_enable.value
		arm_manual_enable = devices.arm_manual_enable.value

		for but_name in self.buttons_defs.iterkeys():
			button = self.xrcctrl(but_name)
			if but_name.startswith('But_arm'):
				button.Enable(arm_manual_enable)
			else:
				button.Enable(table_manual_enable)

	def on_but_move(self, evt):
		id = evt.GetId()
		for b in self.buttons_defs.iterkeys():
			if self.xrcid(b) == id:
				movement = self.buttons_defs[b]
				break
		else:
			raise KeyError('Nenalezeno prislusne tlacitko')

		m = self.model
		if movement[2] == 0.0:
			interface.table_goto(m.rot.value + movement[0], m.tilt.value + movement[1])	
			interface.table_manual_enable()
		else:
			interface.arm_goto(m.arm.value + movement[2])	
			interface.arm_manual_enable()

def gui_devices(parent):
	DevicesFrame(devices, parent.frame).run()

