# -*- encoding: cp1250 -*-
import time

import wx
from numpy import sin, cos, sqrt

from misc import Timer, Counter
from ckdm_interface_common import devices, raise_error, log_event, CKDM_Error
from mvc import Dialog, StrEditor, StrModel
from registry import xresources

lux_func = lambda r,t,arm: 0.0

# FuncDlg ###################################################################

epsilon = 0.001

def clamp(x):
	if x > 0:
		return x
	return 0

def sgn(x):
	if x >= -epsilon:
		return 1
	return 0

def text2func(text):
	try:
		func = eval('lambda x,y,r: ' + text)
		func(0.0, 0.0, 0.0)
		func(-1.0, -1.0, -1.0)
		func(1.0, 1.0, 1.0)
		return func
	except StandardError, e:
		log_error(str(e))
		log_error(u"Chybn definovan funkce")
		return None

def func_validator(text):
	if text2func(text) == None:
		return u"Nen platn funkce."

class FuncDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, xresources.ckdm_interface, 'Dlg_LuxFunc', parent)
		self.add_view(StrEditor(self.model, self.frame, 'Text_func'))

def get_lux_func():
	func_text = StrModel('clamp(1000*(1 - sqrt( (x/100)**2 + (y/100)**2) ))', func_validator)
	if wx.ID_OK != FuncDlg.run_dialog(func_text):
		return lambda x,y, r: abs(x+y+r)
	return text2func(func_text.value)

# CKDM_Interface ############################################################

class CKDM_Interface(object):
	def __init__(self):
		self.timer = Timer()
		self.connected_cnt = Counter()
		self.connected = False

	def measure_prepare(self):
		self.func = get_lux_func()

	# Connect/Disconnect ######################################

	def connect(self, port="COM1"):
		log_event('notice', 'connect %s' % port)
		if self.connected_cnt.is_zero():
			self.connected = True
		self.connected_cnt.inc()

	def disconnect(self):
		log_event('notice', 'disconnect')
		self.connected_cnt.dec()

	# Console ######################################
		
	def ckdm_id(self):
		log_event('notice', 'ckdm_id')
		devices.ckdm_id.value = u"ckdm simulovana verze"

	# Stul #########################################

	def table_id(self):
		log_event('notice', 'table_id')
		devices.table_manual_enable.value = False
		devices.table_id.value = u"stul simulovana verze"

	def table_manual_enable(self): 
		log_event('notice', 'table_manual_enable')
		devices.table_manual_enable.value = True

	def table_manual_disable(self): 
		log_event('notice', 'table_manual_disable')
		devices.table_manual_enable.value = False

	def table_goto(self, rot, tilt):
		log_event('notice', 'table_goto %f, %f' % (rot, tilt))
		devices.table_manual_enable.value = False
		import time, random
		time.sleep(random.random()/8.0)
		devices.rot.value, devices.tilt.value = rot, tilt

	def table_reference(self):
		log_event('notice', 'table_reference')
		devices.table_manual_enable.value = False
		devices.rot.value, devices.tilt.value = 0.0, 0.0

	def table_get_pos(self):
		log_event('notice', 'table_get_pos')
		devices.table_manual_enable.value = False

	def table_get_zero(self):
		log_event('notice', 'table_get_zero')
		devices.table_manual_enable.value = False

	def table_set_zero(self, r, t):
		log_event('notice', 'table_set_zero %f, %f' % (r, t))
		devices.table_manual_enable.value = False
		devices.rot_zero.value, devices.tilt_zero.value = r, t
		devices.rot.value = devices.rot.value - devices.rot.value
		devices.tilt.value = devices.tilt.value - devices.tilt.value

	def table_invert(self, invert):
		log_event('notice', 'table_invert %s' % str(invert))
		devices.table_manual_enable.value = False

	# Rameno #########################################

	def arm_id(self):
		log_event('notice', 'arm_id')
		devices.arm_manual_enable.value = False
		devices.arm_id.value = u"rameno simulovana verze"

	def arm_manual_enable(self):
		log_event('notice', 'arm_manual_enable')
		devices.arm_manual_enable.value = True

	def arm_manual_disable(self):
		log_event('notice', 'arm_manual_disable')
		devices.arm_manual_enable.value = False

	def arm_goto(self, pos):
		log_event('notice', 'arm_goto %f' % pos)
		import time, random
		time.sleep(random.random()/8.0)
		devices.arm_manual_enable.value = False
		devices.arm.value = pos

	def arm_reference(self):
		log_event('notice', 'arm_reference')
		devices.arm_manual_enable.value = False
		devices.arm.value = 0.0

	def arm_get_pos(self):
		log_event('notice', 'arm_get_pos')
		devices.arm_manual_enable.value = False

	def arm_get_zero(self):
		log_event('notice', 'arm_get_zero')
		devices.arm_manual_enable.value = False

	def arm_set_zero(self, zero):
		log_event('notice', 'arm_set_zero %f' % zero)
		devices.arm_manual_enable.value = False
		devices.arm_zero.value = zero
		devices.arm.value = devices.arm.value - devices.arm_zero.value

	# Luxmetr #####################################

	def lux_init(self):
		log_event('notice', 'lux_init')
	
	def lux_get_val(self):
		log_event('notice', 'lux_get_val')
		rot, tilt, arm = devices.rot.value, devices.tilt.value, devices.arm.value
		devices.luxmeter.value = self.func(float(rot), float(tilt), float(arm))

	def lux_manual_enable(self):
		log_event('notice', 'lux_manual_enable')
		devices.luxmeter_manual_enable.value = True
	  
	def lux_manual_disable(self):
		log_event('notice', 'lux_manual_disable')
		devices.luxmeter_manual_enable.value = False
