# -*- encoding: cp1250 -*-
"""Individualni vyvoj - modely a algoritmy"""

from mvc import *
import storage
import registry
import eval
import typetests
import eval_typetest
from misc import SubclassResponsibilityError
import plot
import gfm_common_gui
import re

import debug

##############################################################################
# Obecny vyvoj ###############################################################
##############################################################################

# Devel #####################################################################

class Devel(CompositeModel):
	"""Predek vsech trid individualniho vyvoje"""
	_models = {
		'meas_data': storage.MeasData(),
	}

	default_meas_type = None

	def __init__(self):
		CompositeModel.__init__(self)
		self.report = FuncAdaptor(self.html_report)
		self.plot_views = []

	def html_header(self):
		return u"""
			<meta HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=cp1250">
			<p><font size="+1"><b>%s</b></font><br>
			<p>
			""" % self.get_protocol_title()

	def html_footer(self):
		return ''

	def html_report(self):
		return ''

	def ascii_report(self, include_data = True, include_translated_data = True):
		return self.meas_data.ascii_report(include_data, include_translated_data)

	def get_protocol_path(self):
		return self.path_devel

	def str_eval_timestamp(self):
		return self.eval_result.eval_timestamp.as_str()

	def str_eval_date(self):
		return self.eval_result.eval_timestamp.str_date()

#############################################################################
# Izokandelovy vyvoj ########################################################
#############################################################################

# DevelIso ##################################################################

class DevelIso(Devel):
	"""Vyvoj izokandelovych navestidel"""
	default_meas_type = storage.MeasInfo.NOWEDGE
	_models = {
		'plot': plot.PlotModel(),
		'text_report': StrModel(),
		'def_isocandels': eval.def_isocandels_templates,
		'groups': plot.PlottableGroups(),
		'next_groupname': IntModel(),
	}
	_models.update(Devel._models)
	_models.update(typetests.TypetestIso.models_iso_range)

	def _get_mesh(self):
		return self.plot.mesh
	mesh = property(_get_mesh)

	def set_plot_mark(self):
		self.plot_mark = len(self.plot.plottables)

	def get_marked_plottables(self):
		return self.plot.plottables[self.plot_mark:]

	def group_marked_plottables(self, groupname = None):
		if not groupname:
			groupname = u'%i' % self.next_groupname.value
			self.next_groupname.value = self.next_groupname.value + 1
		self.groups.add_plottables_to_group(self.get_marked_plottables(), groupname)
		return groupname

	def delete_group(self, group):
		self.plot.delete_plottables_with_ids([member.value for member in group.members])
		idx_to_del = None
		groupname = group.name.value
		for idx, g in enumerate(self.groups):
			if g.name.value == groupname:
				idx_to_del = idx
				break
		if idx_to_del != None:
			self.groups.pop(idx_to_del)

	def delete_group_by_name(self, groupname):
		group = self.groups.find_group(groupname)
		if group == None:
			return
		self.delete_group(group)

	def delete_isocandel_by_plottable(self, plottable):
		group = self.groups.find_group_by_plottable(plottable)
		if group == None:
			return
		self.delete_isocandel_by_group_name(group.name.value)

	def delete_isocandel_by_group_name(self, groupname):
		idx = self.find_def_isocandel_idx(groupname)
		if idx != None:
			self.def_isocandels.pop(idx)
		self.delete_group_by_name(groupname)

	def find_def_isocandel_idx(self, name):
		for idx, i in enumerate(self.def_isocandels):
			if i.group.value == name:
				return idx

	def find_def_isocandel_by_plottable(self, plottable):
		group = self.groups.find_group_by_plottable(plottable)
		if group == None:
			return
		return self.find_def_isocandel(group.name.value)

	def find_def_isocandel(self, name):
		idx = self.find_def_isocandel_idx(name)
		if idx == None:
			return None
		return self.def_isocandels[idx]

	def validator(self):
		return typetests.validate_omega_step(self) and typetests.validate_sigma_step(self)

	def prepare_plot(self):
		gfm_common_gui.prepare_isoplot(self.plot, self.get_plot_title())

	def update_mesh_from_meas_data(self):
		try:
			self.mesh.read_from_meas_data(self.meas_data)
		except storage.MeshError:
			log_error(u"Nepodailo se vytvoit mku z namench dat.")
		self.eval_result.eval_timestamp.set_now()
		self.prepare_plot()

	def get_plot_title(self):
		return u'%s, %s' % (self.get_title(), self.str_eval_date())

	def get_protocol_title(self):
		return u'%s, %s' % (self.get_title(), self.str_eval_timestamp())

	def get_title(self):
		raise misc.SubclassResponsibilityError

	_re_br = re.compile(r'$', re.MULTILINE)
	_re_para = re.compile(r'^$', re.MULTILINE)
	@classmethod
	def text2html(cls, txt):
		t = cls._re_br.sub('<br>', txt)
		t = cls._re_para.sub('<p>', t)
		return t

	def html_report(self):
		return '\n'.join((
			self.html_header(),
			u"<h3>%s</h3><br>" % self.get_title(),
			self.text2html(self.text_report.value),
			))

	def eval_def_isocandel(self, typetest_isocandel, abbr, color):
		def_isocandel = eval.EvalIso.get_def_isocandel(typetest_isocandel)()
		try:
			def_isocandel.eval(self.mesh, typetest_isocandel, abbr, abbr)
		except eval.EvalError:
			log_error(u"Chyba pi vyhodnocovn izokandely")
			return
		self.set_plot_mark()
		self.plot.freeze_changes()
		try:
			def_isocandel.add_to_plot(self.plot, color = color)
			# cary (coz je podmnozina vseho :-) udelam pohyblive
			for p in self.get_marked_plottables():
				p.movable.value = True
			groupname = self.group_marked_plottables()
		finally:
			self.plot.thaw_changes()
		def_isocandel.group.value = groupname
		def_isocandel.color.copy_from(color)
		self.def_isocandels.append(def_isocandel)
		return def_isocandel.html_report()

	def move_isocandel(self, def_isocandel, offset):
		typetest_isocandel = def_isocandel.isocandel.copy()
		typetest_isocandel.move(offset)
		report = self.eval_def_isocandel(typetest_isocandel, def_isocandel.abbr.value, def_isocandel.color)
		if report == None: # Vyhodnoceni se nezdarilo
			return
		# Podarilo se, smazeme starou izokandelu
		self.delete_isocandel_by_group_name(def_isocandel.group.value)
		return report

# DevelDirectional ##########################################################

class DevelDirectional(DevelIso):
	"""Vyvoj smerovych navestidel"""
	frame_title = u"Vvoj smrovch nvstidel"
	_models = {
		'eval_result': eval.EvalResultDirectional()
	}
	_models.update(DevelIso._models)
	path_devel = registry.paths.devel_directional
	path_data = registry.paths.typetests_data_directional
	path_typetest = registry.paths.typetests_def_directional
	path_eval = registry.paths.typetests_eval_directional
	eval_typetest_result_class = eval_typetest.EvalTypetestResultDirectional
	typetest_class = typetests.TypetestDirectional

	def	eval_meas_isocandel(self, intensity, abbr, color):
		emi = eval.EvaluatedMeasIsocandelDirectional()
		try:
			emi.eval(self.mesh, intensity, abbr)
		except eval.EvalError:
			log_error(u"Chyba pi vyhodnocovn pidvan izokandely")
			return
		self.set_plot_mark()
		emi.add_to_plot(self.plot, color = color)
		self.group_marked_plottables()

	def eval_brightness(self, def_isocandel):
		brightness = FloatModel()
		try:
			eval.EvalDirectional.eval_main_isocandel(None, None, brightness, self.mesh, def_isocandel.isocandel)
		except eval.EvalError:
			log_error(u"Chyba pi vyhodnocovn svtivosti")
			return
		return u"Stedn svtivost hlavnho svazku: %.1f\n" % brightness.value

	def eval_minmax(self, def_isocandel):
		min, max = storage.MeasuredPoint(), storage.MeasuredPoint()
		try:
			eval.EvalDirectional.eval_main_isocandel(min, max, None, self.mesh, def_isocandel.isocandel)
		except eval.EvalError:
			log_error(u"Chyba pi vyhodnocovn minima a maxima.")
			return
		self.set_plot_mark()
		eval.EvalResultDirectional.prepare_plot_extremes(min, max, self.plot)
		self.group_marked_plottables(def_isocandel.group.value)
		extremes_report = eval.EvalResultDirectional.html_report_main_extremes(min, max)
		try:
			uniformity_report = u"Pomr min/max: %.3f" % float(max.intensity/min.intensity)
		except ZeroDivisionError:
			uniformity_report = u"Pomr min/max: nedefinovan"
		return '\n'.join((extremes_report, uniformity_report))

# DevelApproach #############################################################

class DevelApproach(DevelIso):
	"""Vyvoj sestupovych navestidel"""
	frame_title = u"Vvoj sestupovch nvstidel"
	_models = {
		'eval_result': eval.EvalResultApproach()
	}
	_models.update(DevelIso._models)
	path_devel = registry.paths.devel_approach
	path_data = registry.paths.typetests_data_approach
	path_typetest = registry.paths.typetests_def_approach
	path_eval = registry.paths.typetests_eval_approach
	eval_typetest_result_class = eval_typetest.EvalTypetestResultApproach
	typetest_class = typetests.TypetestApproach

	def	eval_meas_isocandel(self, white_intensity, red_intensity, abbr, color):
		emi = eval.EvaluatedMeasIsocandelApproach()
		try:
			emi.eval(self.mesh, white_intensity, red_intensity, abbr)
		except eval.EvalError:
			log_error(u"Chyba pi vyhodnocovn pidvan izokandely")
			return
		self.set_plot_mark()
		emi.add_to_plot(self.plot, color = color)
		self.group_marked_plottables()

	def eval_symmetry(self, sym_min, sym_max):
		result = eval.EvalResultApproach()
		try:
			eval.EvalApproach.eval_symmetry(result, self.mesh, sym_min, sym_max)
		except eval.EvalError:
			log_error(u"Chyba pi vyhodnocovn symetrie.")
			return
		self.plot.freeze_changes()
		try:
			self.delete_group_by_name('bsp');
			self.set_plot_mark()
			result.add_bsp_to_plot(self.plot)
			self.group_marked_plottables('bsp')
		finally:
			self.plot.thaw_changes()
		return result.html_report_symmetry()

#############################################################################
# Tabulkovy vyvoj ###########################################################
#############################################################################

# DevelTable ################################################################

class DevelTable(Devel):
	_models = {
		'eval_result': eval.EvalResultTable()
	}
	_models.update(DevelIso._models)

	path_devel = registry.paths.devel_table
	default_meas_type = storage.MeasInfo.NOWEDGE
	typetest_class = typetests.TypetestTable

	def html_report(self):
		return '\n'.join((
			self.html_header(),
			u"<font size=\"+2\">Odmen body</font><p>",
			u'<br>'.join(p.as_html() for p in self.eval_result.evaluated_points),
			self.html_footer(),
			))

	def get_protocol_title(self):
		return u"Vvoj bodovho men, %s" % self.str_eval_timestamp()

##############################################################################
# Polarni vyvoj ##############################################################
##############################################################################

class DevelPolar(Devel):
	_models = {
		'plot_omega_spans': plot.PlotModel(), # graf pro konst. sigma
		'plot_sigma_spans': plot.PlotModel(), # graf pro konst. omega
	}
	_models.update(DevelIso._models)

	def __init__(self):
		Devel.__init__(self)
		self.plot_views.append(self.plot_omega_spans)
		self.plot_views.append(self.plot_sigma_spans)

	def html_report(self):
		return '\n'.join((
			self.html_header(), 
			u"<font size=\"+2\">Odmen rozsahy</font><p>",
			self.eval_result.html_report(),
			self.html_footer()
		))

# DevelLight ################################################################

class DevelOmni(DevelPolar):
	_models = {
		'eval_result': eval.EvalResultOmni()
	}
	_models.update(DevelPolar._models)
	path_devel = registry.paths.devel_omni
	default_meas_type = storage.MeasInfo.WEDGE
	typetest_class = typetests.TypetestOmni

	def get_protocol_title(self):
		return u"Vvoj vesmrovch nvstidel, %s" % self.str_eval_timestamp()

	
# DevelLight ################################################################

class DevelLight(DevelPolar):
	_models = {
		'eval_result': eval.EvalResultLight()
	}
	_models.update(DevelPolar._models)
	path_devel = registry.paths.devel_light
	default_meas_type = storage.MeasInfo.ARM
	typetest_class = typetests.TypetestLight

	def get_protocol_title(self):
		return u"Vvoj svtidel, %s" % self.str_eval_timestamp()
