# -*- encoding: cp1250 -*-

"""GUI individualniho vyvoje"""

import re

from mvc import *
import registry
import devel
from ckdm_interface import version as ckdm_interface_version
from viewdata_gui import ExportPrintMixin, LuminogramDlg
import typetests
import typetests_gui
import eval
import measure
import measure_gui
import plot
import misc
import numpy
from gfm_common_gui import OmegaSigmaEditor, IntensityEditor
from misc_num import round_to
import storage
import debug

resources = registry.xresources.devel


##############################################################################
##############################################################################
# Obecny vyvoj ###############################################################
##############################################################################
##############################################################################

# MeasurePrepareDevel #######################################################

class MeasurePrepareDevel(measure_gui.MeasurePrepareBasic):
	resources = resources
	dlg_id = 'Dlg_DevelMeasurePrepare'

	def __init__(self, model, parent = None):
		measure_gui.MeasurePrepareBasic.__init__(self, model, parent)
		self.attach_panel(measure_gui.resources, 'UNKN_custom_panel', resources, 'Panel_PrepareHeader')

# DevelFrame ################################################################

class DevelFrame(DocumentFrame, ExportPrintMixin):
	frame_id = None

	def __init__(self, model, parent):
		DocumentFrame.__init__(self, model, resources, self.frame_id, parent)
		ExportPrintMixin.bind_menu(self)
	
		m, f = self.model, self.frame

		self.set_menu(self.menu_id)
		self.bind_file_events(None, 'MI_Open', 'MI_Save', 'MI_SaveAs', 'MI_Quit')
		self.setup_file_dialogs(m.path_devel, '', 
			u"Otevt soubor s vvojem", 
			u"Uloit soubor s vvojem", registry.wildcards.xml)
		self.fullscreen = True
		self.add_view(HTMLView(m.report, f, 'HTML_report'))
		self.plot_views = []

#############################################################################
#############################################################################
# Izokandelovy vyvoj ########################################################
#############################################################################
#############################################################################

# AbbrColor #################################################################

class AbbrColor(CompositeModel):
	_models = {
		'abbr': StrModel(''),
		'color': Color((0.5, 0.5, 0.5)),
	}

	def from_user(self, abbr = None, color = None, parent = None):
		if abbr != None: self.abbr.value = abbr
		if color != None: self.color.set(color)
		if wx.ID_OK != AbbrColorDlg.run_dialog(self, parent):
			return None
		return self

# AbbrColorDlg ##############################################################

class AbbrColorDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_AbbrColor', parent)
		self.add_view(StrEditor(self.model.abbr, self.frame, 'Text_abbr'))
		self.add_view(ColorEditor(self.model.color, self.frame, 'UNKN_color', resources))

# SetMeasureRangeDlg ########################################################

class SetMeasureRangeDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_SetMeasureRange', parent)

		self.attach_panel(resources, 'UNKN_iso_measure_range', 
		                  typetests_gui.resources, 'Panel_iso_measure_range')
		typetests_gui.TypetestIsoFrame.create_range_editors(self)

# ExpandMeshParams ##########################################################

class ExpandMeshParams(CompositeModel):
	_models = {
		'border': StrEnumModel(('left', 'right', 'bottom', 'top'), 'left'),
		'degrees': FloatModel(1.0),
	}

	def validator(self):
		if not self.degrees.value >= 0.0:
			self.validator_failure(u"Je teba roziovat o vce ne nula stupn", self.degrees)
			return False
		return True

	def gen_expanding_points(self, iso_range):
		"""Vygeneruje body, ktere je treba domerit. 
		
		Vygeneruje body, ktere je treba domerit, aby mrizka specifikovana
		iso_range byla rozsirena o rozsireni dane self. Vzdy vygeneruje
		alespon jednu radku (sloupec) bodu, nezavisle na tom jak male degrees
		je. Pokud bude degrees tesne okolo (mensi nebo vetsi) nez step, bude se
		vzdy snazit vygenerovat vice nez mene bodu.

		Samozrejme je nutne, aby byl max-min nasobkem step pro omega i sigma.

		iso_range: Musi mit atributy omega_min/max/step, sigma_min/max/step
		vraci: seznam tuplu (omega, sigma)
		"""
		deg = self.degrees.value
		om, ox, os = iso_range.omega_min.value, iso_range.omega_max.value, iso_range.omega_step.value
		sm, sx, ss = iso_range.sigma_min.value, iso_range.sigma_max.value, iso_range.sigma_step.value
		# che, krutej diktat :-)
		seq = numpy.arange(*{
			'left':   (om - os, min(om - os, om - deg) - os/2, -os),
			'right':  (ox + os, max(ox + os, ox + deg) + os/2,  os),
			'bottom': (sm - ss, min(sm - ss, sm - deg) - ss/2, -ss),
			'top':    (sx + ss, max(sx + ss, sx + deg) + ss/2,  ss),
		}[self.border.value])
		if self.border.value in ('left', 'right'):
			return list(misc.xlist_cross(seq, numpy.arange(sm, sx + ss/2, ss)))
		else:
			return list(misc.xlist_cross(numpy.arange(om, ox + os/2, os), seq))

	def expand_iso_range(self, iso_range):
		"""Rozsiri iso_range podle self."""
		# Asi nejosklivejsi, ale zaroven nejspolehlivejsi zpusob...
		epts = self.gen_expanding_points(iso_range)
		if self.border.value == 'left':   iso_range.omega_min.value = min(pt[0] for pt in epts)
		if self.border.value == 'right':  iso_range.omega_max.value = max(pt[0] for pt in epts)
		if self.border.value == 'bottom': iso_range.sigma_min.value = min(pt[1] for pt in epts)
		if self.border.value == 'top':    iso_range.sigma_max.value = max(pt[1] for pt in epts)

# ExpandMeshDlg #############################################################

class ExpandMeshDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_ExtendMesh', parent)
		m, f = self.model, self.frame
		self.add_view(RadioEditor(m.border, f, 'Radio_border'))
		self.add_view(OmegaSigmaEditor(m.degrees, f, 'Text_degrees'))

# DevelIsoFrame #############################################################

class DevelIsoFrame(DocumentFrame, ExportPrintMixin):
	frame_id = 'Frame_DevelIso'
	menu_id = 'MB_DevelIso'
	isocandel_defs = []

	_re_remove_tags = re.compile(r'<.*?>')
	_re_remove_spaces = re.compile(r'^\s*', re.MULTILINE)
	_re_remove_newlines = re.compile(r'\n')
	_re_remove_degrees = re.compile(r'&deg;')
	_re_lt = re.compile(r'&lt;')
	_re_gt = re.compile(r'&gt;')
	_re_amp = re.compile(r'&amp;')
	_re_omegify = re.compile(r'<font\ face\ *=\ *"Symbol">o</font>', re.VERBOSE | re.IGNORECASE)
	_re_sigmify = re.compile(r'<font\ face\ *=\ *"Symbol">s</font>', re.VERBOSE | re.IGNORECASE)
	_re_lineify = re.compile(r'<br>', re.VERBOSE | re.IGNORECASE)

	@classmethod
	def html2text(cls, html):
		t = cls._re_remove_newlines.sub('', html)
		t = cls._re_omegify.sub('omega', t)
		t = cls._re_sigmify.sub('sigma', t)
		t = cls._re_lt.sub('<', t)
		t = cls._re_gt.sub('>', t)
		t = cls._re_amp.sub('&', t)
		t = cls._re_remove_degrees.sub('', t)
		t = cls._re_lineify.sub('\n', t)
		t = cls._re_remove_tags.sub('', t)
		t = cls._re_remove_spaces.sub('', t)
		return t

	def __init__(self, model, parent):
		DocumentFrame.__init__(self, model, resources, self.frame_id, parent)
		ExportPrintMixin.bind_menu(self)
	
		m, f = self.model, self.frame

		self.set_menu(self.menu_id)
		self.bind_file_events(None, 'MI_Open', 'MI_Save', 'MI_SaveAs', 'MI_Quit')
		self.setup_file_dialogs(m.path_devel, '', 
			u"Otevt soubor s vvojem", 
			u"Uloit soubor s vvojem", registry.wildcards.xml)
		self.bind(wx.EVT_MENU, self.on_lumi, 'MI_Lumi')
		self.bind(wx.EVT_MENU, self.on_expand_mesh, 'MI_AddPoints')
		self.bind(wx.EVT_MENU, self.on_add_def_isocandel, 'MI_AddDefIsocandel')
		self.bind(wx.EVT_MENU, self.on_add_meas_isocandel, 'MI_AddMeasIsocandel')
		self.bind(wx.EVT_MENU, self.on_add_meas_isocandel_from_meas_data, 'MI_AddExternalMeasIsocandelFromMeasData')
		self.bind(wx.EVT_MENU, self.on_add_meas_isocandel_from_eval_typetest, 'MI_AddExternalMeasIsocandelFromEvalTypetest')
		self.bind(wx.EVT_MENU, self.on_add_meas_isocandel_from_devel, 'MI_AddExternalMeasIsocandelFromDevel')
		self.bind(wx.EVT_MENU, self.on_add_all_isocandels_from_typetest, 'MI_AddAllIsocandelsFromTypetest')
		self.fullscreen = True

		m.update_mesh_from_meas_data()
		plot_iso = plot.PlotView(m.plot, f, resources, 'Plot_plot')
		self.plot_iso_controller = plot.PlotController(m.plot, plot_iso, "omega", "sigma")
		self.plot_iso_controller.cb_on_right.register_default(self.on_right)
		self.plot_iso_controller.cb_plottable_dropped.register_default(self.on_drop)
		self.add_view(plot_iso)
		self.plot_views = [plot_iso]

		self.text_report = StrEditor(m.text_report, f, 'Text_report')
		self.add_view(self.text_report)

	def on_right(self, plottable = None, coords = None):
		if not isinstance(plottable, plot.PlotLines):
			return
		self.plottable = plottable
		self.coords = coords
		popup_menu(self.frame, self.get_popup_menu(self.plottable))

	def on_drop(self, plottable, offset):
		if not isinstance(plottable, plot.PlotLines):
			self.plot_iso_controller.default_cb_plottable_dropped(plottable, offset)
			return
		def_isocandel = self.model.find_def_isocandel_by_plottable(plottable)
		if not def_isocandel:
			return
		self.model.move_isocandel(def_isocandel, offset)
		self.plot_iso_controller.view.update()

	def get_popup_menu(self, plottable):
		return [(u'Smazat izokandelu', self.on_plottable_delete)]

	def on_plottable_delete(self, evt = None):
		self.model.delete_isocandel_by_plottable(self.plottable)

	class ExtendMeshExecutionChain(object):
		def __init__(self, model, parent = None):
			"""model: DevelIso"""
			self.model, self.parent = model, parent

		def run(self):
			self.expand_params = ExpandMeshParams()
			if wx.ID_OK != ExpandMeshDlg.run_dialog(self.expand_params, self.parent):
				return

			self.meas = measure.MeasureTable()
			self.meas.meas_data.meas_info.copy_from(self.model.meas_data.meas_info)
			self.meas.init_on_points(self.expand_params.gen_expanding_points(self.model), u"Domovn")
			measure_frame = measure_gui.MeasureUnsaveablePointsFrame(self.meas, self.parent)
			measure_frame.continue_action = self.on_points_measured
			measure_frame.run()

		def on_points_measured(self):
			self.model.meas_data.extend(self.meas.meas_data)
			self.expand_params.expand_iso_range(self.model)
			self.model.update_mesh_from_meas_data()

	def on_expand_mesh(self, evt = None):
		self.ExtendMeshExecutionChain(self.model, self.frame).run()

	def on_lumi(self, evt):
		LuminogramDlg.run_dialog(self.model.plot, self.frame)

	def on_add_def_isocandel(self, evt = None):
		typetest_isocandel_def = ListMVCBuilder.select_item_def(
				u"Pidat novou definovanou izokandelu", 
				u"Zvolte typ izokandely", self.isocandel_defs)
		if not typetest_isocandel_def:
			return
		typetest_isocandel = typetest_isocandel_def.template.copy()
		if wx.ID_OK != typetest_isocandel_def.edit_dlg(typetest_isocandel, self.frame):
			return
		abbr_color = AbbrColor().from_user(color = 'blue', parent = self.frame)
		if abbr_color == None:
			return
		report = self.model.eval_def_isocandel(typetest_isocandel, abbr_color.abbr.value, abbr_color.color)
		self.extend_report(report)

	def extend_report(self, html_report):
		if not html_report:
			return
		self.text_report.write_text(self.html2text(html_report))

	def add_meas_isocandel(self, mesh):
		raise misc.SubclassResponsibilityError

	def on_add_meas_isocandel(self, evt = None):
		self.add_meas_isocandel(self.model.mesh)

	def on_add_meas_isocandel_from_meas_data(self, evt = None):
		fname = open_dialog(self.model.path_data, '',
		                    u"Vyberte soubor s daty men", registry.wildcards.xml)
		if not fname: 
			return
		meas_data = storage.MeasData()
		try:
			meas_data.xml_load(fname)
		except XMLModelLoadError:
			log_error(u"Chyba pi natn souboru s menmi daty")
			return
		mesh = meas_data.as_mesh()
		if not mesh:
			log_error(u"Chyba pi vytven mky z namench dat")
			return
		self.add_meas_isocandel(mesh)

	def on_add_meas_isocandel_from_eval_typetest(self, evt = None):
		fname = open_dialog(self.model.path_eval, '',
		                    u"Vyberte soubor s vyhodnocenm typov zkouky", registry.wildcards.xml)
		if not fname: 
			return
		eval_typetest = self.model.eval_typetest_result_class()
		try:
			eval_typetest.xml_load(fname)
		except XMLModelLoadError:
			log_error(u"Chyba pi natn souboru s vyhodnocenm typov zkouky")
			return
		self.add_meas_isocandel(eval_typetest.mesh)

	def on_add_meas_isocandel_from_devel(self, evt = None):
		fname = open_dialog(self.model.path_devel, '',
		                    u"Vyberte soubor s vvojem", registry.wildcards.xml)
		if not fname: 
			return
		devel = type(self.model)()
		try:
			devel.xml_load(fname)
		except XMLModelLoadError:
			log_error(u"Chyba pi natn souboru s vvojem")
			return
		self.add_meas_isocandel(devel.mesh)

	def on_add_all_isocandels_from_typetest(self, evt = None):
		fname = open_dialog(self.model.path_typetest, '',
		                    u"Vyberte soubor s definic typov zkouky", registry.wildcards.xml)
		if not fname: 
			return
		typetest = self.model.typetest_class()
		try:
			typetest.xml_load(fname)
		except XMLModelLoadError:
			log_error(u"Chyba pi natn souboru s definic typov zkouky")
			return

		abbr_color = AbbrColor().from_user(color = 'blue', parent = self.frame)
		if abbr_color == None:
			return
		for idx, isocandel in enumerate(typetest.isocandels):
			abbr = abbr_color.abbr.value + u' E%i' % (idx+1)
			report = self.model.eval_def_isocandel(isocandel, abbr, abbr_color.color)
			self.extend_report(report)

# DevelDirectionalFrame #####################################################

class DevelDirectionalFrame(DevelIsoFrame):
	isocandel_defs = typetests_gui.TypetestDirectionalFrame.isocandel_defs
	frame_title = u"Vvoj smrovch nvstidel"

	class MeasIsocandel(CompositeModel):
		_models = {
			'abbr': StrModel(''),
			'color': Color(0x884444), 
			'intensity': FloatModel(1, typetests.validator_at_least_1),
		}

	class MeasIsocandelDlg(Dialog):
		def __init__(self, model, parent = None):
			Dialog.__init__(self, model, resources, 'Dlg_AddMeasuredIsocandelDirectional', parent)
			self.add_view(StrEditor(self.model.abbr, self.frame, 'Text_abbr'))
			self.add_view(ColorEditor(self.model.color, self.frame, 'UNKN_color', resources))
			self.add_view(IntensityEditor(self.model.intensity, self.frame, 'Text_intensity'))

	def add_meas_isocandel(self, mesh):
		mi = self.MeasIsocandel()
		if wx.ID_OK != self.MeasIsocandelDlg.run_dialog(mi, self.frame):
			return
		self.model.eval_meas_isocandel(mi.intensity.value, mi.abbr.value, mi.color)

	def on_eval_minmax(self, evt = None):
		report = self.model.eval_minmax(self.def_isocandel)
		self.extend_report(report)

	def on_eval_brightness(self, evt = None):
		report = self.model.eval_brightness(self.def_isocandel)
		self.extend_report(report)

	def get_popup_menu(self, plottable):
		menu = DevelIsoFrame.get_popup_menu(self, plottable)
		self.def_isocandel = self.model.find_def_isocandel_by_plottable(plottable)
		if self.def_isocandel != None:
			menu.append((None, None))
			menu.append((u"Vyhodnotit minimum a maximum", self.on_eval_minmax))
			menu.append((u"Vyhodnotit svtivost", self.on_eval_brightness))
		return menu

# DevelApproachFrame ########################################################

class DevelApproachFrame(DevelIsoFrame):
	isocandel_defs = typetests_gui.TypetestApproachFrame.isocandel_defs
	menu_id = 'MB_DevelApproach'
	frame_title = u"Vvoj sestupovch nvstidel"

	def __init__(self, model, parent):
		DevelIsoFrame.__init__(self, model, parent)
		self.bind(wx.EVT_MENU, self.on_eval_symmetry, 'MI_EvalSymmetry')

	class MeasIsocandel(CompositeModel):
		_models = {
			'abbr': StrModel(''),
			'color': Color(0x44FF44), 
			'white_intensity': FloatModel(1, typetests.validator_at_least_1), 
			'red_intensity': FloatModel(1, typetests.validator_at_least_1),  
		}

	class MeasIsocandelDlg(Dialog):
		def __init__(self, model, parent = None):
			Dialog.__init__(self, model, resources, 'Dlg_AddMeasuredIsocandelApproach', parent)
			m, f = self.model, self.frame
			self.add_view(StrEditor(self.model.abbr, self.frame, 'Text_abbr'))
			self.add_view(ColorEditor(self.model.color, self.frame, 'UNKN_color', resources))
			self.add_view(IntensityEditor(m.white_intensity, f, 'Text_white_intensity'))
			self.add_view(IntensityEditor(m.red_intensity, f, 'Text_red_intensity'))

	def add_meas_isocandel(self, mesh):
		mi = self.MeasIsocandel()
		if wx.ID_OK != self.MeasIsocandelDlg.run_dialog(mi, self.frame):
			return
		self.model.eval_meas_isocandel(mi.white_intensity.value, mi.red_intensity.value, mi.abbr.value, mi.color)

	def on_eval_symmetry(self, evt = None):
		class SymSpecs(CompositeModel):
			_models = {
				'sym_min': typetests.TypetestApproach._models['sym_min'].copy(),
				'sym_max': typetests.TypetestApproach._models['sym_max'].copy(),
			}
			def __init__(self, *args, **kwargs):
				CompositeModel.__init__(self, *args, **kwargs)

			def validate(self):
				CompositeModel.validate(self)
				return typetests.TypetestApproach.validate_symmetry(self)

		class SymSpecsDlg(Dialog):
			def __init__(self, model, parent = None):
				Dialog.__init__(self, model, resources, 'Dlg_Symmetry', parent)
				self.add_view(FloatEditor(self.model.sym_min, self.frame, 'Text_sym_min'))
				self.add_view(FloatEditor(self.model.sym_max, self.frame, 'Text_sym_max'))

		sym_specs = SymSpecs()
		if wx.ID_OK != SymSpecsDlg.run_dialog(sym_specs, self.frame):
			return
		report = self.model.eval_symmetry(sym_specs.sym_min.value, sym_specs.sym_max.value)
		self.extend_report(report)

#############################################################################
#############################################################################
# Vsesmerova navestidla a svitidla ##########################################
#############################################################################
#############################################################################

# PolarAddSpansFrame ########################################################

class DefinedSpans(CompositeModel, typetests.SpansMixin):
	_models = {}
	_models.update(typetests.SpansMixin._models)

class PolarAddSpansFrame(OkCancelFrame):
	def __init__(self, model, parent = None):
		OkCancelFrame.__init__(self, model, resources, 'Frame_DevelPolar_AddSpans', parent, False)

		m, f = self.model, self.frame
		tpf = typetests_gui.TypetestPolarFrame
		tpgr = typetests_gui.resources

		self.attach_panel(resources, 'UNKN_omega_spans', tpgr, 'Panel_omega_spans')
		self.add_view(tpf.create_omega_spans_editor(f, 'List_omega_spans', m.omega_spans))

		self.attach_panel(resources, 'UNKN_sigma_spans', tpgr, 'Panel_sigma_spans')
		self.add_view(tpf.create_sigma_spans_editor(f, 'List_sigma_spans', m.sigma_spans))

# DevelPolarFrame ###########################################################

class DevelPolarFrame(DevelFrame):
	frame_id = 'Frame_DevelPolar'
	menu_id = 'MB_DevelPolar'

	class AddSpansExecutionChain(object):
		def __init__(self, model, parent = None):
			"""model: DevelPolar"""
			self.model, self.parent = model, parent

		def run(self):
			self.def_spans = DefinedSpans()
			add_spans_frame = PolarAddSpansFrame(self.def_spans, self.parent)
			add_spans_frame.set_ok_callback(self.on_spans_defined)
			add_spans_frame.run()

		def on_spans_defined(self):
			if not len(self.def_spans.omega_spans) and not len(self.def_spans.sigma_spans):
				return
			self.meas = measure.MeasurePolar()
			self.meas.meas_data.meas_info.copy_from(self.model.meas_data.meas_info)
			self.meas.init_on_spans(self.def_spans.omega_spans, self.def_spans.sigma_spans, u"Domovn")
			measure_frame = measure_gui.MeasureUnsaveablePointsFrame(self.meas, self.parent)
			measure_frame.continue_action = self.on_spans_measured
			measure_frame.run()

		def on_spans_measured(self):
			eval_result = self.model.eval_result
			try:
				ds = self.def_spans
				eval.EvalPolar.eval_spans(eval_result, self.meas.meas_data, ds.omega_spans, ds.sigma_spans)
				eval_result.prepare_plot_omega_spans(self.model.plot_omega_spans)
				eval_result.prepare_plot_sigma_spans(self.model.plot_sigma_spans)
			except eval.EvalError:
				log_error(u"Chyba pi vyhodnocovn namench rozsah")
			self.model.report.changed()

	def __init__(self, model, parent = None):
		DevelFrame.__init__(self, model, parent)

		m, f = self.model, self.frame

			# graf omega_spans
		m.plot_omega_spans.title.value = u"Vvoj, konst. sigma"

		plot_omega_spans = plot.PlotView(m.plot_omega_spans, f, resources, 'Plot_omega_spans')
		self.plot_omega_spans_controller = \
				plot.PlotController(m.plot_omega_spans, plot_omega_spans, 'omega', u"svtivost")
		self.add_view(plot_omega_spans)
		self.plot_views.append(plot_omega_spans)
			# graf sigma_spans
		m.plot_sigma_spans.title.value = u"Vvoj, konst. omega"

		plot_sigma_spans = plot.PlotView(m.plot_sigma_spans, f, resources, 'Plot_sigma_spans')
		self.plot_sigma_spans_controller = \
				plot.PlotController(m.plot_sigma_spans, plot_sigma_spans, 'sigma', u"svtivost")
		self.add_view(plot_sigma_spans)
		self.plot_views.append(plot_sigma_spans)

		self.bind(wx.EVT_MENU, self.on_add_spans, 'MI_AddSpans')

	def on_add_spans(self, evt = None):
		self.AddSpansExecutionChain(self.model, None).run()

# DevelOmniFrame ############################################################

class DevelOmniFrame(DevelPolarFrame):
	frame_title = u"Vvoj vesmrovch nvstidel"

# DevelLightFrame ##########################################################

class DevelLightFrame(DevelPolarFrame):
	frame_title = u"Vvoj svtidel"

##############################################################################
##############################################################################
# Tabulkovy vyvoj ############################################################
##############################################################################
##############################################################################

# TableAddPointsFrame ########################################################

class TableAddPointsFrame(OkCancelFrame):
	def __init__(self, model, parent = None):
		"""model: ArrayModel(TablePoint())"""
		OkCancelFrame.__init__(self, model, resources, 'Frame_DevelTable_AddPoints', parent, False)
		m, f = self.model, self.frame
		self.attach_panel(resources, 'UNKN_table_points', typetests_gui.resources, 'Panel_table_points')
		self.add_view(typetests_gui.TypetestTableFrame.create_points_editor(f, 'List_points', m))

# DevelTableFrame ###########################################################

class DevelTableFrame(DevelFrame):
	frame_id = 'Frame_DevelTable'
	menu_id = 'MB_DevelTable'

	#-- AddPointsExecutionChain

	class AddPointsExecutionChain(object):
		def __init__(self, model, parent = None):
			"""model: DevelTable"""
			self.model, self.parent = model, parent

		def run(self):
			self.table_points = ArrayModel(typetests.TablePoint())
			add_points_frame = TableAddPointsFrame(self.table_points, self.parent)
			add_points_frame.set_ok_callback(self.on_points_selected)
			add_points_frame.run()

		def on_points_selected(self):
			if len(self.table_points) == 0:
				return
			self.meas = measure.MeasureTable()
			self.meas.meas_data.meas_info.copy_from(self.model.meas_data.meas_info)
			self.meas.init_on_table_points(self.table_points)
			measure_frame = measure_gui.MeasureUnsaveablePointsFrame(self.meas, self.parent)
			measure_frame.continue_action = self.on_points_measured
			measure_frame.run()

		def on_points_measured(self):
			try:
				eval.EvalTable.eval_points(self.model.eval_result, self.meas.meas_data, self.table_points)
			except eval.EvalError:
				log_error(u"Chyba pi vyhodnocovn mench bod")
				return 
			self.model.report.changed()

	#-- DevelTableFrame

	def __init__(self, model, parent = None):
		DevelFrame.__init__(self, model, parent)
		self.bind(wx.EVT_MENU, self.on_add_points, 'MI_AddPoints')

	def on_add_points(self, evt=None):
		self.AddPointsExecutionChain(self.model, None).run()

##############################################################################
# funkce #####################################################################
##############################################################################

def _undefined_func(parent = None):
	log_error(u"Zatm nen implementovno")

def _get_run_devel_new_func(type):
	devel_class = getattr(devel, 'Devel%s'%type)
	devel_frame_class = globals()['Devel%sFrame'%type]

	def devel_func(parent = None):
		devel = devel_class()

		def run_devel_frame():
			devel_frame_class.run_frame(devel, parent_frame)

		def run_devel_iso_measure_frame():
			# U izokandelovych mereni jeste potrebuji odmerit prvotni mrizku.
			# To by sice melo byt asi jinde nez tady, je to takovy maly hack,
			# ale nestoji mi za to to nejak resit... zatim :-)
			def continue_action():
				devel.meas_data.copy_from(meas.meas_data)
				devel_frame_class.run_frame(devel, parent_frame)
		
			if wx.ID_OK != SetMeasureRangeDlg.run_dialog(devel):
				return
			meas = measure.MeasureIso()
			meas.meas_data.meas_info.copy_from(devel.meas_data.meas_info)
			meas.init_on_iso_range(devel, u"Odmovn prvotn mky")
			measure_frame = measure_gui.MeasureUnsaveableIsoFrame(meas)
			measure_frame.continue_action = continue_action
			measure_frame.run()

		if parent:
			parent_frame = parent.frame
		else:
			parent_frame = None
		devel.meas_data.meas_info.meas_type.value = devel.default_meas_type
		mpd = MeasurePrepareDevel(devel.meas_data.meas_info, parent_frame)
		if type in ('Directional', 'Approach'): # Nojo, no, asi by to melo byt jinde..
			mpd.set_ok_callback(run_devel_iso_measure_frame)
		else:
			mpd.set_ok_callback(run_devel_frame)
		try:
			if ckdm_interface_version == 'simul':
				mpd.run_modeless()
			else:
				mpd.run_modal()
		except measure.MeasurePrepareError:
			return

	return devel_func

def _get_run_devel_open_func(type):
	devel_class = getattr(devel, 'Devel%s'%type)
	devel_frame_class = globals()['Devel%sFrame'%type]

	def open_devel_func(parent = None):
		devel = devel_class()

		fname = open_dialog(devel.path_devel, '', u"Vyberte soubor s vvojem", registry.wildcards.xml, parent.frame)
		if fname == None:
			return
		try:
			devel.xml_load(fname)
		except XMLModelLoadError:
			log_error(u"Chyba pi natn souboru s vvojem")
			return
		devel_frame_class.run_frame(devel, parent.frame)

	return open_devel_func

gui_devel_directional_new = _get_run_devel_new_func('Directional')
gui_devel_directional_open = _get_run_devel_open_func('Directional')

gui_devel_approach_new = _get_run_devel_new_func('Approach')
gui_devel_approach_open = _get_run_devel_open_func('Approach')

gui_devel_table_new = _get_run_devel_new_func('Table')
gui_devel_table_open = _get_run_devel_open_func('Table')

gui_devel_omni_new = _get_run_devel_new_func('Omni')
gui_devel_omni_open = _get_run_devel_open_func('Omni')

gui_devel_light_new = _get_run_devel_new_func('Light')
gui_devel_light_open = _get_run_devel_open_func('Light')
