# -*- encoding: cp1250 -*-

"""Logovani neosetrenych chyb (ukradne si sys.excepthook)"""

import sys, traceback

from mvc import *

import debug

__all__ = ('init_error_logging', )

# Exceptions ##################################################################

class GFMError(Exception): pass
class GFMWarn(Exception): pass
class GFMDebug(Exception): pass

def error(msg = ''):
	raise GFMError(msg)

def warn(msg = ''):
	raise GFMWarn(msg)

def inspect(self):
	from wx.py.filling import FillingFrame
	root = [frame.f_locals for frame in get_stack(self.tb)]
	introspect_frame = FillingFrame(rootObject = root)
	# chyba introspect_frame.filling.tree.Expand(root)
	introspect_frame.Show()

# get_stack #################################################################

def get_stack(tb):
	while 1:
		if not tb.tb_next:
			break
		tb = tb.tb_next
	stack = []
	f = tb.tb_frame
	while f:
		stack.append(f)
		f = f.f_back
	stack.reverse()
	return stack

# Error #####################################################################

class Error(CompositeModel):
	_models = {
		'error': StrModel(),
		'info': StrModel(),
		'timestamp': TimeModel(),
	}
	def __init__(self, *args, **kwargs):
		CompositeModel.__init__(self, *args, **kwargs)
		self.timestamp.set_now()

	def format_exception_plus(self, type, value, tb):
		"""
		Print the usual traceback information, followed by a listing of all the
		local variables in each frame.

		From http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52215
		"""
		stack = get_stack(tb)
		retval = traceback.format_exception(type, value, tb)
		retval.append("Locals by frame, innermost last")
		for frame in stack:
			retval.append('')
			retval.append("Frame %s in %s at line %s" % (frame.f_code.co_name,
			                                             frame.f_code.co_filename,
			                                             frame.f_lineno))
			for key, val in frame.f_locals.items():
				val_str = None
				if isinstance(val_str, Model):
					try:
						val_str = val.as_str()
					except:
						pass
				if val_str == None:
					try:				   
						val_str = str(val)
					except:
						pass
				if val_str == None:
					val_str = "<ERROR WHILE PRINTING VALUE>"
				retval.append("\t%20s = %s" % (key, val_str))
		return retval

	def format_exception(self, type, value, tb):
		# return traceback.format_exception(type, value, tb)
		return self.format_exception_plus(type, value, tb)

	def set_error(self, type, value, tb):
		self.error.value = '\n'.join(self.format_exception(type, value, tb))

# ErrorDlg ##################################################################

class ErrorDlg(Dialog):
	def __init__(self, model, parent = None):
		import registry
		self.fullscreen = True
		Dialog.__init__(self, model, registry.xresources.logging, 'Dlg_Error', parent)
		m, f = self.model, self.frame
		self.add_view(StrEditor(m.error, f, 'Text_error'))
		self.add_view(StrEditor(m.info, f, 'Text_info'))
		# Zrejme nejde udelat na cely frame..
		self.xrcctrl('Text_info').Bind(wx.EVT_KEY_DOWN, self.on_key_down)

	def on_key_down(self, evt):
		if not evt.ControlDown() or evt.GetKeyCode() != ord('I'): # Ctrl-I
			evt.Skip()
			return
		self.inspect()

	def inspect(self):
		from wx.py.filling import FillingFrame
		root = [frame.f_locals for frame in get_stack(self.tb)]
		introspect_frame = FillingFrame(rootObject = root)
		# chyba introspect_frame.filling.tree.Expand(root)
		introspect_frame.Show()

	def can_close(self):
		import os, registry
		fname = os.path.join(registry.paths.logs, '%s.xml' % self.model.timestamp.str_timestamp())
		try:
			self.model.xml_save(fname)
		except EnvironmentError, e:
			log_environment_error(e)
			log_error("Chyba pi ukldn protokolu o chyb")
			return True
		msgbox_info("Protokol o chyb byl uloen do souboru %s." % fname)
		return True

# log_exception ##############################################################

def log_exception(type, value, tb):
	e = Error()
	e.set_error(type, value, tb)
	edlg = ErrorDlg(e)
	edlg.tb = tb
	edlg.run_modal()

# init ######################################################################

def init_error_logging():
	pass
	#sys.excepthook = log_exception
