# -*- encoding: cp1250 -*-

import wx

from mvc import *
import plot
import eval_typetest
import registry
import storage
import typetests
import pdf
import misc
from viewdata_gui import LuminogramDlg, ExportPrintMixin
import debug

resources = registry.xresources.eval

# EvalFrame #################################################################

class EvalFrame(DocumentFrame, ExportPrintMixin):
	def __init__(self, model, parent = None):
		"""Model: eval_typetest.EvalTypetest*"""
		DocumentFrame.__init__(self, model, resources, self.frame_id, parent)
		ExportPrintMixin.bind_menu(self)

		m, f = self.model, self.frame

		self.set_menu(self.menu_id)
		self.bind_file_events(None, 'MI_Open', 'MI_Save', 'MI_SaveAs', 'MI_Quit')
		self.setup_file_dialogs(m.typetest.dir_eval, '', 
			u"Otevt soubor s vyhodnocenm typov zkouky", 
			u"Uloit soubor s vyhodnocenm typov zkouky", registry.wildcards.xml)
		self.fullscreen = True
		self.add_view(HTMLView(m.report, f, 'HTML_report'))
		self.plot_views = []

# EvalIsoFrame ###############################################################

class EvalIsoFrame(EvalFrame):
	frame_id = 'Frame_EvalIso'
	menu_id = ('MB_EvalIso')
	def __init__(self, model, parent = None):
		EvalFrame.__init__(self, model, parent)

		m, f = self.model, self.frame

		plot_view = plot.PlotView(m.plot, f, resources, 'Plot_plot')
		self.plot_views.append(plot_view)
		self.add_view(plot_view)
		self.plot_controller = plot.PlotController(m.plot, plot_view, 'omega', 'sigma')

		self.bind(wx.EVT_MENU, self.on_lumi, 'MI_Lumi')

	def on_lumi(self, evt):
		LuminogramDlg.run_dialog(self.model.plot, self.frame)

# EvalDirectionalFrame ######################################################

class EvalDirectionalFrame(EvalIsoFrame):
	"""K funkcionalite EvalIsoFrame jeste umi zmenit zamereni navestidel."""
	def __init__(self, model, parent = None):
		EvalIsoFrame.__init__(self, model, parent)
		pc = self.plot_controller
		pc.cb_plottable_dropped.register_movegroup('def_lines', self.on_drop)
		pc.cb_get_plottable_description.register_movegroup('def_lines', self.get_typetest_description)
		pc.cb_get_dragged_plottable_description.register_movegroup('def_lines', self.get_dragged_typetest_description)

	def get_typetest_description(self, plottable, coords):
		m = self.model
		# z nejakeho duvodu se mi pod W98 nepodarilo nastavit ve statusbaru
		# text > ~85 znaku (usekne se konec).
		txt = u"%s; souasn zmna zamen: omega = %.3f, sigma = %.3f; Lev: zmnit zamen" \
		      % ((plottable.get_description(), ) + m.defined_offset.get_xy())
		return txt

	def get_dragged_typetest_description(self, plottable, coords, offset):
		total_offset = offset + self.model.defined_offset
		txt = u"Definovan izokandely; zmna zamen: omega = %.3f, sigma = %.3f" % total_offset.get_xy()
		return txt

	def on_drop(self, plottable, offset):
		busyinfo = wx.BusyInfo(u"Pevyhodnocuji typovou zkouku...")
		model_backup = self.model.copy()
		self.model.defined_offset.add(offset)
		try:
			self.model.reeval_typetest()
		except eval_typetest.EvalTypetestError, e:
			self.model.copy_from(model_backup)
			self.plot_controller.view.update()
			log_error(u"Chyba pi optovnm vyhodnocovn typov zkouky")

# EvalApproachFrame #########################################################

class EvalApproachFrame(EvalIsoFrame):
	pass

# EvalPolarFrame ############################################################

class EvalPolarFrame(EvalFrame):
	frame_id = 'Frame_EvalPolar'
	menu_id = 'MB_EvalPolar'
	def __init__(self, model, parent = None):
		EvalFrame.__init__(self, model, parent)

		m, f = self.model, self.frame
			# graf omega_spans
		plot_omega_spans = plot.PlotView(m.plot_omega_spans, f, resources, 'Plot_omega_spans')
		self.plot_omega_spans_controller = \
				plot.PlotController(m.plot_omega_spans, plot_omega_spans, 'omega', u"svtivost")
		self.add_view(plot_omega_spans)
		self.plot_views.append(plot_omega_spans)
			# graf sigma_spans
		plot_sigma_spans = plot.PlotView(m.plot_sigma_spans, f, resources, 'Plot_sigma_spans')
		self.plot_sigma_spans_controller = \
				plot.PlotController(m.plot_sigma_spans, plot_sigma_spans, 'sigma', u"svtivost")
		self.add_view(plot_sigma_spans)
		self.plot_views.append(plot_sigma_spans)

# EvalOmniFrame #############################################################

class EvalOmniFrame(EvalPolarFrame):
	frame_title = "Vyhodnocen typov zkouky vesmrovch nvstidel"

# EvalLightFrame ############################################################

class EvalLightFrame(EvalPolarFrame):
	frame_title = "Vyhodnocen typov zkouky svtidel"

# EvalTableFrame ############################################################
			
class EvalTableFrame(EvalFrame):
	frame_id = 'Frame_EvalTable'
	menu_id = 'MB_EvalTable'

# funkce ####################################################################

def gui_ask_eval_iso(type):
	typetest = getattr(typetests, 'Typetest%s'%type)()
	prompt = u"Vyberte soubor s typovou zkoukou"
	fd = FileDialogs(None, typetest.dir_def, '', prompt, prompt, registry.wildcards.xml)
	typetest_path_fname = fd.open_dialog()
	if not typetest_path_fname:
		return
	try:
		typetest.xml_load(typetest_path_fname[0])
	except XMLModelLoadError:
		return

	meas_data = storage.MeasData()
	prompt = u"Vyberte soubor s namenmi daty"
	fd = FileDialogs(None, typetest.dir_data, '', prompt, prompt, registry.wildcards.xml)
	data_path_fname = fd.open_dialog()
	if not data_path_fname:
		return
	try:
		meas_data.xml_load(data_path_fname[0])
	except XMLModelLoadError:
		return

	evaluator = getattr(eval_typetest, 'EvalTypetest%s'%type)
	try:
		result = evaluator.eval_typetest(meas_data, typetest)
	except eval_typetest.EvalTypetestError:
		log_error(u"Chyba pi vyhodnocovn typov zkouky")
		return
	globals()['Eval%sFrame'%type](result).run()

def gui_ask_eval_directional(parent = None):
	gui_ask_eval_iso('Directional')

def gui_view_directional(parent = None):
	EvalDirectionalFrame(eval_typetest.EvalTypetestResultDirectional()).run()

def gui_ask_eval_approach(self, parent = None):
	gui_ask_eval_iso('Approach')

def gui_view_approach(parent = None):
	EvalApproachFrame(eval_typetest.EvalTypetestResultApproach()).run()

def gui_view_omni(parent = None):
	EvalOmniFrame(eval_typetest.EvalTypetestResultOmni()).run()

def gui_view_light(parent = None):
	EvalLightFrame(eval_typetest.EvalTypetestResultLight()).run()

def gui_view_table(parent = None):
	EvalTableFrame(eval_typetest.EvalTypetestResultTable()).run()

