# -*- encoding: cp1250 -*-

from mvc import *
from plot import PlotModel
from ckdm_interface import round_rot, round_tilt, round_arm

# prepare_preview_plot_model ################################################

def prepare_preview_plot_model(title, extent, omega_min, omega_step,
                                              sigma_min, sigma_step):
		p = PlotModel()
		p.extent.copy_from(extent)
		p.title.value = u"Nhled: %s" % title
		p.xlabel.value = r'$\omega [deg]$'
		p.ylabel.value = r'$\sigma [deg]$'
		p.colorbar_title.value = u"Svtivost"
		p.grid.set_grid_offset(True, omega_min, omega_step, sigma_min, sigma_step)
		p.lumi_on.value = p.colorbar_on.value = True
		return p

# prepare_isoplot ###########################################################

def prepare_isoplot(plot_model, title):
	p = plot_model
	p.freeze_changes()
	try:
		p.lumi_on.value = p.colorbar_on.value = True
		p.title.value = title
		p.xlabel.value = r'$\omega [deg]$'
		p.ylabel.value = r'$\sigma [deg]$'
		p.colorbar_title.value = u"Svtivost"
		p.grid.set_grid_fixed(True, p.mesh.cols.value, p.mesh.rows.value)
		p.extent.copy_from(p.mesh.get_extent())
	finally:
		p.thaw_changes()

# prepare_isoplot_from_mesh ##################################################

def prepare_isoplot_from_mesh(plot_model, mesh, title):
	plot_model.freeze_changes()
	try:
		plot_model.mesh.copy_from(mesh)
		prepare_isoplot(plot_model, title)
	finally:
		plot_model.thaw_changes()

# prepare_isoplot_from_meas_data #############################################

def prepare_isoplot_from_meas_data(plot_model, meas_data, title):
	if meas_data.is_empty():
		return
	mesh = meas_data.as_mesh()
	if mesh != None:
		prepare_isoplot_from_mesh(plot_model, mesh, title)

# RotTiltArmView ###################################################################

class RotTiltArmView(FloatView):
	"""View pro souradnice Rot/Tilt/Arm"""
	def format_val(self, val):
		return u'%.3f' % round_rot(val)

# RotTiltArmEditor ###############################################################

class RotTiltArmEditor(RotTiltArmView, Editor):
	"""Editor pro souradnice Rot/Tilt/Arm"""
	def __init__(self, model, frame, xrcid):
		RotTiltArmView.__init__(self, model, frame, xrcid)
		self.set_controller(FloatEditController(model, self))

# OmegaSigmaView #################################################################

class OmegaSigmaView(FloatView):
	"""View pro souradnice Omega/Sigma"""
	def format_val(self, val):
		return u'%.1f' % round_arm(val)

# OmegaSigmaEditor ###############################################################

class OmegaSigmaEditor(OmegaSigmaView, Editor):
	"""Editor pro souradnice Omega/Sigma"""
	def __init__(self, model, frame, xrcid):
		OmegaSigmaView.__init__(self, model, frame, xrcid)
		self.set_controller(FloatEditController(model, self))

# IntensityView #############################################################

class IntensityView(FloatView):
	"""View pro intenzitu"""
	def format_val(self, val):
		return u'%.1f' % val

# IntensityEditor ###############################################################

class IntensityEditor(IntensityView, Editor):
	"""Editor intenzity"""
	def __init__(self, model, frame, xrcid):
		IntensityView.__init__(self, model, frame, xrcid)
		self.set_controller(FloatEditController(model, self))

# CoordsView #################################################################

class CoordsView(FloatView):
	"""View pro souradnice x/y/z (napr. poloha ohniska)"""

	class MMMConverter(Model):
		"""Adapter prevadejici milimetry na metry a naopak"""
		def __init__(self, single_value_model):
			Model.__init__(self)
			self.converted_model = single_value_model
			self.converted_model.add_change_observer(self.on_model_changed)

		def on_model_changed(self):
			self.changed()

		def _get_value(self):
			return self.converted_model.value*1000.0
		def _set_value(self, value):
			self.converted_model.value = value/1000.0
		value = property(_get_value, _set_value)

	def __init__(self, model, frame, textctrl_id, format_str = None):
		FloatView.__init__(self, self.MMMConverter(model), frame, textctrl_id, format_str)

	def format_val(self, val):
		return u'%.3f' % val

# CoordsEditor ###############################################################

class CoordsEditor(CoordsView, Editor):
	"""Editor pro souradnice x/y/z (napr. poloha ohniska)
	
	Jeho specialitou je, ze prevadi vstupni jednotku na vystupni.
	Prevadi milimetry(uzivatel) na metry(model).
	""" 

	def __init__(self, model, frame, xrcid):
		CoordsView.__init__(self, model, frame, xrcid)
		self.set_controller(FloatEditController(self.model, self))
