# -*- encoding: cp1250 -*-

import wx
from wx import xrc

from mvc import *
import registry
import typetests_gui
import calibration_gui
import measure_typetest_gui
import settings_gui
import eval_typetest_gui
import viewdata_gui
import devel_gui
import meas_corrections_gui
import ckdm_interface_gui
from error_logging import *

resources = registry.xresources.main

# MainFrame #################################################################

class MainFrame(Frame):
	frame_title = registry.appname
	def __init__(self):
		Frame.__init__(self, None, resources, 'Frame_Main', statusbar = False)

		self.set_menu('MB_main')

		# self.xrcctrl('Text_main').SetLabel(registry.appname)

		import os.path
		if os.path.exists(registry.fnames.img_logo):
			img = wx.Image(registry.fnames.img_logo, wx.BITMAP_TYPE_BMP).ConvertToBitmap()
			w, h = img.GetWidth(), img.GetHeight()
			self.b = wx.StaticBitmap(self.xrcctrl('Panel_logo'), -1, img, (0, 0), (w, h))

		menu_bindings = (
			(self.           close,                         'MI_Quit'),

			(measure_typetest_gui.gui_measure_directional,  'MI_MeasureDirectional'),
			(measure_typetest_gui.gui_measure_approach,     'MI_MeasureApproach'),
			(measure_typetest_gui.gui_measure_table,        'MI_MeasureTable'),
			(measure_typetest_gui.gui_measure_omni,         'MI_MeasureOmni'),
			(measure_typetest_gui.gui_measure_light,        'MI_MeasureLight'),
			(meas_corrections_gui.gui_measure_matrix,       'MI_MeasureCalibrationMatrix'),

			(eval_typetest_gui.gui_ask_eval_directional,    'MI_EvalDirectional'),
			(eval_typetest_gui.gui_ask_eval_approach,       'MI_EvalApproach'),

			(eval_typetest_gui.gui_view_directional,        'MI_ViewDirectional'),
			(eval_typetest_gui.gui_view_approach,           'MI_ViewApproach'),
			(eval_typetest_gui.gui_view_table,              'MI_ViewTable'),
			(eval_typetest_gui.gui_view_omni,               'MI_ViewOmni'),
			(eval_typetest_gui.gui_view_light,              'MI_ViewLight'),
			(viewdata_gui.     gui_viewdata,                'MI_ViewIsoData'),
			(meas_corrections_gui.gui_view_calibration_matrix, 'MI_ViewCalibrationMatrix'),

			(typetests_gui.  gui_def_typetests_directional, 'MI_DefDirectional'),
			(typetests_gui.  gui_def_typetests_approach,    'MI_DefApproach'),
			(typetests_gui.  gui_def_typetests_table,       'MI_DefTable'),
			(typetests_gui.  gui_def_typetests_omni,        'MI_DefOmni'),
			(typetests_gui.  gui_def_typetests_light,       'MI_DefLight'),

			(devel_gui.      gui_devel_directional_new,      'MI_DevelDirectionalNew'),
			(devel_gui.      gui_devel_directional_open,     'MI_DevelDirectionalOpen'),
			(devel_gui.      gui_devel_approach_new,         'MI_DevelApproachNew'),
			(devel_gui.      gui_devel_approach_open,        'MI_DevelApproachOpen'),
			(devel_gui.      gui_devel_table_new,            'MI_DevelTableNew'),
			(devel_gui.      gui_devel_table_open,           'MI_DevelTableOpen'),
			(devel_gui.      gui_devel_omni_new,             'MI_DevelOmniNew'),
			(devel_gui.      gui_devel_omni_open,            'MI_DevelOmniOpen'),
			(devel_gui.      gui_devel_light_new,            'MI_DevelLightNew'),
			(devel_gui.      gui_devel_light_open,           'MI_DevelLightOpen'),

			(calibration_gui.gui_calibration_table,          'MI_CalibrateTable'),
			(calibration_gui.gui_calibration_arm,            'MI_CalibrateArm'),
			(calibration_gui.gui_luxmeters,                  'MI_Luxmeters'),
			(settings_gui.   gui_settings,                   'MI_Settings'),

			(ckdm_interface_gui.gui_devices,                 'MI_Devices'),
			(about,                                          'MI_About'),
		)

		self.menu_callbacks = {}
		for callback, id in menu_bindings:
			self.bind(wx.EVT_MENU, self.on_menu, id)
			self.menu_callbacks[self.xrcid(id)] = callback

	def can_close(self):
		if registry.open_subwindows.cnt > 1:
			log_error(
				"Ped ukonenm programu je teba zavt vechna ostatn okna")
			return False

		    
		if ask_yes_no(
				"Potvrte ukonen programu",
				"Opravdu chcete program ukonit?"): sys.exit(0)

	def on_menu(self, evt):
		self.menu_callbacks[evt.GetId()](self)

# AboutDlg ##################################################################

class AboutDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_About', parent)
		self.xrcctrl('SText_appname').SetLabel(registry.appname)

		from wx.lib import hyperlink
		hl = hyperlink.HyperLinkCtrl(self.frame, wx.ID_ANY, "MeasProg s.r.o.", 
			style = wx.ALIGN_CENTRE, URL="http://www.measprog.com")
		hl.SetToolTip(wx.ToolTip("Navtvit webov strnky vrobce"))
		hl.SetBackgroundColour(self.frame.GetBackgroundColour())
		hl.SetForegroundColour(wx.Colour(255, 150, 100))
		self.attach_window(resources, 'UNKN_hyperlink', hl)

def about(parent):
	AboutDlg.run_dialog(None, parent.frame)
	
# GonioApp #####################################################################

class GonioApp(wx.App):
	def OnInit(self):
		MainFrame().run()
		return True

# run #######################################################################

def run():
	#import psyco
	#psyco.log()
	#psyco.profile(0.2)

	#import gc
	#gc.set_debug(gc.DEBUG_STATS)

	#import numpy
	#numpy.seterr(over = 'raise', divide = 'raise', invalid = 'raise', under = 'print')

	init_error_logging()
	app = GonioApp(0)
	app.MainLoop()

if __name__ == "__main__":
	run()
