# -*- encoding: cp1250 -*-

import thread

import wx
import wx.lib.newevent

from mvc import *
from gfm_common_gui import *
from plot import PlotView
import registry
from storage import MeasuredPoint, MeshError
import table_math
from ckdm_interface import interface, devices, CKDM_Error
from misc import SubclassResponsibilityError
from measure import TRY_LATER, NextPointError

import debug

#Tuto udalost generuje matika po prepoctu souradnic.
(CoordsComputedEvent, EVT_COORDS_COMPUTED) = wx.lib.newevent.NewEvent()
#Tuto udalost generuje komunikace s CKDM po odmereni hodnoty
(ValueMeasuredEvent, EVT_VALUE_MEASURED) = wx.lib.newevent.NewEvent()

resources = registry.xresources.measure

# MeasurePrepareError #######################################################

class MeasurePrepareError(Exception):
	pass

#############################################################################
# Measure prepare ###########################################################
#############################################################################

# MeasurePrepareCommon ######################################################

class MeasurePrepareCommon(Dialog):
	"""Dialog priprava na mereni. defaultni model je storage.MeasInfo
	
	Umoznuje nastavit pouze udaje, ktere se netykaji orientace stolu, zdroje
	svetla atd.. Vlastni funkci po stisku tlacitka OK nastav pretizenim
	post_ok(nejprve zavolej puvodni, pote vykonej vlastni akci).
	"""
	#-- init
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_MeasurePrepare', parent)

		m, f, mi = self.model, self.frame, self.get_meas_info()
		self.respect_zero = BoolModel(True)

		self.add_view(StrEditor(mi.person, f, 'Text_person', '%s'))
		self.add_view(StrEditor(mi.description, f, 'Text_description', '%s'))

		luxmeters = registry.luxmeters.value
		if luxmeters == None:
			raise MeasurePrepareError
		focused_luxmeter = IntModel(0)
		self._copy_sensor_luxmeter = ArrayItemCopier(luxmeters.luxmeters,
				focused_luxmeter, devices.luxmeter_id)
		self.add_view(ChoiceEditor(luxmeters.luxmeters, f, 'Choice_luxmeter',
		                         lambda i: i.value, focused_luxmeter))

		self.add_view(StrEditor(mi.bulb1, f, 'Text_bulb1', '%s'))
		self.add_view(StrEditor(mi.bulb2, f, 'Text_bulb2', '%s'))

		self.ok_callback = None

	def set_ok_callback(self, ok_callback):
		self.ok_callback = ok_callback

	def get_meas_info(self):
		return self.model

	#-- vycitani hodnot stolu, komunikace s CKDM

	def on_run(self):
		try:
			interface.connect()
		except CKDM_Error, e:
			e.wx_log_error()
			self.measure_failed = True
			return False

		try:
			# Najedu do polohy nastavene v posledni priprave k mereni
			rs = registry.runtime_settings.value
			if ask_yes_no(u"Podtvrte", u"Mm pesunout stl do polohy nastaven ped poslednm menm?"):
				interface.table_goto(rs.last_rot.value, rs.last_tilt.value)
			mi = self.get_meas_info()
			interface.table_invert(False)
			interface.table_manual_enable()
			interface.arm_manual_enable()
			interface.lux_manual_enable()
		except CKDM_Error, e:
			e.wx_log_error()
			self.measure_failed = True
			return False

	def rdcoords(self, *args, **kwargs):
		try:
			interface.table_get_pos()
			interface.arm_get_pos()
			interface.table_manual_enable()
			interface.arm_manual_enable()
		except CKDM_Error, e:
			e.wx_log_error()
			self.measure_failed = True

	def pre_ok(self):
		mi = self.get_meas_info()

		if mi.meas_type.value != mi.NOWEDGE:
			mi.f.x = mi.f.y = 0.0
			mi.changed()

		try:
			if self.respect_zero.value:
				interface.table_get_pos()
				mi.light_rot_zero.value = devices.rot.value
				mi.light_tilt_zero.value = devices.tilt.value
				rtilt = round_tilt(devices.tilt.value)
				if rtilt != 0 and mi.meas_type.value != mi.NOWEDGE:
					msgbox_info(u'\n'.join((
						u"Pi tomto typu men mus mt stl v nulov poloze nulov naklopen.",
						u"Nenulov naklopen bude ignorovno.")))
					mi.light_tilt_zero.value = 0.0
			else:
				mi.light_rot_zero.value = 0.0
				mi.light_tilt_zero.value = 0.0

			mi.table_rot_zero.value = devices.rot_zero.value
			mi.table_tilt_zero.value = devices.tilt_zero.value
			mi.arm_zero.value = devices.arm_zero.value

			interface.ckdm_id(); mi.ckdm.value = devices.ckdm_id.value
			interface.table_id(); mi.table.value = devices.table_id.value
			interface.arm_id(); mi.arm.value = devices.arm_id.value
			mi.luxmeter.value = devices.luxmeter_id.value
			mi.timestamp.set_now()
		except CKDM_Error, e:
			e.wx_log_error()
			self.measure_failed = True
			return False

	def post_ok(self):
		rs = registry.runtime_settings.value
		rs.last_rot.value, rs.last_tilt.value = devices.rot.value, devices.tilt.value
		registry.runtime_settings.save()

		if self.ok_callback != None:
			self.ok_callback()

# MeasurePrepareBasic #######################################################

class MeasurePrepareBasic(MeasurePrepareCommon):
	""" Dialog priprava na mereni.

	K funkcionalite MeasurePrepareCommon dale pridava moznost zvolit typ mereni
	(bez pripravku, s pripravkem, s ramenem) a nastaveni nuly pristroju, vcetne
	toho zda bude respektovana. Dale umoznuje nastavit polohy ohniska a cidla.
	"""

	#-- init

	def __init__(self, model, parent = None):
		MeasurePrepareCommon.__init__(self, model, parent)

		m, f, mi = self.model, self.frame, self.get_meas_info()

		# Vyplnim cidla
		cal = registry.calibration_table.value
		cal.validate()
		if not cal.validation.valid:
			log_error(u"Chybn nastaven stolu, zkontrolujte prosm nastaven v pslunm menu.")
			raise MeasurePrepareError
		sensors = cal.sensors
		focused_sensor = IntModel(0)
		self._copy_sensor_choice = ArrayItemCopier(sensors, focused_sensor, mi.sensor)
		# typ mereni
		self.add_view(ChoicebookEditor(mi.meas_type, f, 'Choice_meas_type'))
		# souradnice ohniska
		for coord in 'x', 'y', 'z': # pouze NOWEDGE
			self.add_view(CoordsEditor(mi.f.float_model(coord), f, 'Text_nowedge_f_%s' % coord))
		for meas_type in 'wedge', 'arm': # pouze 'z'
			self.add_view(CoordsEditor(mi.f.float_model('z'), f, 'Text_%s_f_z' % meas_type))
		# poloha cidla
		for meas_type in 'nowedge', 'wedge': # U ramene neni
			get_description = lambda sensor: sensor.description.value
			self.add_view(ChoiceEditor(sensors, f, 'Choice_%s_sensor' % meas_type, get_description, focused_sensor))
		# respektovat nastavenou nulu
		self.add_view(CheckEditor(self.respect_zero, f, 'Check_respect_zero'))

# UnSaveableMeasuresMixin ###################################################

class UnSaveableMeasuresMixin(object):
	"""Mixin bindujici Cancel, Continue akce pri mereni neukladatelnych dat.
	
	Podrobneji zdokumentovany ve stavovem diagramu v dokumentaci.

	Predefinovava can_close, musi byt kvuli MRO uvedena jako prvni v seznamu
	predku dedicnosti!!!

	Na zacatku musi byt zavolano init_unsaveable_measures

	Potomci MUSI pretizit nasledujici metody:
	continue_action(): Volana pri pokracovani (tj. vyhodnocuje typzk apod.)
	"""

	def init_unsaveable_measures(self):
		self._can_close = False
		self.attach_panel(resources, 'UNKN_buttons', resources, 'Panel_unsaveable_buttons')
		self.bind(wx.EVT_BUTTON, self._on_continue_action, 'But_OK')
		self.bind(wx.EVT_BUTTON, self._on_cancel_action, 'But_Cancel')

	def can_close(self):
		retval = False
		if self._can_close:
			retval = True # Pro pripady kdy proste chci zavrit a na nic se neptat
		elif self.measure_failed:
			retval = True
		elif not self.finished:
			retval = ask_yes_no(u"Potvrde zruen men",
			                    u"Men jet neskonilo. Opravdu chcete men zruit?")
		else:
			retval = ask_yes_no(u"Potvrde ukonen men",
			                    u"Opravdu chcete zahodit namen data a zavt okno?")
		# Tohle je dulezity, pak si v hlavnim mericim vlakne prestanu posilat
		# udalosti do neexistujiciho (zavreneho) framu.
		if retval:
			self.finished = True
		return retval

	def _do_close(self):
		self._can_close = True
		self.finished = True
		self.close()

	def _on_cancel_action(self, evt = None):
		if self.can_close():
			self._do_close()

	def _on_continue_action(self, evt = None):
		if self.measure_failed:
			msgbox_info(u"Men nedobhlo v podku, nelze pokraovat.")
			return

 		if not self.finished:
			msgbox_info(u"Stle se m, nelze pokraovat.")
			return

		self._do_close()
		self.continue_action()

# SaveableMeasuresMixin #####################################################

class SaveableMeasuresMixin(object):
	"""Mixin bindujici Save, Cancel, Continue akce pri mereni ukladatelnych dat.
	
	Podrobneji zdokumentovany ve stavovem diagramu v dokumentaci.

	Predefinovava can_close, musi byt kvuli MRO uvedena jako prvni v seznamu
	predku dedicnosti!!!

	Na zacatku musi byt zavolano init_saveable_measures

	Potomci MUSI pretizit nasledujici metody:
	save_data(): Necht ulozi namerena data a vrati True pokud se povedlo.
	continue_action(): Volana pri pokracovani (tj. vyhodnocuje typzk apod.)
	"""

	def init_saveable_measures(self):
		self._data_saved = False
		self._can_close = False
		self.attach_panel(resources, 'UNKN_buttons', resources, 'Panel_saveable_buttons')
		self.bind(wx.EVT_BUTTON, self._on_save_action, 'But_Save')
		self.bind(wx.EVT_BUTTON, self._on_continue_action, 'But_OK')
		self.bind(wx.EVT_BUTTON, self._on_cancel_action, 'But_Cancel')

	def _save_action(self):
		if not self.finished:
			msgbox_info(u"Stle se m, data nelze uloit.")
			return False

		retval = self.save_data()
		if retval:
			self._data_saved = True
		return retval

	def can_close(self):
		retval = False
		if self._can_close:
			retval = True # Pro pripady kdy proste chci zavrit a na nic se neptat
		elif self.measure_failed:
			retval = True
		elif not self.finished:
			retval = ask_yes_no(u"Potvrde zruen men",
			                    u"Men jet neskonilo. Opravdu chcete men zruit?")
		elif self._data_saved:
			retval = True
		else:
			choice = single_choice_dlg(u"Uloen dat", 
				u"\n".join((
				u"Namen data nejsou uloena.",
				u"Pokud je neulote nyn, budou ztracena. Jak chcete pokraovat?",
				)),
				[u"Uloit namen data a zavt okno",
				 u"Zahodit data a zavt okno",
				 u"Nedlat nic"]
			)

			if choice == 0:
				retval = self._save_action()
			elif choice == 1:
				retval = True

		# Tohle je dulezity, pak si v hlavnim mericim vlakne prestanu posilat
		# udalosti do neexistujiciho (zavreneho) framu.
		if retval:
			self.finished = True
		return retval

	def _do_close(self):
		self._can_close = True
		self.finished = True
		self.close()

	def _on_cancel_action(self, evt = None):
		if self.can_close():
			self._do_close()

	def _on_save_action(self, evt = None):
		if self.measure_failed:
			msgbox_info(u"Men nedobhlo v podku, nelze jej uloit.")
			return
		self._save_action()

	def _on_continue_action(self, evt = None):
		if self.measure_failed:
			msgbox_info(u"Men nedobhlo v podku, nelze pokraovat.")
			return

		if not self.finished:
			msgbox_info(u"Stle se m, nelze pokraovat.")
			return

		if self._data_saved:
			self._do_close()
			self.continue_action()
			return

		choice = single_choice_dlg(u"Uloen dat", 
			u"\n".join((
			u"Namen data nejsou uloena.",
			u"Pokud je neulote nyn, budou ztracena. Jak chcete pokraovat?",
			)),
			[u"Uloit namen data a pokraovat",
			 u"Neukldat data a pokraovat",
			 u"Nedlat nic"]
		)

		if choice == 0:
			if self._save_action():
				self._do_close()
				self.continue_action()
		elif choice == 1:
			self._do_close()
			self.continue_action()

#############################################################################
# Measure Frames ############################################################
#############################################################################

class MeasureFrame(Frame):
	"""Samotny frame pro mereni dat.

	Odmer probiha nasledovne: Mam tri vlakna, jedno je GUI (vlakno 1), v druhem
	se pocitaji souradnice transformovane do souradneho systemu stolu(vlakno
	2), ve tretim probiha komunikace se stolem(pohyb, cteni hodnot luxmetru)
	(vlakno 3). Na zacatku vlakno 1 spusti vlakno 2 s prvni souradnici. Vzdy
	kdyz vlakno 2 spocte nejakou souradnici, posle udalost CoordsComputedEvent
	a ukonci se.  V udalosti vyplni polozky omega, sigma, rot, tilt. Obsluha
	CoordsComputedEvent spusti opet vlakno 2 za predpokladu, ze uz nebyla
	odmerena posledni souradnice. Prijate transformovane souradnice ulozi do
	fronty self.computed_coords. Dale se podiva, zda prave probiha mereni
	(self.measuring_thread_running), pokud ne, spusti vlakno 3 pro pohyb stolu
	a odmer intezity. Kdyz odmer skonci, vlakno 3 vygeneruje udalost
	ValueMeasuredEvent a ukonci se. Handler teto udalosti prekresli dialog,
	podiva se zda je ve fronte transformovanych souradnic nejaka dalsi
	spoctena. Pokud ano, opet spusti vlakno 3 pro dalsi souradnici a pripadne
	opet spusti dalsi mereni.

	Odstartovani mereni probiha tak, ze spustim vlakno 2 pro pocitani prvnich
	souradnic.

	"""
	def __init__(self, model, parent): 
		Frame.__init__(self, model, resources, 'Frame_Measure', parent, False)

		self.bind(EVT_COORDS_COMPUTED, self.on_coords_computed, None)
		self.bind(EVT_VALUE_MEASURED, self.on_value_measured, None)

		self.table_math = table_math.create_table_math(self.model.meas_data.meas_info)
		self.finished = False # True pokud jsme uspesne odmerili
		self.measure_failed = False # True pokud nastala chyba a zemreli jsme...

		self.omega, self.sigma = FloatModel(0), FloatModel(0)
		self.intensity = FloatModel(0)
		self.measured = IntModel(0)

		# Souradnice prepocitane do sour. soust. stolu. None znaci konec mereni
		# Jinak je kazdy prvek tuple (omega, sigma, rot, tilt, arm).
		self.computed_coords = []
		# Odmerujeme? (tj. pohybuje se stul nebo cte se udaj luxmetru?)
		self.measuring_thread_running = False
		# Pocitame prave nove souradnice ?
		self.computing_thread_running = False

		f = self.frame
		self.add_view(OmegaSigmaEditor(self.omega, f, 'Text_omega'))
		self.add_view(OmegaSigmaEditor(self.sigma, f, 'Text_sigma'))
		self.add_view(IntensityView(self.intensity, f, 'Text_intensity'))
		self.add_view(IntView(self.measured, f, 'Text_measured', '%i'))
		self.add_view(GaugeView(self.measured, f, 'Gauge_measured', max = self.model.n_points))

	def set_measure_failed(self):
		if not self.measure_failed:
			self.frame.SetTitle(self.frame.GetTitle() + u" - Men skonilo nespn")
		self.measure_failed = True

	def on_run(self):
		try:
			interface.connect()
		except CKDM_Error, e:
			e.wx_log_error()
			self.set_measure_failed()
			return False

		try:
			interface.measure_prepare() # mimo jine se v simulaci zepta na fci luxmetru
		except CKDM_Error, e:
			e.wx_log_error()
			self.set_measure_failed()
			return False
		
		# Spustime pocitani souradnic, to odstartuje cele mereni.
		self.model.reset_points()
		self.compute_coords()

	def post_event(self, event):
		"""Pokud nenastala chyba, zaradi udalost event do fronty udalosti
		
		event: instance wx.Event
		"""
		if self.finished or self.measure_failed and (not hasattr(event, 'err')):
			return
		wx.PostEvent(self.frame, event)

	#-- support

	def handle_event_error(self, evt):
		"""Osetri pripadnou chybu v udalosti

		evt je moje vlastni udalost(pohyb stolu, odmer, ...) pokud ma nastaveny
		atribut err, nastala chyba a v err je seznam chyb oddelenych znakem
		nove radky. fce hodi pro kazdou chybu log_error a vrati True. Pokud
		chyba nenastala, vraci False.
		"""

		"""
		#DEBUG
		t = type(evt).__name__
		attrs = 'omega', 'sigma', 'rot', 'tilt', 'i', 'err'
		t = t + ', '.join('%s = %s' % (a, str(getattr(evt, a))) for a in attrs if hasattr(evt, a))
		print t
		"""
			
		retval = False
		if hasattr(evt, 'err'):	
			for e in evt.err.split('\n'):
				log_error(e)
			self.set_measure_failed()
			retval = True
		return retval

	#-- Vlastni mereni a pocitani

	def measure(self):
		"""Popne prepocitanou souradnici (pokud neni posledni) a zacne merit"""
		if len(self.computed_coords) == 0:
			return
		coords = self.computed_coords.pop(0)
		if coords != None:
			self.measuring_thread_running = True
			thread.start_new_thread(self.measure_thread, coords)
		else:
			# Mame odmerino, popnuli jsme znacku posledni souradnice :-))
			thread.start_new_thread(self.finish_thread, ())
			self.finished = True

	def finish_thread(self):
		"""Zakonci mereni tim, ze stul presune do polohy pred merenim."""
		rs = registry.runtime_settings.value
		interface.table_goto(rs.last_rot.value, rs.last_tilt.value)

	def measure_thread(self, omega, sigma, rot, tilt, arm):
		""" Presune stul do souradnic rot, tilt a luxmetrem odmeri intenzitu

		Po odmeru vygeneruje udalost EVT_VALUE_MEASURED

		Urcena k tomu aby se vykonavala v samostatnem vlakne.
		"""

		event = ValueMeasuredEvent()
		event.omega, event.sigma = omega, sigma
		try:
			if arm != None:
				interface.arm_goto(arm)
			event.arm = devices.arm.value
			interface.table_goto(rot, tilt)
			event.rot = devices.rot.value
			event.tilt = devices.tilt.value
			interface.lux_get_val()
			event.i = devices.luxmeter.value
		except CKDM_Error, e:
			self.set_measure_failed()
			event.err = e.msg
			
		if not self.finished:
			self.post_event(event)

	def compute_coords(self):
		try:
			new_coords = self.model.next_point()
		except NextPointError:
			log_error(u"Chyba pi potn dal souadnice")	
			self.set_measure_failed()
			return

		if new_coords == None:
			self.computed_coords.append(None) # Znacka posledni souradnice
		elif new_coords != TRY_LATER:
			self.computing_thread_running = True
			thread.start_new_thread(self.compute_coords_thread, new_coords)

	def compute_coords_thread(self, omega, sigma):
		""" Prepocte souradnice svetla na souradnice co se maji poslat stolu.

		Pouziva jiz inicializovany self.table_math. Kdyz souradnice spocte,
		hodi udalost EVT_COORDS_COMPUTED

		Urcena k tomu aby se vykonavala v samostatnem vlakne.
		"""

		event = CoordsComputedEvent()
		try:
			event.rot, event.tilt, event.arm = self.table_math.get_rot_tilt_arm(omega, sigma)
			event.omega, event.sigma = omega, sigma
		except table_math.TableMathError, e:
			self.set_measure_failed()
			event.err = u"\n".join((str(e), u"Chyba pi hledn potebnch souadnic stolu"))
		self.post_event(event)

	#-- handlery udalosti
		
	def on_coords_computed(self, evt):
		"""Byla prepocitana dalsi souradnice"""

		self.computing_thread_running = False
		if self.handle_event_error(evt) or self.measure_failed:
			return

		self.computed_coords.append((evt.omega, evt.sigma, evt.rot, evt.tilt, evt.arm))

		if not self.measuring_thread_running:
			self.measure()
		if not self.computing_thread_running:
			self.compute_coords()

	def on_value_measured(self, evt):
		"""Byla odmerena dalsi hodnota"""

		self.measuring_thread_running = False
		if self.handle_event_error(evt) or self.measure_failed:
			return

		self.measured.value += 1
		mp = MeasuredPoint()
		mp.set_rtai(evt.rot, evt.tilt, evt.arm, evt.i)
		self.table_math.compute_measpoint_osi(mp)
		mp.omega_def, mp.sigma_def = evt.omega, evt.sigma
		self.model.meas_data.points.append(mp)

		self.omega.value, self.sigma.value = mp.omega_def, mp.sigma_def
		self.intensity.value = mp.intensity

		if not self.measuring_thread_running:
			self.measure()
		if not self.computing_thread_running:
			self.compute_coords()

# MeasurePointsFrame ########################################################

class MeasurePointsFrame(MeasureFrame):
	def __init__(self, model, parent): 
		MeasureFrame.__init__(self, model, parent)
		self.attach_panel(resources, 'UNKN_preview', resources, 'Panel_points_list')
		whdr = wx.LIST_AUTOSIZE_USEHEADER
		display_defs = {
			'rot': (u"natoen stolu[deg]", whdr, list_text_novalue('%.3f', 'rot')),
			'tilt': (u"naklopen stolu[deg]", whdr, list_text_novalue('%.3f', 'tilt')),
			'arm': (u"poloha ramene[deg]", whdr, list_text_novalue('%.3f', 'arm')),
			'omega': (u"omega[deg]", whdr, list_text_novalue('%.3f', 'omega_def')),
			'sigma': (u"sigma[deg]", whdr, list_text_novalue('%.3f', 'sigma_def')),
			'illuminance': (u"osvtlenost[lux]", whdr, list_text_novalue('%.3f', 'illuminance')),
			'intensity': (u"svtivost[cd]", whdr, list_text_novalue('%.3f', 'intensity')),
		}
		self.add_view(MulticolArrayView(self.model.meas_data.points, self.frame, 
			'List_points', ('omega', 'sigma', 'intensity', 'rot', 'tilt', 'arm'), 
			display_defs))
	
# MeasureIsoFrame ###################################################

class MeasureIsoFrame(MeasureFrame):
	def __init__(self, model, parent = None): 
		MeasureFrame.__init__(self, model, parent)

		m = self.model
		self.plot = prepare_preview_plot_model(
		                 m.title.value, m.get_extent(),
		                 m.omega_min.value, m.omega_step.value,
		                 m.sigma_min.value, m.sigma_step.value)
		self.add_view(PlotView(self.plot, self.frame, resources, 'UNKN_preview'))

	def on_value_measured(self, evt):
		MeasureFrame.on_value_measured(self, evt)
		if not self.model.replot_hint(evt.omega, evt.sigma):
			return
		try:
			self.plot.mesh.read_from_meas_data(self.model.meas_data)
		except MeshError:
			log_error(u"Chyba pi vykreslovn nhledu mench dat")
			return

# MeasureUnsaveablePointsFrame ###############################################

class MeasureUnsaveablePointsFrame(UnSaveableMeasuresMixin, MeasurePointsFrame):
	def __init__(self, model, parent = None):
		MeasurePointsFrame.__init__(self, model, parent)
		self.init_unsaveable_measures()

# MeasureUnsaveableIsoFrame ###############################################

class MeasureUnsaveableIsoFrame(UnSaveableMeasuresMixin, MeasureIsoFrame):
	def __init__(self, model, parent = None):
		MeasureIsoFrame.__init__(self, model, parent)
		self.init_unsaveable_measures()
