# -*- encoding: cp1250 -*-

import wx

from mvc import *
import registry
import typetests
from storage import MeasInfo
from ckdm_interface import version as ckdm_interface_version
import eval_typetest
import eval_typetest_gui
import measure
import measure_gui
import debug

resources = registry.xresources.measure_typetest

#############################################################################
# Measure prepare ###########################################################
#############################################################################

# MeasurePrepareTypetestDlg #################################################

class MeasurePrepareTypetestDlg(measure_gui.MeasurePrepareBasic):
	#-- init

	def __init__(self, model, typetest_class, measure_frame_class, parent = None):
		measure_gui.MeasurePrepareBasic.__init__(self, model, parent)
		self.attach_panel(measure_gui.resources, 'UNKN_custom_panel', resources, 'Panel_choose_typetest')

		m, f, mi = self.model, self.frame, self.model.meas_data.meas_info

		self.measure_frame_class = measure_frame_class
		self.typetest = typetest_class()

		self.typetest_file = FilenameModel()

		self.filename_browse_editor = FilenameBrowseEditor(
			self.typetest_file, f, 'BrowseText_typetest', 'BrowseBut_typetest', 
			dir = self.typetest.dir_def, wildcard = registry.wildcards.xml, 
			callback = self.read_typetest)
		self.add_view(self.filename_browse_editor)
		self.add_view(IntView(self.model.n_points, f, 'Text_n_points'))

	def get_meas_info(self):
		return self.model.meas_data.meas_info

	def post_ok(self):
		measure_gui.MeasurePrepareBasic.post_ok(self)
		mf = self.measure_frame_class(self.model, self.parent)
		mf.set_typetest(self.typetest)
		mf.run()

	#-- obsluha akci uzivatele

	def valid_or_report_error(self):
		return self.read_typetest()

	def read_typetest(self):
		try:
			self.typetest.xml_load(self.typetest_file.value)
		except XMLModelLoadError:
			self.filename_browse_editor.notify_validation(False)
			log_error("Chyba pi natn typov zkouky")
			return False
		self.typetest.validate()
		if not self.typetest.validation.valid:
			log_model_errors(self.typetest)
			self.filename_browse_editor.notify_validation(False)
			log_error("Chyba pi kontrole typov zkouky")
			return False
		self.filename_browse_editor.notify_validation(True)
		self.model.init_on_typetest(self.typetest)
		return True

# MeasureTypetestFrame ######################################################

class MeasureTypetestMixin(measure_gui.MeasureFrame):
	#-- vyhodnoceni mereni

	def set_typetest(self, typetest):
		self.typetest = typetest

	def continue_action(self):
		evaluator = self.evaluator_class()
		try:
			eval_typetest_result = evaluator.eval_typetest(self.model.meas_data, self.typetest)
		except eval_typetest.EvalTypetestError, e:
			if len(e.args):
				for a in e.args:
					log_error(a)
			else:
				log_error("Chyba pi vyhodnocovn typov zkouky")
			return
		self.eval_frame_class(eval_typetest_result).run()

# MeasureTypetestPointsFrame ################################################

class MeasureTypetestPointsFrame(measure_gui.UnSaveableMeasuresMixin, 
                                 measure_gui.MeasurePointsFrame, MeasureTypetestMixin):
	def __init__(self, *args, **kwargs):
		measure_gui.MeasurePointsFrame.__init__(self, *args, **kwargs)
		measure_gui.UnSaveableMeasuresMixin.init_unsaveable_measures(self)

# MeasureTypetestIsoFrame ###################################################

class MeasureTypetestIsoFrame(measure_gui.SaveableMeasuresMixin, 
                              measure_gui.MeasureIsoFrame, MeasureTypetestMixin):
	def __init__(self, model, parent): 
		measure_gui.MeasureIsoFrame.__init__(self, model, parent)
		measure_gui.SaveableMeasuresMixin.init_saveable_measures(self)

	def save_data(self):
		return try_save_model(self.frame, self.model.meas_data, 
		                      "Vyberte soubor, do kterho se maj uloit namen data",
		                      self.typetest.dir_data, '', registry.wildcards.xml)

##############################################################################
# functions ##################################################################
##############################################################################

class MeasureDirectionalFrame(MeasureTypetestIsoFrame):
	evaluator_class = eval_typetest.EvalTypetestDirectional
	eval_frame_class = eval_typetest_gui.EvalDirectionalFrame

class MeasureApproachFrame(MeasureTypetestIsoFrame):
	evaluator_class = eval_typetest.EvalTypetestApproach
	eval_frame_class = eval_typetest_gui.EvalApproachFrame

class MeasureOmniFrame(MeasureTypetestPointsFrame):
	evaluator_class = eval_typetest.EvalTypetestOmni
	eval_frame_class = eval_typetest_gui.EvalOmniFrame

class MeasureLightFrame(MeasureTypetestPointsFrame):
	evaluator_class = eval_typetest.EvalTypetestLight
	eval_frame_class = eval_typetest_gui.EvalLightFrame

class MeasureTableFrame(MeasureTypetestPointsFrame):
	evaluator_class = eval_typetest.EvalTypetestTable
	eval_frame_class = eval_typetest_gui.EvalTableFrame

def _gui_measure_func(type, default_meas_type):
	measure_class = getattr(measure, 'Measure%s' % type)
	measure_frame_class = globals()['Measure%sFrame' % type]
	typetest_class = getattr(typetests, 'Typetest%s'%type)
	def gui_measure_func(parent):
		try:
			meas = measure_class()
			meas.meas_data.meas_info.meas_type.value = default_meas_type
			mpd = MeasurePrepareTypetestDlg(meas, typetest_class, measure_frame_class)
		except measure_gui.MeasurePrepareError:
			return

		if ckdm_interface_version == 'simul':
			mpd.run_modeless()
		else:
			mpd.run_modal()

	return gui_measure_func

gui_measure_directional = _gui_measure_func('Directional', MeasInfo.NOWEDGE)
gui_measure_approach = _gui_measure_func('Approach', MeasInfo.NOWEDGE)
gui_measure_table = _gui_measure_func('Table', MeasInfo.NOWEDGE)
gui_measure_omni = _gui_measure_func('Omni', MeasInfo.WEDGE)
gui_measure_light = _gui_measure_func('Light', MeasInfo.ARM)
