"""Ruzne pomocne matematicke funkce"""

from numpy import *
from typecheck import *

atan2 = arctan2

@returns(float)
def vec_len(vec):
	sum2 = reduce(lambda sum, e: sum + e**2, vec, 0.0)
	return sqrt(sum2)

@takes((float, int))
@returns(float)
def deg2rad(num):
	return num*pi/180.0

@takes((float, int))
@returns(float)
def rad2deg(num):
	return num*180.0/pi

@takes((float, int))
@returns(float)
def round_up_10(num):
	"""Zaokrouhli cislo na nejblizsi vyssi desitkovy rad (800 -> 1000)"""
	if num <= 0:
		return 0
	return 10**ceil(log10(num))

@takes((float, int))
@returns(float)
def round_down_10(num):
	"""Zaokrouhli cislo na nejblizsi nizsi desitkovy rad (800 -> 100)"""
	if num <= 0:
		return 0
	return 10**floor(log10(num))

@takes((float, int))
@returns(float)
def round_up_1(num):
	"""Zaokrouhli cislo na nejblizsi vyssi cislo stejneho radu (855 -> 900)"""
	if num <= 0:
		return 0
	down_10 = round_down_10(num)
	return down_10*ceil(num/down_10)

@takes((float, int), (float, int), int)
@returns(float)
def round_to(num, step, resolution = 5):
	"""Zaokrouhli num na nejblizsi k*step (k cele). num, step jsou floaty"""
	eres = 10**resolution
	num = int(eres*num)
	step = int(eres*step)
	# v C plati ze int(-1/2) = 0, v pythonu ze int(-1/2) = -1
	result = ((num + step/2)/step)*step;
	return float(result)/eres

def avg_dist(array):
	"""Vraci prumernou vzdalenost prvku v poli array. 
	
	implementace je trochu podezrela, ale pro zhruba rovnomerne
	rozmistene prvky docela funguje :-)
	"""
	return abs(array[-1] - array[0]) / (len(array) - 1)

def arange_nelems(start, step, nelems, dtype = 'f'):
	"""vraci numpy.array se zadanym poctem prvku

	start: Hodnota prvniho prvku
	step: prirustek
	nelems: pocet prvku
	"""
	start = float(start)
	a = fromfunction(lambda x: start + x*step, (nelems,))
	if dtype == a.dtype:
		return a
	return array(a, dtype)

def vec_equals(v1, v2, epsilon = 0.0):
	"""Test zda jsou dva vektory shodne

	v1: prvni vektor, numpy.array
	v2: druhy vektor, numpy.array
	epsilon: Max. odchylka prvku
	vraci: True pokud jsou shodne, False jinak
	"""
	if len(v1) != len(v2):
		return False
	if allclose(v1, v2, 0.0, epsilon):
		return True
	return False

def vec_find_neighbourgs(array, val):
	"""Vraci levy index do pole, pravy index do pole a novou hodnotu
	
	V setrizenem poli array se pokusi najit takove dva indexy i1, i2, ze
	array[i1] <= val <= array[i2]. Pokud se to podari, vraci (i1, i2, val).
	Pokud to nelze (tj. val lezi mimo pole), vraci (0, 1, array[0]) pokud je
	val < array[0], nebo (len(array) - 2, len(array) - 1, array[-1]) pokud je
	val > array[-1].
	"""
	if val <= array[0]:
		return (0, 1, array[0])
	if val >= array[-1]:
		l = len(array)
		return (l-2, l-1, array[-1])
	idx = searchsorted(array, val)
	return (idx - 1, idx, val)

def is_float_modulo(a, b, epsilon):
	"""Test, zda se da b k-krat poskladat do a, k je cele cislo
	"""	
	
	_, m = divmod(a, b)
	if abs(m) > epsilon and abs(m - b) > epsilon:
		return False
	return True

def safe_asin(x):
	if x < -1: x = -1
	elif x > 1: x = 1
	return arcsin(x)

def safe_acos(x):
	if x < -1: x = -1
	elif x > 1: x = 1
	return arccos(x)

def safe_sqrt(n):
	if n < 0:
		return 0
	return sqrt(n)
