# -*- encoding: cp1250 -*-
""" Model View Controller for wxPython.

documentation totally missing, learn from existing modules already using mvc.
"""

import sys, os
from xml.sax.saxutils import quoteattr, escape
from time import localtime
from misc import native, uni

import pyRXP

import wx
from wx import xrc
import wx.lib.mixins.listctrl as listmix

from misc import is_iterable, is_callable, assert_type, assert_type_str, assert_instance, ordered_iteritems
import debug

import numpy

indent_len = 4 # of spaces for each XML indentation level

class ShouldNotImplementError(Exception):
	pass

def log_error(msg):
	wx.LogError(uni(msg))

def log_environment_error(e):
	"""Zahlasi pomoci log_error co se stalo za chybu

	e: instance EnvironmentError
	"""
	try:
		log_error(u"Chyba %i: %s" % (e[0], e[1]))
	except IndexError:
		log_error(u"%s" % str(e))

def msgbox(title, msg, icon):
	dlg = wx.MessageDialog(None, msg, title, icon | wx.OK)
	dlg.ShowModal()
	dlg.Destroy()

def msgbox_info(msg):
	msgbox(u"Informace", uni(msg), wx.ICON_INFORMATION)

def msgbox_warn(msg):
	msgbox(u"Varovn", uni(msg), wx.ICON_WARNING)

def msgbox_err(msg):
	msgbox(u"Chyba", uni(msg), wx.ICON_ERROR)

# ValidationResult ##########################################################

class ValidationResult(object):
	__slots__ = ('valid', 'str', 'item')
	def __init__(self, valid, str = "", item = None):
		"""
		valid: bool
		str: Description (why is it invalid)
		item: item is either None which means that the model containing this
		validation result is the invalid model, or it refers to another model.
		Following items up to the point where it is None should get you to the
		very item that caused the whole tree to be invalid.
		"""
		self.valid, self.str, self.item = valid, str, item
	set = __init__

	def as_str(self):
		if self.valid:
			return u"Valid"
		invld = u"Invalid (%s)" % uni(self.str)
		if self.item != None:
			return invld + u' => %s %s' % (uni(str(self.item)), uni(self.item.validation.pstr()))
		return invld

	def __str__(self):
		return self.as_str()

# XMLModelLoadError #########################################################

class XMLModelLoadError(Exception):
	pass

# WxCommon ##################################################################

class WxCommon(object):
	def xrcctrl(self, xml_id):
		result = xrc.XRCCTRL(self.frame, xml_id)
		if not result:
			log_error(u'Error looking for control `%s\'' % str(xml_id))
		return result

	def xrcid(self, xml_id):
		result = xrc.XRCID(xml_id)
		if not result:
			log_error(u'Error looking for id ' + str(xml_id))
		return result

	def bind(self, event, func, id = None):
		if not id:
			self.frame.Bind(event, func)
		else:
			self.frame.Bind(event, func, id = self.xrcid(id))

	def _get_default_attach_window_parent(self):
		return None

	def attach_window(self, target_xrc, target_id, window, parent = None):
		"""Attaches a wx.Window via AttachUnknownControl

		the window is attached to wxUnknown with target_id in target_xrc
		"""
		if parent == None:
			parent = self._get_default_attach_window_parent()

		target_xrc.value.AttachUnknownControl(target_id, window, parent)
		# This is the neccessary trick I was looking for soo long..
		# see http://article.gmane.org/gmane.comp.python.wxpython/34491
		# for more thorough explanation.
		window.GetParent().SetMinSize(window.GetMinSize())
		#window.GetParent().Layout()
		#window.Layout()
		#if hasattr(self, 'frame'): # ??? !!!
			#self.frame.Layout()

	def attach_panel(self, target_xrc, target_id, src_xrc, src_id, parent = None):
		"""Attaches a panel via AttachUnknownControl

		loads a panel with src_id from src_xrc and attaches it to wxUnknown
		with target_id in target_xrc
		"""
		panel = src_xrc.value.LoadPanel(self.frame, src_id)
		if not panel:
			log_error(u"Error while loading panel with id %s", str(src_id))
			return
		self.attach_window(target_xrc, target_id, panel, parent)

#############################################################################
# Models ####################################################################
#############################################################################

class Model(object):
	def __init__(self):
		object.__setattr__(self, '_change_observers', {})
		object.__setattr__(self, '_changed', False)
		object.__setattr__(self, '_frozen', 0)
		object.__setattr__(self, '_modified', False)
		object.__setattr__(self, 'validation', ValidationResult(True))

	@classmethod
	def new_from(cls, model):
		new = cls()
		new.copy_from(model)
		return new
	
	def copy_from(self, model):
		pass

	def copy(self):
		return type(self).new_from(self)

	def _modified_get(self):
		return self._modified
	def _modified_set(self, val):
		object.__setattr__(self, '_modified', val)
	modified = property(_modified_get, _modified_set)

	def add_change_observer(self, func):
		#print "adding %s to %s" % (str(func), str(self))
		self._change_observers[func] = 1

	def del_change_observer(self, func):
		#print "deleting %s from %s" % (str(func), str(self))
		del self._change_observers[func]

	def changed(self, **kwargs):
		if self._frozen:
			object.__setattr__(self, '_changed', True)
		else:
			self._emit_changed(**kwargs)

	def freeze_changes(self):
		object.__setattr__(self, '_frozen',  self._frozen + 1)

	def thaw_changes(self):
		object.__setattr__(self, '_frozen',  self._frozen - 1)
		assert (self._frozen >= 0)
		if self._changed and not self._frozen:
			self._emit_changed()
			object.__setattr__(self, '_changed', False)
	
	def _emit_changed(self, **kwargs):
		#print "class %s: _emit_changed()" % str(self.__class__)
		from inspect import getargspec
		# Make a copy. Once I got a "dictionary changed size during iteration"
		# RuntimeError during the append operation on List.
		for cl in dict(self._change_observers): 
			_args, _wargs, _kwargs, _default_vals = getargspec(cl)
			if not _kwargs:
				cl() # observer not designed to accept **kwargs
			else:
				cl(**kwargs)

	def as_str(self):
		return self.__str__()

	def validate(self):
		self.validation.set(True)

	#-- xml serializace

	def _xml_write_tag(self, stream, tag, indent = 0):
		stream.write(' '*indent)
		stream.write('<'); stream.write(tag); stream.write('>')

	def _xml_opening_tag(self, stream, tag, indent = 0):
		self._xml_write_tag(stream, tag, indent)

	def _xml_closing_tag(self, stream, tag, indent = 0):
		self._xml_write_tag(stream, '/'+tag, indent)
		stream.write('\n')
		
	def _xml_oneline_tag(self, stream, tag, data, indent = 0):
		"""Automatically escapes the data"""
		self._xml_opening_tag(stream, tag, indent)
		stream.write(escape(native(data)))
		self._xml_closing_tag(stream, tag, 0)

	def _xml_get_oneline_data(self, ps):
		if len(ps[2]) == 0:
			return ''
		elif len(ps[2]) == 1:
			return uni(ps[2][0])
		else:
			self._xml_deserr(u"Chyba pi hledn dat")

	def _xml_deserr(self, str):
		log_error(u"Chyba pi natn z XML (%s): %s" % (type(self).__name__, str))
		raise XMLModelLoadError

	def xml_serialize(self, stream, tag, indent = 0):
		"""XML serialize to file stream.

		stream: class with write() method
		tag: name attribute of the tag or None
		indent: indentation level in the stream
		"""
		self._xml_oneline_tag(stream, tag, '', indent)

	def xml_save(self, fname):
		"""Saves to xml file fname. 
		
		Raises EnvironmentError upon error
		"""
		try:
			f = open(fname, 'w')
			self.xml_serialize(f, type(self).__name__, 0)
			f.close()
		except EnvironmentError:
			log_error(u"Chyba pi ukldn do souboru %s" % fname)
			raise
		self.modified = False

	def xml_load(self, fname):
		"""Updates model data from xml file fname.

		Raises EnvironmentError or XMLModelLoadError upon error
		"""
		try:
			f = open(fname, 'r')
			s = f.read()
			f.close()
		except EnvironmentError, e:
			log_environment_error(e)
			log_error(u"Chyba pi ten souboru " + fname)
			raise XMLModelLoadError

		try:
			ps = pyRXP.Parser().parse(s, ExpandEmpty = 1)
		except pyRXP.error, e:
			log_error(str(e))
			log_error(u"Chyba pi zpracovn XML souboru " + fname)
			raise XMLModelLoadError

		try:
			if ps[0] != type(self).__name__:
				log_error(u"Koenov tag neodpovd td (je: %s, mlo bt: %s)" % 
				            (ps[0], type(self).__name__))
				log_error(u"patn typ souboru")
				raise XMLModelLoadError
			self.xml_deserialize(ps)
		except XMLModelLoadError:
			log_error(u"Chyba pi zpracovn XML souboru " + fname)
			raise
		self.modified = False

	def xml_deserialize(self, ps):
		"""Loads itself from PyRXP structure ps
		
		Generally, if deserialization fails in the middle, the model is in
		inconsistent state. There are no transactions, commits or rollbacks.
		"""
		pass

# SingleValueModel ##########################################################

class SingleValueModel(Model):
	default_val = None

	def __init__(self, val = None, validators = None):
		Model.__init__(self)

		if validators:
			if is_iterable(validators):
				self.validators = list(validators)
			elif is_callable(validators):
				self.validators = [validators]
			else:
				assert(None == u"Validatory musi byt funkce nebo sekvence")
		else:
			self.validators = []

		# Must be here in this order. Assigning value sets modified
		if val == None:
			val = self.default_val
		self.val = None
		self.value = val
		self.modified = False

	def copy_from(self, model):
		self.freeze_changes()
		self.value = model.value
		self.validators = model.validators
		self.modified = model.modified
		self.thaw_changes()

	def _value_get(self):
		return self.val
	
	def _value_set(self, val):
		if val != self.val:
			self.modified = True
		self.val = val
		self.changed()

	value = property(_value_get, _value_set)

	def __str__(self):
		return str(self.value)

	def xml_serialize(self, stream, tag, indent):
		self._xml_oneline_tag(stream, tag, str(self.value), indent)

	def validate(self):
		self.validation.set(True)
		for v in self.validators:
			result = v(self.value)
			if not self.validation.valid:
				return
			if result != None:
				self.validation.set(False, result)
				return
		self.validation.set(True)

	def as_str(self):
		return str(self.val)

# StrModel ################################################################

class StrModel(SingleValueModel):
	default_val = u''

	def __init__(self, val = None, validators = None):
		if val == None:
			SingleValueModel.__init__(self, None, validators)
		else:
			SingleValueModel.__init__(self, uni(val), validators)

	def _value_set(self, val):
		assert_type_str(val)
		if self.value != val:
			self.modified = True
		self.val = val
		self.changed()

	value = property(SingleValueModel._value_get, _value_set)

	def xml_deserialize(self, ps):
		"""Loads itself from PyRXP structure ps"""
		self.value = uni(self._xml_get_oneline_data(ps))

	def __str__(self):
		return self.value

# FloatModel ################################################################

class FloatModel(SingleValueModel):
	default_val = 0.0

	def _value_set(self, val):
		try:
			val = float(val)
		except ValueError:
			print u"Error converting %s of type %s to float" % (str(val), str(type(val)))
			raise
		if self.value != val:
			self.modified = True
		self.val = val
		self.changed()

	value = property(SingleValueModel._value_get, _value_set)

	def xml_deserialize(self, ps):
		"""Loads itself from PyRXP structure ps"""
		val = self._xml_get_oneline_data(ps)
		try:
			self.value = float(val)
		except:
			self._xml_deserr(u"Chyba pi pevodu hodnoty na reln slo")

	def __str__(self):
		return str(self.value)

# IntModel ################################################################

class IntModel(SingleValueModel):
	default_val = 0

	def _value_set(self, val):
		assert_type(val, int)
		if self.value != val:
			self.modified = True
		self.val = val
		self.changed()

	value = property(SingleValueModel._value_get, _value_set)

	def xml_deserialize(self, ps):
		"""Loads itself from PyRXP structure ps"""
		val = self._xml_get_oneline_data(ps)
		try:
			self.value = int(val)
		except:
			self._xml_deserr(u"Chyba pi pevodu value na cel slo")

	def __str__(self):
		return str(self.value)

# BoolModel #################################################################

class BoolModel(SingleValueModel):
	default_val = True

	def _value_set(self, val):
		assert_type(val, bool)
		if self.value != val:
			self.modified = True
		self.val = val
		self.changed()

	value = property(SingleValueModel._value_get, _value_set)

	def xml_deserialize(self, ps):
		"""Loads itself from PyRXP structure ps"""
		val = self._xml_get_oneline_data(ps)
		if val == 'True':
			self.value = True
		elif val == 'False':
			self.value = False
		else:
			self._xml_deserr("Chyba pi pevodu hodnoty na bool")

	def __str__(self):
		return str(self.value)

# StrEnumModel ##############################################################

class StrEnumModel(StrModel):
	def __init__(self, choices = None, default_val = '', validators = None):
		if choices != None:
			self.choices = list(choices)
		else:
			self.choices = []
		StrModel.__init__(self, default_val, validators)

	def _value_set(self, val):
		#assert_type(val, str)
		if type(val) != str:
			print u"val %s has wrong type" % val
		assert val == '' or val in self.choices
		if self.value != val:
			self.modified = True
		self.val = val
		self.changed()

	def copy_from(self, other):
		self.choices = other.choices
		self.value = other.value

	def get_idx(self):
		return self.choices.index(self.value)

	def set_idx(self, idx):
		self.value = self.choices[idx]

	value = property(SingleValueModel._value_get, _value_set)

# FilenameModel #############################################################

class FilenameModel(StrModel):
	def basename(self):
		return os.path.basename(self.value)

# VersionModel ################################################################

class VersionModel(Model):
	def __init__(self, version = ''):
		#assert_type(version, str)
		Model.__init__(self)
		self.version = version
		self.modified = False
		self.validators = []

	def copy_from(self, model):
		self.version = model.version

	def __str__(self):
		return self.version

	def as_str(self):
		return 'version ' + self.version

	def xml_serialize(self, stream, tag, indent):
		self._xml_oneline_tag(stream, tag, str(self.version), indent)

	def xml_deserialize(self, ps):
		"""Loads itself from PyRXP structure ps"""
		val = self._xml_get_oneline_data(ps)
		if val != self.version:
			self._xml_deserr(u"Chybn verze XML. Oekval jsem %s ale nalezl %s." % (self.version, val))

# CompositeModel ############################################################

class CompositeModel(Model):
	_models = {}
	_models_order = ()
	def __init__(self):
		Model.__init__(self)

		Model.__setattr__(self, 'my_models', {})

		for name, template in self._models.iteritems():
			setattr(self, name, template.copy())
		self.source_file = FilenameModel('')
		self.modified = False

	def copy_from(self, model):
		self.freeze_changes()
		try:
			for name, m in self.my_models.iteritems():
				m.copy_from(getattr(model, name))
			self.source_file.copy_from(model.source_file)
		finally:
			self.thaw_changes()

	def __setattr__(self, name, val):
		if not name in self._models:
			Model.__setattr__(self, name, val)
			return
		if hasattr(self, name):
			old_model = getattr(self, name)
			old_model.del_change_observer(self._on_changed)
		self.my_models[name] = val
		Model.__setattr__(self, name, val)
		val.add_change_observer(self._on_changed)
		self._modified = True
		self.changed()

	def _modified_get(self):
		if self._modified:
			return True
		for name, m in self.my_models.iteritems():
			if m.modified:
				return True
		return False

	def _modified_set(self, val):
		self._modified = val
		for name, m in self.my_models.iteritems():
			m.modified = val

	modified = property(_modified_get, _modified_set)

	def _on_changed(self):
		self.changed()

	def validate(self):
		self.validation.set(True)
		for name, m in self.my_models.iteritems():
			m.validate()
			if not m.validation.valid:
				self.validation.set(False, u"Neplatn poloka %s" % type(m).__name__, m)
				return
		if hasattr(self, 'validator'):
			self.validator()

	def validator_failure(self, msg, item = None):
		if item != None:
			self.validation.set(False, msg, item)
			item.validation.set(False, msg)
		else:
			self.validation.set(True)

	def xml_serialize(self, stream, tag, indent):
		self._xml_opening_tag(stream, tag, indent)
		stream.write('\n')
		for name, m in ordered_iteritems(self.my_models, self._models_order):
			m.xml_serialize(stream, name, indent+indent_len)
		self._xml_closing_tag(stream, tag, indent)

	def xml_deserialize(self, ps):
		"""Loads itself from PyRXP structure ps"""
		self.freeze_changes()
		try:
			to_process = set(self.my_models.keys())
			for child in ps[2]:
				if type(child) != tuple:
					continue
				child_name = child[0]
				if not child_name in to_process:
					self._xml_deserr(u"Neoekvan prvek se jmnem %s" % child_name)
				self.my_models[child_name].xml_deserialize(child)
				to_process.remove(child_name)
			if len(to_process):
				self._xml_deserr(u"Nebyly zpracovny vechny atributy, chyb " + 
								 ', '.join(n for n in to_process))
		finally:
			self.thaw_changes()

	def as_str(self):
		return u"\n".join((
			type(self).__name__ + u'{',
			u',\n'.join([name + u': ' + m.as_str() 
			            for name, m in self.my_models.iteritems()]),
			u'}'))

# ListModel ##################################################################

class ListModel(Model):

	#-- inicializace

	def __init__(self, item_templates = None, max_items = sys.maxint):
		Model.__init__(self)
		self.item_templates = item_templates
		self.items = []
		self.modified = False
		self.max_items = max_items

	def _on_changed(self, **kwargs):
		self.changed(**kwargs)

	#-- modified

	def _modified_get(self):
		if self._modified:
			return True
		for i in self.items:
			if i.modified:
				return True
		return False

	def _modified_set(self, val):
		self._modified = val
		for i in self.items:
			i.modified = val

	modified = property(_modified_get, _modified_set)

	#-- copying

	def copy_from(self, model):
		self.freeze_changes()
		try:
			self.item_templates = model.item_templates
			self.empty()
			for i in model.items:
				self.append(i.copy())
			self._modified = True
			self.max_items = model.max_items
		finally:
			self.thaw_changes()

	#-- list wrapping
	
	def __len__(self):
		return self.items.__len__()

	def __getitem__(self, idx):
		return self.items.__getitem__(idx)

	def __setitem__(self, idx, o):
		items = self.items
		old_o = items[idx]
		items[idx] = o
		o.add_change_observer(self._on_changed)
		if not old_o in items:
			old_o.del_change_observer(self._on_changed)
		self._modified = True
		self.changed(action = 'set', idx = idx)

	def __delitem__(self, idx):
		o = self.items[idx]
		self.items.__delitem__(idx)
		if not o in self.items:
			o.del_change_observer(self._on_changed)
		self._modified = True
		self.changed(action = 'delete', idx = idx)

	def index(self, x, start = None, stop = None, cmp_func = None):
		if start != None or stop != None:
			raise NotImplementedError(u"index start/stop not implemented")
		if cmp_func == None:
			return self.items.index(x)
		for idx, i in enumerate(self.items):
			if cmp_func(x, i):
				return idx
		raise ValueError(u"ListModel.index: x not in list")

	def insert(self, idx, o):
		self.freeze_changes()
		while len(self.items) >= self.max_items:
			self.pop(0)
		self.thaw_changes()

		self.items.insert(idx, o)
		o.add_change_observer(self._on_changed)
		self._modified = True
		self.changed(action = 'insert', idx = idx)

	def append(self, o):
		self.insert(sys.maxint, o)

	def extend(self, other):
		self.freeze_changes()
		try:
			for i in other:
				self.append(i)
		finally:
			self.thaw_changes()

	def pop_indices(self, indices):
		"""removes all items with specified indices(sequence of ints)."""
		sorted_indices = list(indices)
		sorted_indices.sort()
		self.freeze_changes()
		try:
			for i, index in enumerate(sorted_indices):
				self.pop(index - i)
		finally:
			self.thaw_changes()

	def pop(self, idx = -1):
		o = self.items.pop(idx)
		if not o in self.items:
			o.del_change_observer(self._on_changed)
		if idx == -1:
			idx = len(self.items)
		self._modified = True
		self.changed(action = 'delete', idx = idx)

	def empty(self):
		self.freeze_changes()
		try:
			try:
				while True:
					self.pop()
			except IndexError:
				pass
		finally:
			self.thaw_changes()

	#-- validation

	def validate(self):
		self.validation.set(True)
		for idx, i in enumerate(self.items):
			i.validate()
			if not i.validation.valid:
				self.validation.set(False, u"Neplatn poloka %i" % idx, i)
				return

	def validator_failure(self, msg, item = None):
		if item != None:
			self.validation.set(False, msg, item)
			item.validation.set(False, msg)
		else:
			self.validation.set(True)
		for idx, m in enumerate(self.items):
			m.validate()
			if not m.validation.valid:
				self.validation.set(False, u"Neplatn poloka %i" % (idx+1), m)
				return

	#-- serialization

	def xml_serialize(self, stream, tag, indent):
		self._xml_opening_tag(stream, tag, indent)
		stream.write('\n')
		for i in self.items:
			i.xml_serialize(stream, type(i).__name__, indent+indent_len)
		self._xml_closing_tag(stream, tag, indent)

	def _xml_find_template(self, class_name):
		for t in self.item_templates:
			if type(t).__name__ == class_name:
				return t
		self._xml_deserr(u"Neumm nast prvek tdy " + class_name)
	
	def xml_deserialize(self, ps):
		"""Loads itself from PyRXP structure ps"""
		self.freeze_changes()
		try:
			self.empty()
			for child in ps[2]:
				if type(child) != tuple:
					continue
				child_class = child[0]
				t = self._xml_find_template(child_class)
				new_item = t.copy()
				new_item.xml_deserialize(child)
				self.append(new_item)
		finally:
			self.thaw_changes()

	#-- as_str

	def as_str(self):
		return u'\n'.join([u'['] + 
		                 [u'%s(%s),' % (m.as_str(), type(m).__name__) for m in self.items] + 
		                 [u']'])

# ArrayModel #################################################################

class ArrayModel(ListModel):
	"""A list, but all items must be of the same type (type(items_type)).
	"""
	def __init__(self, item_template = None, max_items = sys.maxint):
		self.item_template = item_template
		ListModel.__init__(self, item_templates = (item_template,), max_items = max_items)

	def _assert_item_type(self, item):
		assert_type(item, type(self.item_template))
	
	def copy_from(self, model):
		self.item_template = model.item_template
		ListModel.copy_from(self, model)

	def copy_from_values(self, values):
		""" Creates the list from a primitive list of primitive values.

		Example: ArrayModel(StrModel()).create_from_values(['hello', 'world'])
		Only works for items supporting the item.value = something protocol.
		"""
		self.freeze_changes()
		try:
			self.empty()
			for v in values:
				item = self.create_new_item()
				item.value = v
				self.append(item)
		finally:
			self.thaw_changes()
		return self

	def create_new_item(self):
		return self.item_template.copy()	

	#-- serialization

	def xml_serialize(self, stream, tag, indent):
		self._xml_opening_tag(stream, tag, indent)
		stream.write('\n')
		for i in self.items:
			i.xml_serialize(stream, 'item', indent+indent_len)
		self._xml_closing_tag(stream, tag, indent)

	def xml_deserialize(self, ps):
		"""Loads itself from PyRXP structure ps"""
		self.freeze_changes()
		try:
			self.empty()
			for child in ps[2]:
				if type(child) != tuple:
					continue
				new_item = self.create_new_item()
				new_item.xml_deserialize(child)
				self.append(new_item)
		finally:
			self.thaw_changes()

	def as_str(self):
		return u'\n'.join([u'['] + 
		                 [u'%s,' % m.as_str() for m in self.items] + 
		                 [u']'])

# FloatContainerModel #######################################################

class FloatContainerModel(Model):
	_float_attrs = ()

	def __init__(self):
		Model.__init__(self)
		for attr in self._float_attrs:
			setattr(self, attr, 0.0)
		self._float_models = {}

	def copy_from(self, other):
		for attr in self._float_attrs:
			setattr(self, attr, getattr(other, attr))
		self.changed()
		return self

	#-- FloatModel proxying. TODO: weak dictionaries, destroy after noref.

	def changed(self):
		for name, m in self._float_models.iteritems():
			m.value = getattr(self, name) # invokes m.changed as a side effect
		Model.changed(self)

	def proxy_changed(self): # TODO: modify only the modified object.
		for name, m in self._float_models.iteritems():
			setattr(self, name, m.value) 

	def float_model(self, model):
		try:
			return self._float_models[model]
		except KeyError:
			pass
		fm = FloatModel(getattr(self, model))
		self._float_models[model] = fm
		fm.add_change_observer(self.proxy_changed)
		return fm

	#-- serialization

	def xml_serialize(self, stream, tag, indent):
		self._xml_opening_tag(stream, tag, indent)
		stream.write('\n')
		for attr in self._float_attrs:
			self._xml_oneline_tag(stream, attr, str(getattr(self, attr)), indent+indent_len)
		self._xml_closing_tag(stream, tag, indent)

	def xml_deserialize(self, ps):
		to_process = set(self._float_attrs)
		for child in ps[2]:
			if type(child) != tuple:
				continue
			child_name = child[0]
			if not child_name in to_process:
				self._xml_deserr(u"Neoekvan prvek se jmnem %s" % child_name)
			val = float(self._xml_get_oneline_data(child))
			try:
				float_val = float(val)
			except:
				self._xml_deserr(u"Chyba pi pevodu hodnoty na reln slo")
			setattr(self, child_name, val)
			to_process.remove(child_name)
		if len(to_process):
			self._xml_deserr(u"Nebyly zpracovny vechny atributy, chyb " + 
							 u', '.join(n for n in to_process))

	def as_str(self):
		return u'{ ' + u', '.join(name + u':' + str(getattr(self, name)) 
		                        for name in self._float_attrs) + u' }'

# Point #####################################################################

class Point(FloatContainerModel):
	_float_attrs = ('x', 'y')

	#-- get/set

	def get_xy(self):
		return (self.x, self.y)

	def set_xy(self, x, y):
		self.x, self.y = x, y
		self.changed()
		return self

	def set_tuple(self, xy_tuple):
		self.x, self.y = xy_tuple[0], xy_tuple[1]
		self.changed()
		return self

	#-- testing

	def is_zero(self, epsilon = 0.0):
		if epsilon == 0.0:
			return self.x == 0.0 and self.y == 0.0
		return abs(self.x) < epsilon and abs(self.y) < epsilon

	#-- overloaded operators. 
	
	# do not use in-place operators, they are dangerous. 
	# For example imagine an __iadd__ operator on Point which is a member of a
	# Composite model.  calling Point p1 += p2 causes changed to be emmited
	# twice, once in the body in the __iadd__ operator, and the other time
	# because += actually causes the CompositeModel.__setattr__ method to be
	# invoked!

	def __add__(self, other):
		assert_instance(other, Point)
		return Point().set_xy(self.x + other.x, self.y + other.y)

	def __sub__(self, other):
		assert_instance(other, Point)
		return Point().set_xy(self.x - other.x, self.y - other.y)

	def __eq__(self, other):
		if type(self) != type(other):
			return False
		return self.x == other.x and self.y == other.y

	#-- Modifications

	def bound(self, rect):
		l, b, r, t = rect.get_lbrt()
		changed = False
		if self.x < l: 
			self.x = l 
			changed = True
		if self.x > r: 
			self.x = r 
			changed = True
		if self.y < b: 
			self.y = b 
			changed = True
		if self.y > t: 
			self.y = t 
			changed = True
		if changed:
			self.changed()

	def add(self, other):
		assert_instance(other, Point)
		self.x += other.x
		self.y += other.y
		self.changed()
		return self

	def negate(self):
		self.x, self.y = -self.x, -self.y
		self.changed()
		return self

	def negated(self):
		return Point().set_xy(-self.x, -self.y)

# Line ######################################################################

class Line(FloatContainerModel):
	_float_attrs = ('x0', 'y0', 'x1', 'y1')

	def set(self, x0, y0, x1, y1):
		self.x0, self.y0, self.x1, self.y1 = x0, y0, x1, y1
		self.changed()
		return self

	def get(self):
		return self.x0, self.y0, self.x1, self.y1

	def set_tuples(self, p1, p2):
		self.x0, self.y0 = p1
		self.x1, self.y1 = p2
		self.changed()
		return self

	def move(self, offset_x, offset_y):
		self.x0 += offset_x
		self.x1 += offset_x
		self.y0 += offset_y
		self.y1 += offset_y
		self.changed()
		return self

	def get_points(self):
		return (Point().set_xy(self.x0, self.y0), 
		        Point().set_xy(self.x1, self.y1))

# Rect ######################################################################

class Rect(FloatContainerModel):
	_float_attrs = ('l', 'b', 'w', 'h')

	#-- get/set

	def _get_r(self):
		return self.l + self.w
	r = property(_get_r)

	def _get_t(self):
		return self.b + self.h
	t = property(_get_t)

	def get_lbwh(self):
		return (self.l, self.b, self.w, self.h)

	def set_lbwh(self, l, b, w, h):
		self.l, self.b, self.w, self.h = l, b, w, h
		self.changed()
		return self

	def get_lbrt(self):
		l, b, w, h = self.get_lbwh()
		return (l, b, l+w, b+h)

	def set_lbrt(self, l, b, r, t):
		return self.set_lbwh(l, b, r-l, t-b)

	def get_lrbt(self):
		l, b, w, h = self.get_lbwh()
		return (l, l+w, b, b+h)

	def set_lrbt(self, l, r, b, t):
		return self.set_lbwh(l, b, r-l, t-b)

	def set_p1p2(self, p1, p2):
		"""dva rohy - Point()"""
		if p1.x < p2.x:
			self.l, self.w = p1.x, p2.x-p1.x
		else:
			self.l, self.w = p2.x, p1.x-p2.x

		if p1.y < p2.y:
			self.b, self.h = p1.y, p2.y-p1.y
		else:
			self.b, self.h = p2.y, p1.y-p2.y
		self.changed()
		return self

	# math

	def area(self):
		return self.w * self.h

	def contains_point(self, pt):
		l, r, b, t = self.get_lrbt()
		if l > r:
			l, r = r, l
		if b > t:
			b, t = t, b
		x, y = pt.get_xy()
		return (l <= x <= r) and (b <= y <= t)

	def contains_rect(self, rect):
		"""Tests whether rect is fully contained in self.

		returns True if rect is fully contained in self. If not, returns a set
		containing some (or all elements) 'left', 'right', 'bottom', 'top'
		indicating which boundaries where crossed.
		"""
		crossings = set()
		sl, sb, sr, st = self.get_lbrt()
		rl, rb, rr, rt = rect.get_lbrt()
		if (rl < sl):
			crossings.add('left')
		if (rr > sr):
			crossings.add('right')
		if (rb < sb):
			crossings.add('bottom')
		if (rt > st):
			crossings.add('top')
		if len(crossings) == 0:
			return True
		return crossings

	# modifications

	def set_union(self, rects):
		l, b, r, t = rects[0].get_lbrt()
		for rect in rects[1:]:
			_l, _b, _r, _t = rect.get_lbrt()
			if _l < l: l = _l
			if _b < b: b = _b
			if _r > r: r = _r
			if _t > t: t = _t
		self.set_lbrt(l, b, r, t)
		return self

	def expand(self, x, y):
		"""Expands the rectangle by x(width) and y(height) on each side."""
		self.l -= x
		self.b -= y
		self.w += 2*x
		self.h += 2*y
		self.changed()
		return self

	def expand_ratio(self, ratio_width, ratio_height):
		"""Expands the rectangle ratio_width and ratio_height times on each side."""
		x = self.w*ratio_width
		y = self.h*ratio_height
		return self.expand(x, y)

	def move(self, p):
		assert_instance(p, Point)
		self.l += p.x
		self.b += p.y
		self.changed()
		return self

# Color #####################################################################

class Color(CompositeModel):

	# TODO neni doreseny prevod barev se jmeny a barev s hodnotou - je doresen
	# pouze na urovni matplotlibu.
	_models = {
		'r': FloatModel(1.0),
		'g': FloatModel(1.0),
		'b': FloatModel(1.0),
		'name': StrModel('white'),
	}
	def __init__(self, color = None):
		CompositeModel.__init__(self)
		self.set(color)

	def set(self, color):
		if color == None:
			return

		if type(color) == str:
			self.set_name(color)
			return

		if isinstance(color, Color):
			self.copy_from(color)
			return

		if isinstance(color, wx.Color):
			r, g, b = color.Get()
			self.set_rgb(r/255.0, g/255.0, b/255.0)

		if type(color) in (tuple, list) and len(color) == 3:
			self.set_rgb(*color)
			return

		if isinstance(color, int):
			self.set_hex(color)

	def as_wx_color(self):
		c = wx.Color()
		c.Set(int(self.r.value*255.0), int(self.g.value*255.0), int(self.b.value*255.0))
		return c

	def get(self):
		if self.name.value == '':
			return (self.r.value, self.g.value, self.b.value)
		else:
			return self.name.value

	def set_rgb(self, r, g, b):
		self.freeze_changes()
		try:
			self.r.value, self.g.value, self.b.value = r, g, b
			self.name.value = ''
		finally:
			self.thaw_changes()
		return self

	def set_name(self, name):
		self.name.value = name
		return self

	def set_hex(self, hexcode):
		"""accepts color as 0xRRGGBB"""
		return self.set_rgb((hexcode>>16)/256.0, ((hexcode>>8) & 0xff)/256.0, (hexcode & 0xff)/256.0)

# NumpyArray ##############################################################
	
class NumpyArray(Model):
	def __init__(self, dtype = numpy.dtype('float64')):
		Model.__init__(self)
		self.dtype = dtype
		self.value = numpy.array((), self.dtype)
		self.minmax_updated = False
		self.minmax = (0.0, 0.0)

	def resize(self, shape):
		a = numpy.zeros(0, dtype = self.dtype)
		self.value = numpy.resize(a, shape)
		self.changed()

	def copy_from(self, model):
		self.value = model.value.copy()
		self.dtype = model.dtype

	def find_minmax(self):
		def minmax(a):
			if len(numpy.shape(a)) == 1:
				return (min(a), max(a))
			else:
				_min, _max = minmax(a[0])
				for sub_a in a[1:]:
					_sub_min, _sub_max = minmax(sub_a)
					_min = min(_min, _sub_min)
					_max = max(_max, _sub_max)
				return (_min, _max)
		return minmax(self.value)

	def changed(self, **kwargs):
		Model.changed(self, **kwargs)
		self.minmax_updated = False

	def xml_serialize(self, stream, tag, indent):
		def serialize_dimension(stream, indent, array):
			stream.write(' '*indent)
			stream.write('<dimension>\n')

			if len(numpy.shape(array)) == 1:
				indent_str = ' '*(indent + indent_len)
				for elem in array:
					stream.write(indent_str)
					stream.write('<num>%f</num>"/>\n' % float(elem))
			else:
				for dim in array:
					serialize_dimension(stream, indent+indent_len, dim)

			stream.write(' '*indent)
			stream.write('</dimension>\n')

		self._xml_opening_tag(stream, tag, indent)
		stream.write('\n')
		shape_str = StrModel(','.join(str(s) for s in numpy.shape(self.value)))
		shape_str.xml_serialize(stream, 'shape', indent + indent_len)
		dtype_str = StrModel(str(self.value.dtype))
		dtype_str.xml_serialize(stream, 'dtype', indent + indent_len)
		serialize_dimension(stream, indent + indent_len, self.value)
		stream.write(' '*indent)
		self._xml_closing_tag(stream, tag, 0)

	def xml_deserialize(self, ps):
		"""Loads itself from PyRXP structure ps"""

		def ps_childs(ps):
			for p in ps[2]:
				if type(p) != str:
					yield p

		def deserialize_dimension(ps, array, shape):
			if ps[0] != 'dimension':
				self._xml_deserr(u"Oekval jsem tag <dimension>, nalezl %s." % ps[0])
			count = shape[0]
			if len(shape) == 1:
				i = 0
				for i, num in enumerate(ps_childs(ps)):
					if i >= count:
						self._xml_deserr(u"Pli mnoho sel, oekval jsem %i" % count)
					if not num[0] == 'num':
						self._xml_deserr(u"Oekval jsem tag num, pilo %s" % ps[0])
					val_str = self._xml_get_oneline_data(num)
					try:
						val = float(val_str)
					except ValueError:
						self._xml_deserr(u"Nemohu pevst hodnotu atributu value na slo")
					array[i] = val

				if i+1 < count:
					self._xml_deserr(u"Pli mlo sel, oekval jsem %i, pilo %i" % (count, i+1))
			else:
				for i, num in enumerate(ps_childs(ps)):
					if i >= count:
						self._xml_deserr(u"Pli dimenz, oekval jsem %i" % count)
					deserialize_dimension(num, array[i], shape[1:])
				if i+1 < count:
					self._xml_deserr(u"Pli mlo dimenz, oekval jsem %i, pilo %i" % (count, i+1))

		# nactu shape
		child_tags = ps_childs(ps)
		try:
			shape_ps = child_tags.next()
		except StopIteration:
			self._xml_deserr(u"Chyba pi ten pole, nenalezeno shape.")
		if shape_ps[0] != 'shape':
			self._xml_deserr(u"Chyba pi ten pole, nenalezeno shape.")
		shape_str = self._xml_get_oneline_data(shape_ps)
		try:
			shape = tuple(int(x) for x in shape_str.split(','))
		except:
			self._xml_deserr(u"Chyba pi ten pole, neplatn shape.")
		self.resize(shape)
		# nactu dtype
		try:
			dtype_ps = child_tags.next()
		except StopIteration:
			self._xml_deserr(u"Chyba pi ten pole, nenalezeno dtype.")
		if dtype_ps[0] != 'dtype':
			self._xml_deserr(u"Chyba pi ten pole, nenalezeno dtype.")
		dtype_str = self._xml_get_oneline_data(dtype_ps)
		if numpy.dtype(dtype_str) != self.value.dtype:
			self._xml_deserr(u"Chybn dtype, oekval jsem %s, pilo %s." % 
			                 (self.value.dtype, dtype_str))
		# nactu pole
		try:
			array_ps = child_tags.next()
		except StopIteration:
			self._xml_deserr(u"Chyba pi ten pole, nenalezeny hodnoty.")
		deserialize_dimension(array_ps, self.value, shape)

	def contains_invalid_point(self):
		return numpy.any(numpy.isinf(self.value)) or numpy.any(numpy.isnan(self.value))

	def validate(self):
		self.validation.set(True)
		if self.contains_invalid_point():
			self.validation.set(False, u"Matice obsahuje body s nekonenou hodnotou")

	def __str__(self):
		return str(self.value)

# TimeModel #################################################################

class TimeModel(CompositeModel):
	_models = {
		'year': IntModel(0), 'month': IntModel(0), 'day': IntModel(0),
		'hour': IntModel(0), 'min':   IntModel(0), 'sec': IntModel(0),
	}

	def set_now(self):
		(self.year.value, self.month.value, self.day.value,
		 self.hour.value, self.min.value, self.sec.value,
		 _, _, _) = localtime()
		return self

	def str_datetime(self):
		return self.__str__()

	def as_str(self):
		return self.str_datetime()

	def str_time(self):
		return u'%.02i:%.02i:%.02i' % (self.hour.value, self.min.value, self.sec.value)

	def str_date(self):
		return u'%i.%i %i' % (self.day.value, self.month.value, self.year.value)

	def str_timestamp(self):
		return u'%.02i-%.02i-%.02i %.02i-%.02i-%.02i' % tuple([getattr(self, a).value for a in
				'year', 'month', 'day', 'hour', 'min', 'sec'])

	def __str__(self):
		return u'%i.%i %i %.02i:%.02i:%.02i' % tuple([getattr(self, a).value for a in
				'day', 'month', 'year', 'hour', 'min', 'sec'])

#############################################################################
# Adaptors ##################################################################
#############################################################################

# FuncAdaptor ###############################################################

class FuncAdaptor(Model):
	def __init__(self, func):
		Model.__init__(self)
		self.func = func

	def _value_get(self):
		return self.func()
	value = property(_value_get)

#############################################################################
# Views #####################################################################
#############################################################################

class View(WxCommon):
	def __init__(self, model, frame):
		self.model, self.frame = model, frame
		if hasattr(self, 'update') and model != None:
			self.model.add_change_observer(self.update)
		self.update_ok = True
		self.validation_ok = True
		self.enabled = None # BoolModel

	def done(self):
		if self.model != None:
			self.model.del_change_observer(self.update)
		if self.enabled != None:
			self.enabled.del_change_observer(self.on_enabled_changed)

	def update(self):
		pass

	def set_enabled_model(self, enabled = None):
		if self.enabled != None:
			self.enabled.del_change_observer(self.on_enabled_changed)
		self.enabled = enabled
		if self.enabled != None:
			self.enabled.add_change_observer(self.on_enabled_changed)
			self.on_enabled_changed()

	def on_enabled_changed(self):
		if hasattr(self, 'ctrl'):
			self.ctrl.Enable(self.enabled.value)

	def notify_update(self, ok):
		if ok != self.update_ok:
			self.update_ok = ok
			self.update_highlight()

	def notify_validation(self, ok):
		if ok != self.validation_ok:
			self.validation_ok = ok
			self.update_highlight()

	def update_highlight(self):
		if not self.update_ok:
			self.highlight('thistle')
		elif not self.validation_ok:
			self.highlight('yellow')
		else:
			self.highlight(None)

	def highlight(self, color = None):
		if not hasattr(self, 'ctrl'):
			return

		if color == None:
			color = wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW)
		self.ctrl.SetBackgroundColour(color)
		self.ctrl.Refresh()

	def set_focused(self):
		if not hasattr(self, 'ctrl'):
			return
		self.ctrl.SetFocus()

# StaticTextView ############################################################

class StaticTextView(View):
	def __init__(self, model, frame, textctrl_id, format_str = u'%s'):
		View.__init__(self, model, frame)
		self.ctrl = self.xrcctrl(textctrl_id)
		self.format_str = format_str

	def update(self):
		self.ctrl.SetLabel(self.format_str % self.model.value)

# StaticTextBoolView ############################################################

class StaticTextBoolView(View):
	def __init__(self, model, frame, textctrl_id, yes_str = u"Ano", no_str = u"Ne"):
		View.__init__(self, model, frame)
		self.ctrl = self.xrcctrl(textctrl_id)
		self.str_dict = {True: yes_str, False: no_str}

	def update(self):
		self.ctrl.SetLabel(self.str_dict[self.model.value])

# TextView ##############################################################

class TextView(View):
	default_format_str = u'%s'

	def __init__(self, model, frame, textctrl_id, format_str = None):
		View.__init__(self, model, frame)
		self.textctrl_id = textctrl_id
		self.ctrl = self.xrcctrl(textctrl_id)
		self.deafness = 0
		if format_str == None:
			self.format_str = self.default_format_str
		else:
			self.format_str = format_str

	def format_val(self, val):
		return uni(self.format_str % val)

	def set(self, val):
		try:
			self.ctrl.SetValue(self.format_val(val))
		except UnicodeEncodeError:
			print [type(val), val]
			self.ctrl.SetValue(self.format_val(val))


	def get(self):
		return native(self.ctrl.GetValue())

	def update(self):
		if not self.deafness:
			self.set(self.model.value)

# StrView ###################################################################

class StrView(TextView):
	default_format_str = u'%s'

# IntView ###################################################################

class IntView(TextView):
	default_format_str = u'%i'

# FloatView ###################################################################

class FloatView(TextView):
	default_format_str = u'%f'

# CheckView ##############################################################

class CheckView(View):
	def __init__(self, model, frame, ctrl_id):
		View.__init__(self, model, frame)
		self.ctrl = self.xrcctrl(ctrl_id)

	def update(self):
		if self.ctrl.GetValue() != self.model.value:
			self.ctrl.SetValue(self.model.value)

# RadioView ##############################################################

class RadioView(View):
	def __init__(self, model, frame, ctrl_id):
		View.__init__(self, model, frame)
		self.ctrl = self.xrcctrl(ctrl_id)

	def update(self):
		if self.ctrl.GetSelection() != self.model.value:
			self.ctrl.SetSelection(self.model.get_idx())

# GaugeView #################################################################

class GaugeView(View):
	def __init__(self, model, frame, gauge_id, min = None, max = None):
		View.__init__(self, model, frame)
		self.gauge = self.xrcctrl(gauge_id)
		self.gauge.SetRange(10000)
		if min == None:
			min = FloatModel(0)
		if max == None:
			max = FloatModel(1)
		self.min, self.max = min, max

		self.min.add_change_observer(self.update)
		self.max.add_change_observer(self.update)

	def done(self):
		self.min.del_change_observer(self.update)
		self.max.del_change_observer(self.update)

	def update(self):
		min, max = float(self.min.value), float(self.max.value)
		val = float(self.model.value)
		s = 10000.0 * (val - min) / (max - min)
		if s < 0.0:
			s = 0
		elif s > 10000.0:
			s = 10000
		self.gauge.SetValue(int(s))

# ListView ##################################################################
		
class ListView(View):
	class ListCtrl_Autowidth(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
		def Focus(self, idx):
			if self.GetFocusedItem() == idx:
				return
			state_bits = wx.LIST_STATE_FOCUSED | wx.LIST_STATE_SELECTED
			self.SetItemState(idx, state_bits, state_bits)
			self.EnsureVisible(idx)
			
	def __init__(self, model, frame, listctrl_id, focused_item = None):
		View.__init__(self, model, frame)
		self.listctrl_id = listctrl_id
		self.ctrl = self.xrcctrl(listctrl_id)
		self.display_definitions = {}
		self.columns = []

		self.ctrl.__class__ = self.ListCtrl_Autowidth
		listmix.ListCtrlAutoWidthMixin.__init__(self.ctrl)

		if focused_item == None:
			self.focused_item = IntModel(-1)
		else:
			self.focused_item = focused_item
		self.focused_item.add_change_observer(self.on_focused_changed)
		self.bind(wx.EVT_LIST_ITEM_FOCUSED, self.on_evt_focused, self.listctrl_id)

	def set_display_definitions(self, display_definitions):
		"""
		display_definitions: a dictionary {
			item_class: {column_id: set_item_func(wx.ListItem, model), ...},
			...
		}
		"""
		self.display_definitions = display_definitions

	def add_display_definition(self, item_class, display_definition):
		"""
		display_definition: {column_id: set_item_func(wx.ListItem, model), ...}
		"""
		self.display_definitions[item_class] = display_definition

	def set_columns(self, columns):
		"""Sets the columns for the view

		columns: a sequence [(column_id, column_title, column_width), ...]
		"""
		self.columns = columns
		for i in xrange(self.ctrl.GetColumnCount()):
			self.ctrl.DeleteColumn(0)
		for idx, col in enumerate(self.columns):
			self.ctrl.InsertColumn(idx, col[1])
		self.resize_columns()

	def resize_columns(self):
		for idx, col in enumerate(self.columns):
			self.ctrl.SetColumnWidth(idx, col[2])
		self.ctrl.resizeLastColumn(0)

	def insert_item(self, idx, item):
		try:
			item_display = self.display_definitions[type(item)]
		except KeyError:
			return
		for col, col_def in enumerate(self.columns):
			try:
				set_item_func = item_display[col_def[0]]
			except KeyError:
				continue
			li = wx.ListItem()
			set_item_func(li, item)
			# The wx.ListCtrl item interface is IMO pretty clumsy designed
			li.SetColumn(col)
			if col == 0:
				li.SetId(idx)
				new_idx = self.ctrl.InsertItem(li)
			else:
				li.SetId(new_idx)
				self.ctrl.SetItem(li)

	def append_item(self, item):
		self.insert_item(sys.maxint, item)

	def refresh(self, reinsert_items = True):
		last_focused = self.focused_item.value
		if reinsert_items:
			self.ctrl.DeleteAllItems()
			for i in self.model.items:
				self.append_item(i)
		if last_focused >= len(self.model) - 1:
			last_focused = len(self.model) - 1
		self.focused_item.value = last_focused

	def update(self, **kwargs):
		action = kwargs.get('action')
		if action == None:
			self.refresh(reinsert_items = True)
			return
		idx = kwargs.get('idx')
		if action == 'insert':
			if idx == sys.maxint:
				self.insert_item(idx, self.model.items[-1])
				self.focused_item.value = len(self.model.items) - 1
			else:
				self.insert_item(idx, self.model.items[idx])
				self.focused_item.value = idx
		elif action == 'delete':
			self.ctrl.DeleteItem(idx)
			self.on_evt_focused()
			self.refresh(reinsert_items = False)
		else:
			self.refresh(reinsert_items = True)

	def on_focused_changed(self):
		self.ctrl.Select(self.focused_item.value, True)
		self.ctrl.Focus(self.focused_item.value)

	def on_evt_focused(self, evt = None):
		self.focused_item.value = self.ctrl.GetFocusedItem()

	def done(self):
		View.done(self)
		self.focused_item.del_change_observer(self.on_focused_changed)

# ArrayView #################################################################

class ArrayView(ListView):
	def __init__(self, model, frame, listctrl_id, display_definition, 
	             focused_item = None):
		"""
		display_definition: {column_id: set_item_func(wx.ListItem, model), ...}
		"""
		ListView.__init__(self, model, frame, listctrl_id, focused_item)
		self.display_definitions = {type(model.item_template): display_definition}

	def set_display_definitions(self, display_definitions):
		raise ShouldNotImplementError

	def add_display_definition(self, item_class, display_definition):
		raise ShouldNotImplementError

# MulticolArrayView #########################################################

class MulticolArrayView(ArrayView):
	def __init__(self, model, frame, listctrl_id, cols, display_definition,
	             focused_item = None):
		dd = dict((col_id, set_func)
			   for col_id, (col_title, col_width, set_func)
			   in display_definition.iteritems())
		ArrayView.__init__(self, model, frame, listctrl_id, dd, focused_item)
		self.set_columns([(col_id, display_definition[col_id][0], display_definition[col_id][1])
		                  for col_id in cols])

# ColorView #################################################################

class ColorView(View):
	def __init__(self, model, frame, colorpickerctrl_id = None, attach_to_resource = None):
		"""View of Color
		
		Either views a certain colorpickerctrl_id, or creates the
		wx.ColourPickerCtrl itself and attaches it to wx.Unknown with
		colorpickerctrl_id in resource attach_to_resource.
		"""
		View.__init__(self, model, frame)

		if attach_to_resource == None:
			self.ctrl = self.xrcctrl(colorpickerctrl_id)
		else:
			self.ctrl_id = wx.NewId()
			self.ctrl = wx.ColourPickerCtrl(frame, self.ctrl_id)
			attach_to_resource.value.AttachUnknownControl(colorpickerctrl_id, self.ctrl, frame)

	def update(self):
		self.ctrl.SetColour(self.model.as_wx_color())

# HTMLView ##################################################################

class HTMLView(View):
	def __init__(self, model, frame, htmlctrl_id):
		View.__init__(self, model, frame)
		# import wx.html as html #Tak bez tehle radky to nefunguje.. proc, nevim.
		from wx import html
		self.htmlctrl_id = htmlctrl_id
		self.ctrl = self.xrcctrl(htmlctrl_id)
		self.ctrl.SetFonts('Arial', 'Lucida Console')
		self.ctrl.SetFonts('Verdana', 'Lucida Console')

	def update(self):
		# import wx.html as html #Tak bez tehle radky to nefunguje.. proc, nevim.
		from wx import html
		self.ctrl.SetPage(self.model.value)

# ViewGroup #################################################################

class ViewGroup(list):
	def update_all_views(self):
		for v in self:
			if hasattr(v, 'update_all_views'):
				v.update_all_views()
			if hasattr(v, 'update'):
				v.update()
	
	def all_models_updated(self):
		for v in self:
			if not v.update_ok:
				return False
		return True

	def notify_validation(self, ok):
		for v in self:
			v.notify_validation(ok)

	def done(self):
		for v in self:
			v.done()

#############################################################################
# Controllers ###############################################################
#############################################################################

class Controller(WxCommon):
	def __init__(self, model, view):
		self.model, self.view, self.frame = model, view, view.frame

	def done(self):
		pass

# SingleValueEditController #################################################

class SingleValueEditController(Controller):
	def __init__(self, model, view):
		Controller.__init__(self, model, view)
		self.view.ctrl.Bind(wx.EVT_TEXT, self.update_model)
		self.view.ctrl.Bind(wx.EVT_KILL_FOCUS, self.update_view)

	def update_view(self, evt):
		self.view.update()

	def update_model(self, evt):
		s = self.view.get()
		try:
			value = self.convert(s)
		except:
			self.view.notify_update(False)
			return False
		self.view.notify_update(True)
		self.view.deafness += 1
		self.model.value = value
		self.view.deafness -= 1
		return True
	
# StrEditController #########################################################

class StrEditController(SingleValueEditController):
	def convert(self, s):
		return s

# IntEditController #########################################################

class IntEditController(SingleValueEditController):
	def convert(self, s):
		return int(s)
	
# FloatEditController #######################################################

class FloatEditController(SingleValueEditController):
	def convert(self, s):
		return float(s)

# CheckController ###########################################################

class CheckController(Controller):
	def __init__(self, model, view):
		Controller.__init__(self, model, view)
		self.view.ctrl.Bind(wx.EVT_CHECKBOX, self.update_model)

	def update_view(self, evt):
		self.view.update()

	def update_model(self, evt):
		self.model.value = self.view.ctrl.GetValue()
		return True
	
# RadioController ###########################################################

class RadioController(Controller):
	def __init__(self, model, view):
		Controller.__init__(self, model, view)
		self.view.ctrl.Bind(wx.EVT_RADIOBOX, self.update_model)

	def update_view(self, evt):
		self.view.update()

	def update_model(self, evt):
		self.model.set_idx(self.view.ctrl.GetSelection())
		return True
	
# ListController ############################################################

class ListController(Controller):
	def __init__(self, model, view,
		         but_up, but_down, but_add, but_del, but_edit,
				 select_item_def_func, edit_definitions):
		"""

		select_item_def_func: should return a new object with definiton of
			item's GUI (TODO doc)

		edit_definitions: dict {item_class: edit_function, ...}
			edit_function(self, item): return: wx.ID_OK when changes are to be
			accepted, wx.ID_CANCEL when not.
		"""

		Controller.__init__(self, model, view)

		self.but_up, self.but_down = but_up, but_down
		self.but_add, self.but_del, self.but_edit = but_add, but_del, but_edit
		self.select_item_def_func = select_item_def_func
		self.edit_definitions = edit_definitions

		for b in 'but_up', 'but_down', 'but_add', 'but_del', 'but_edit':
			but_id = getattr(self, b)
			if but_id == None:
				continue
			self.bind(wx.EVT_BUTTON, getattr(self, 'on_'+b), but_id)
		self.bind(wx.EVT_LIST_ITEM_ACTIVATED, self.on_but_edit, self.view.listctrl_id)
		self.focused_item = self.view.focused_item

	def on_but_up(self, evt):
		items = self.model
		idx = self.focused_item.value
		if idx == -1 or idx == 0:
			return

		self.focused_item.freeze_changes()
		items.freeze_changes()
		items[idx], items[idx-1] = items[idx-1], items[idx]
		items.thaw_changes()

		self.focused_item.value = idx - 1
		self.focused_item.thaw_changes()

	def on_but_down(self, evt):
		items = self.model
		idx = self.focused_item.value
		if idx == -1 or idx == len(items) - 1:
			return

		self.focused_item.freeze_changes()
		items.freeze_changes()
		items[idx], items[idx+1] = items[idx+1], items[idx]
		items.thaw_changes()

		self.focused_item.value = idx + 1
		self.focused_item.thaw_changes()

	def create_new_item(self):
		item_def = self.select_item_def_func()
		if item_def == None:
			return None
		return item_def.template.copy()

	def on_but_add(self, evt):
		new_item = self.create_new_item()
		if new_item == None:
			return
		if wx.ID_OK == self._edit_item(new_item):
			self.model.insert(self.focused_item.value + 1, new_item)

	def on_but_del(self, evt):
		if self.focused_item.value == -1:
			return
		self.pop_current_item()
			
	def _edit_item(self, item):
		try:
			edit_func = self.edit_definitions[type(item)]
		except KeyError:
			return wx.ID_CANCEL
		return edit_func(item, self.frame)

	def on_but_edit(self, evt):
		focused = self.focused_item.value
		if focused == -1:
			return
		new_item = self.model[focused].copy()
		if wx.ID_OK == self._edit_item(new_item):
			self.model[focused] = new_item

	def pop_current_item(self):
		try:
			self.model.pop(self.focused_item.value)
		except IndexError:
			pass

# ArrayController ############################################################

class ArrayController(ListController):

	def __init__(self, model, view, but_up, but_down, but_add, but_del, but_edit, edit_func):
		it = model.item_template
		ListController.__init__(self, model, view, but_up, but_down, but_add, but_del, but_edit, 
		                        it.copy, {type(it): edit_func})

	def create_new_item(self):
		return self.model.create_new_item()

# ColorController ###########################################################

class ColorController(Controller):
	def __init__(self, model, view):
		Controller.__init__(self, model, view)
		self.view.ctrl.Bind(wx.EVT_COLOURPICKER_CHANGED, self.update_model)

	def update_view(self, evt):
		self.view.update()

	def update_model(self, evt):
		self.model.set(self.view.ctrl.GetColour())
		return True
	
#############################################################################
# Editors ###################################################################
#############################################################################

# Editor ####################################################################

class Editor(object):
	def set_controller(self, controller):
		self.__controller = controller
		if hasattr(self.__controller, 'update_model'):
			self.update_model = self.__controller.update_model

# StrEditor #################################################################

class StrEditor(StrView, Editor):
	def __init__(self, model, frame, xrcid, format = u"%s"):
		StrView.__init__(self, model, frame, xrcid, format)
		self.set_controller(StrEditController(model, self))

	def write_text(self, text):
		"""Writes text to current position"""
		self.ctrl.WriteText(uni(text))

# IntEditor #################################################################

class IntEditor(IntView, Editor):
	def __init__(self, model, frame, xrcid, format = u"%i"):
		IntView.__init__(self, model, frame, xrcid, format)
		self.set_controller(IntEditController(model, self))

# FloatEditor ###############################################################

class FloatEditor(FloatView, Editor):
	def __init__(self, model, frame, xrcid, format = u"%f"):
		FloatView.__init__(self, model, frame, xrcid, format)
		self.set_controller(FloatEditController(model, self))

# CheckEditor ###############################################################

class CheckEditor(CheckView, Editor):
	def __init__(self, model, frame, xrcid):
		CheckView.__init__(self, model, frame, xrcid)
		self.set_controller(CheckController(model, self))

# RadioEditor ###############################################################

class RadioEditor(RadioView, Editor):
	def __init__(self, model, frame, xrcid):
		RadioView.__init__(self, model, frame, xrcid)
		self.set_controller(RadioController(model, self))

# FilenameBrowseEditor ########################################################

class FilenameBrowseEditor(StrEditor):
	def __init__(self, model, frame, textctrl_id, browse_button_id = None,
	             dir = '', prompt = '', wildcard = '', callback = None):
		StrEditor.__init__(self, model, frame, textctrl_id)
		if browse_button_id != None:
			self.fd = FileDialogs(self.frame, dir, '', prompt, prompt, wildcard)
			self.browse_button_id = browse_button_id
			self.bind(wx.EVT_BUTTON, self.on_button_pressed, self.browse_button_id)
			if callback:
				self.callback = callback
			else:
				self.callback = self.__dummy_callback

	def __dummy_callback(self, *args, **kwargs):
		pass

	def on_button_pressed(self, evt):
		path_fname = self.fd.open_dialog()
		if not path_fname:
			return
		self.model.value = path_fname[0]
		self.callback()

# MulticolArrayEditor #######################################################

def list_str():
	def f(list_item, model):
		list_item.SetText(uni(model.value))
	return f

def list_text(format, attr):
	def f(list_item, model):
		list_item.SetText(uni(format) % getattr(model, attr).value)
	return f

def list_text_novalue(format, attr):
	def f(list_item, model):
		list_item.SetText(uni(format) % uni(getattr(model, attr)))
	return f

def list_bool(attr, yes_str = u"Ano", no_str = u"Ne"):
	def f(list_item, model):
		if getattr(model, attr).value == True:
			list_item.SetText(yes_str)
		else:
			list_item.SetText(no_str)
	return f

def list_datetime(attr):
	def f(list_item, model):
		list_item.SetText(uni(getattr(model, attr).str_datetime()))
	return f

def list_time(attr):
	def f(list_item, model):
		list_item.SetText(uni(getattr(model, attr).str_time()))
	return f

class MulticolArrayEditor(MulticolArrayView, Editor):
	def __init__(self, model, frame, listctrl_id, focused_item, cols, 
			display_definition,
			but_up, but_down, but_add, but_del, but_edit, edit_item_func):
		MulticolArrayView.__init__(self, model, frame, listctrl_id, cols,
		                           display_definition, focused_item)
		self.set_controller(ArrayController(model, self,
		                    but_up, but_down, but_add, but_del, but_edit, 
							edit_item_func))

# ChoiceEditor ################################################################

class ChoiceEditor(View):
	"""displays ArrayModel in choice control
	
	If model is None, it assumes that the choices have already been filled.
	This is handy for just getting/setting focused_item.
	"""
	def __init__(self, model, frame, choicectrl_id, str_func, focused_item = None):
		View.__init__(self, model, frame)
		self.choicectrl_id = choicectrl_id
		self.ctrl = self.xrcctrl(choicectrl_id)
		self.str_func = str_func
		if focused_item == None:
			self.focused_item = IntModel(-1)
		else:
			self.focused_item = focused_item
		self.focused_item.add_change_observer(self.on_focused_changed)
		self.bind(wx.EVT_CHOICE, self.on_evt_focused, self.choicectrl_id)

	def insert_item(self, idx, item):
		self.ctrl.Insert(idx, self.str_func(item))

	def append_item(self, item):
		self.ctrl.Append(self.str_func(item))

	def update(self):
		if self.model == None:
			return
		last_focused = self.focused_item.value
		while not self.ctrl.IsEmpty():
			self.ctrl.Delete(0)
		for i in self.model.items:
			self.append_item(i)
		if last_focused < 0:
			last_focused = 0
		if last_focused >= len(self.model) - 1:
			last_focused = len(self.model) - 1
		self.focused_item.value = last_focused

	def on_focused_changed(self):
		if self.focused_item.value == -1:
			self.ctrl.SetSelection(wx.NOT_FOUND)
			return
		self.ctrl.SetSelection(self.focused_item.value)

	def on_evt_focused(self, evt):
		selected = self.ctrl.GetCurrentSelection()
		if selected == wx.NOT_FOUND:
			selected = -1
		if selected != self.focused_item.value:
			self.focused_item.value = selected

	def done(self):
		View.done(self)
		self.focused_item.del_change_observer(self.on_focused_changed)

# ColorEditor #################################################################

class ColorEditor(ColorView, Editor):
	def __init__(self, model, frame, colorpickerctrl_id = None, attach_to_resource = None):
		ColorView.__init__(self, model, frame, colorpickerctrl_id, attach_to_resource)
		self.set_controller(ColorController(model, self))

# Note/Choice/ListBookEditor #################################################

class BookEditor(View):
	"""common editor for wx.Notebook, wx.Listbook, wx.Choicebook
	
	model is an IntModel representing currently selected page or -1 when none
	is selected.
	"""
	event_page_changed = None
	def __init__(self, model, frame, book_id):
		View.__init__(self, model, frame)
		self.ctrl = self.xrcctrl(book_id)
		self.bind(self.event_page_changed, self.on_evt_page_changed, book_id)

	def update(self):
		if self.model.value != self.ctrl.GetSelection():
			self.ctrl.SetSelection(self.model.value)

	def on_evt_page_changed(self, evt):
		self.model.value = evt.GetSelection()

class NotebookEditor(BookEditor):
	event_page_changed = wx.EVT_NOTEBOOK_PAGE_CHANGED

class ChoicebookEditor(BookEditor):
	event_page_changed = wx.EVT_CHOICEBOOK_PAGE_CHANGED

class ListbookEditor(BookEditor):
	event_page_changed = wx.EVT_LISTBOOK_PAGE_CHANGED

#############################################################################
# Other #####################################################################
#############################################################################

# ListMVCBuilder ############################################################

class ListMVCBuilder(object):
	def __init__(self, item_definitions):
		self.item_definitions = item_definitions

	def build_model(self):
		return ListModel([id.template for id in self.item_definitions])

	def build_view(self, model, frame, listctrl_id,
	               column_definitions = None, focused_item = None):
		v = ListView(model, frame, listctrl_id, focused_item)
		for id in self.item_definitions:
			v.add_display_definition(type(id.template), id.column_display_defs)
		if column_definitions:
			v.set_columns(column_definitions)
		return v

	@classmethod
	def select_item_def(cls, title, msg, item_definitions):
		if len(item_definitions) < 1:
			result = None
		elif len(item_definitions) == 1:
			result = 0
		else:
			result = single_choice_dlg(title, msg, [id.title for id in item_definitions])
		if result == None:
			return None
		return item_definitions[result]

	def build_controller(self, model, view,
		         but_up, but_down, but_add, but_del, but_edit,
				 new_item_dlg_title, new_item_dlg_msg):

		def select_item_def_func():
			return self.select_item_def(new_item_dlg_title, new_item_dlg_msg, self.item_definitions)

		edit_definitions = {}
		for id in self.item_definitions:
			edit_definitions[type(id.template)] = id.edit_dlg

		c = ListController(model, view, 
		                   but_up, but_down, but_add, but_del, but_edit,
						   select_item_def_func, edit_definitions)
		return c

# ArrayItemCopier ###########################################################

class ArrayItemCopier(object):
	def __init__(self, array, focused_item, item_to_set):
		self.array, self.focused_item, self.item_to_set = array, focused_item, item_to_set
		self.focused_item.add_change_observer(self.on_focused_changed)

	def on_focused_changed(self):
		try:
			i = self.array[self.focused_item.value]
		except IndexError:
			return
		self.item_to_set.copy_from(i)

# log_model_errors ##########################################################

def log_model_errors(model):
	item = model.validation.item
	log_error(u'Chyba pi kontrole: ' + model.validation.str)
	if item:
		log_model_errors(item)

# report_validation_problem #################################################

def report_validation_problem(model, view_group = None):
	def find_model_view(model, view_group):
		for v in view_group:
			if v.model == model:
				return v
		return None

	def find_deepest_model_view(model, view_group):
		item = model.validation.item
		if item != None:
			mv = find_deepest_model_view(item, view_group)
			if mv:
				return mv
		v = find_model_view(model, view_group)
		if v:
			return(model, v)
		return None

	if model.validation.valid:
		return

	view_group.notify_validation(True)

	mv = find_deepest_model_view(model, view_group)
	if mv == None:
		log_model_errors(model)
	else:
		m, v = mv
		v.notify_validation(False)
		v.set_focused()
		log_model_errors(m)

#############################################################################
# Frames, Dialogs ###########################################################
#############################################################################

# FileDialogs ###############################################################

class FileDialogs(WxCommon):
	def __init__(self, frame, dir, full_path, open_prompt, save_prompt, wildcard):
		self.frame = frame
		self.dir, self.fname = dir, ''
		if full_path and full_path != '':
			self.set_full_path(full_path)
		self.open_prompt = open_prompt
		self.save_prompt = save_prompt
		self.wildcard = wildcard

	def set_full_path(self, path):
		self.dir, self.fname = os.path.split(path)

	def get_full_path(self):
		if not self.have_fname():
			return None
		return os.path.join(self.dir, self.fname)

	def have_fname(self):
		return self.fname != ''

	def file_dialog(self, msg, dir, type, fname, wildcard):
		wc = u'|'.join([wildcard, u"Vechny soubory(*.*)|*.*"])
		dlg = wx.FileDialog(self.frame, msg, dir, fname, wc, type)
		result = dlg.ShowModal()
		fname = dlg.GetFilename()
		path = dlg.GetPath()
		dlg.Destroy() 
		if result == wx.ID_OK:
			return (path, fname)
		return None
	
	def open_dialog(self):
		return self.file_dialog(self.open_prompt, self.dir, wx.OPEN,
		                        self.fname, self.wildcard)

	def save_dialog(self):
		return self.file_dialog(self.save_prompt, self.dir, 
		       wx.SAVE | wx.OVERWRITE_PROMPT, self.fname, self.wildcard)

	def ask_yes_no_cancel(self, title=u"Zavt dokument"):
		if not self.fname:
			fname = ""
		else:
			fname = self.fname
		dlg = wx.MessageDialog(self.frame, 
			u"Dokument %s byl zmnn. Chcete jej uloit?" % fname, 
			title, wx.ICON_QUESTION | wx.YES_NO | wx.CANCEL | wx.YES_DEFAULT)
		result = dlg.ShowModal()
		dlg.Destroy()
		return { wx.ID_YES:    'save', 
		         wx.ID_NO:     'dontsave', 
		         wx.ID_CANCEL: 'cancel',
		       }[result]

# TopLevelWindow ############################################################

class TopLevelWindow(WxCommon):
	frame_title = None
	#icon = None
	icon = 'icon.ico'
	def __init__(self, close_handler = None):
		self.view_group = ViewGroup()
		self._close_handler = close_handler
		self.shown = False
		self.bind(wx.EVT_CLOSE, self._on_close, None)
		self.return_code = None
		self.fullscreen = False
		if self.frame_title != None:
			self.frame.SetTitle(self.frame_title)
			self.orig_frame_title = self.frame_title
		else:
			self.orig_frame_title = self.frame.GetTitle()

		if self.icon != None:
			from registry import paths
			i = wx.Icon(os.path.join(paths.images, self.icon), wx.BITMAP_TYPE_ICO)
			if i.IsOk():
				self.frame.SetIcon(i)

	def set_return_code(self, return_code):
		self.return_code = return_code
		if isinstance(self, Dialog):
			self.frame.SetReturnCode(return_code)

	def on_run(self):
		pass

	def on_destroy(self):
		pass
	
	def add_view(self, view):
		self.view_group.append(view)

	def valid_or_report_error(self):
		self.view_group.notify_validation(True)
		self.model.validate()
		if self.model.validation.valid:
			return True
		report_validation_problem(self.model, self.view_group)
		return False

	def destroy(self):
		self.shown = False
		self.on_destroy()
		self.view_group.done()
		self.frame.Destroy()
		from registry import open_subwindows
		open_subwindows.dec()

	def can_close(self):
		return True

	def _on_close(self, evt):
		if evt.CanVeto() and hasattr(self, 'can_close') and not self.can_close():
			evt.Veto()
			return

		self.set_return_code(wx.ID_CANCEL)
		self.destroy()

	def set_sizes(self):
		self.frame.Fit()
		self.frame.Layout()
		# Not sure what's wrong, but sometimes it helps to layout children explicitly.
		for c in self.frame.GetChildren():
			c.Layout()

	def _get_default_attach_window_parent(self):
		return self.frame

# Dialog ####################################################################

class Dialog(TopLevelWindow):
	def __init__(self, model, xml_resources, xrc_name, parent = None):
		self.model = model
		self.frame = xml_resources.value.LoadDialog(parent, xrc_name)
		self.parent = parent
		if not self.frame:
			log_error(u"Error loading dialog %s" % xrc_name)
			raise EnvironmentError(u"Error loading dialog")

		self.bind(wx.EVT_BUTTON, self.on_ok, 'But_OK')
		self.bind(wx.EVT_BUTTON, self.on_cancel, 'But_Cancel')
		TopLevelWindow.__init__(self)

	def run_prepare(self):
		if False == self.on_run():
			return False
		self.view_group.update_all_views()
		self.shown = True
		from registry import open_subwindows
		open_subwindows.inc()
		self.set_sizes()
		return True

	def run_modal(self):
		if not self.run_prepare():
			return None
		retcode = self.frame.ShowModal()
		# doesn't work
		#if self.fullscreen:
		#	self.frame.ShowFullScreen(True, style = 0)
		self.set_return_code(retcode)
		return retcode

	def run_modeless(self):
		if not self.run_prepare():
			return None
		retcode = self.frame.Show()
		# doesn't work
		# if self.fullscreen:
		# 	self.frame.ShowFullScreen(True, style = 0)
		return retcode

	def end(self, return_code):
		if self.frame.IsModal():
			return self.end_modal(return_code)
		else:
			return self.end_modeless(return_code)

	def end_modeless(self, return_code):
		if not self.can_close():
			return 
		self.frame.SetReturnCode(return_code)
		self.frame.Hide()
		self.destroy()

	def end_modal(self, return_code):
		if not self.can_close():
			return 
		self.set_return_code(return_code)
		self.frame.EndModal(return_code)
		self.destroy()

	def on_ok(self, evt):
		if self.pre_ok() == False:
			return
		if not self.view_group.all_models_updated():
			log_error(u"Vechny poloky mus bt korektn vyplnny")
			return
		if self.valid_or_report_error():
			self.post_ok()
			self.end(wx.ID_OK)

	def pre_ok(self):
		pass

	def post_ok(self):
		pass

	def on_cancel(self, evt):
		self.end(wx.ID_CANCEL)

	@classmethod
	def run_dialog(cls, model, parent = None):
		# TODO: ugly code, works only with specially overriden __init__, blewhg
		return cls(model, parent).run_modal()

	@classmethod
	def run_dialog_modeless(cls, model, parent = None):
		# TODO: ugly code, works only with specially overriden __init__, blewhg
		return cls(model, parent).run_modeless()

# Frame #####################################################################

class Frame(TopLevelWindow):
	def __init__(self, model, xml_resources, xrc_name, parent = None, statusbar = True):
		self.model = model
		self.xml_resources = xml_resources
		self.frame = self.xml_resources.value.LoadFrame(parent, xrc_name)
		if not self.frame:
			log_error(u"Error loading frame %s" % xrc_name)
		self.parent = parent
		if statusbar:
			self.frame.CreateStatusBar()
		TopLevelWindow.__init__(self)

	@classmethod
	def run_frame(cls, model, parent = None):
		# TODO: ugly code, works only with specially overriden __init__, blewhg
		frame = cls(model, parent)
		frame.run()

	def set_menu(self, xrc_menu = None):
		if xrc_menu != None:
			self.menubar = self.xml_resources.value.LoadMenuBar(xrc_menu)
			self.frame.SetMenuBar(self.menubar)
		else:
			self.menubar = None

	def run(self):
		if False == self.on_run():
			return
		self.view_group.update_all_views()
		from registry import open_subwindows
		open_subwindows.inc()
		if self.fullscreen:
			self.frame.ShowFullScreen(True, style = 0)
		else:
			self.set_sizes()
			self.frame.Show()
		self.shown = True

	def close(self, evt = None):
		if not self.can_close():
			return False
		self.frame.Show(False)
		self.destroy()
		return True

	def statustext(self, text):
		sb = self.frame.GetStatusBar()
		if not sb:
			return
		sb.SetStatusText(uni(text))

# OkCancelFrame #############################################################

class OkCancelFrame(Frame):
	def __init__(self, *args, **kwargs):
		Frame.__init__(self, *args, **kwargs)
		self.bind(wx.EVT_BUTTON, self.close, 'But_Cancel')
		self.bind(wx.EVT_BUTTON, self.on_ok, 'But_OK')
		self.ok_callback = None

	def set_ok_callback(self, ok_callback):
		self.ok_callback = ok_callback

	def on_ok(self, evt = None):
		if not self.valid_or_report_error():
			self.statustext(u"Chyba pi kontrole")
			return
		if not self.close():
			return
		if self.ok_callback != None:
			self.ok_callback()

# DocumentFrame #############################################################

class DocumentFrame(Frame):
	"""Frame class with document management (open/save/...) functionality"""
	def bind_file_events(self, file_new = None, file_open = None, 
	                     file_save = None, file_saveas = None, 
						 file_exit = None, bind_close_event = False):

		args = vars()
		for item in ('new', 'open', 'save', 'saveas'):
			xrc_id = args['file_'+item]
			if xrc_id != None:
				self.bind(wx.EVT_MENU, getattr(self, '_on_file_'+item), xrc_id)
		if file_exit:
			self.bind(wx.EVT_MENU,  self.close, file_exit)
			self.bind(wx.EVT_CLOSE, self.close, None)

	def setup_file_dialogs(self, dir, full_path, open_prompt, save_prompt, wildcard):
		self.file_dialogs = FileDialogs(self.frame, dir, full_path, open_prompt, 
		                                save_prompt, wildcard)

	def _on_file_new(self, evt = None):
		raise NotImplementedError

	def _close_document(self):
		if not self.model.modified:
			return True
		action = self.file_dialogs.ask_yes_no_cancel()
		if action == 'dontsave':
			return True
		if action == 'cancel':
			return False
		if action == 'save':
			return self._on_file_save()

	def _save_document(self, full_path):
		try:
			self.do_file_save(full_path)
		except EnvironmentError, e:
			log_environment_error(e)
			self.statustext(u"Chyba pi ukldn souboru %s" % self.file_dialogs.fname)
			return False
		self.file_dialogs.set_full_path(full_path)
		fname = self.file_dialogs.fname
		self.statustext(u"Soubor %s spn uloen" % fname)
		self.frame.SetTitle(self.orig_frame_title + ' - ' + fname)
		return True
		
	def _open_document(self, full_path):
		try:
			self.do_file_open(full_path)
		except EnvironmentError, e:
			log_environment_error(e)
			self.statustext(u"Chyba pi otevrn souboru %s" % self.file_dialogs.fname)
			return False
		retval = self.post_open()
		if retval != False:
			self.file_dialogs.set_full_path(full_path)
			fname = self.file_dialogs.fname
			self.statustext(u"Soubor %s spn oteven" % fname)
			self.frame.SetTitle(self.orig_frame_title + ' - ' + fname)

	def _on_file_open(self, evt = None):
		if not self._close_document():
			return False
		full_path__fname = self.file_dialogs.open_dialog()
		if full_path__fname == None:
			return False
		return self._open_document(full_path__fname[0])

	def _on_file_save(self, evt = None):
		if not self.valid_or_report_error():
			self.statustext(u"Chyba pi kontrole dokumentu")
			return False
		full_path = self.file_dialogs.get_full_path()
		if not full_path:
			return self._on_file_saveas()
		return self._save_document(full_path)

	def _on_file_saveas(self, evt = None):
		if not self.valid_or_report_error():
			self.statustext(u"Chyba pi kontrole dokumentu")
			return False
		full_path__fname = self.file_dialogs.save_dialog()
		if full_path__fname == None:
			return False
		return self._save_document(full_path__fname[0])

	def post_open(self):
		"""This method gets called after successfull file open, user can
		override it. """
		return True

	def can_close(self):
		return self._close_document()

	def do_file_save(self, fname):
		self.model.xml_save(fname)

	def do_file_open(self, fname):
		try:
			self.model.xml_load(fname)
		except XMLModelLoadError, e:
			raise EnvironmentError(u"Soubor %s se nepodailo otevt" % fname)

# TextEntryDialog ###########################################################

class TextEntryDialog(Dialog):
	def __init__(self, model, title, msg, format = None, parent = None):
		from registry import xresources
		resources = xresources.common

		Dialog.__init__(self, model, resources, 'Dlg_TextEntry', parent)
		self.frame.SetTitle(title)
		self.xrcctrl('Label_msg').SetLabel(msg)
		self.frame.Layout()
		self.frame.Fit()

		if format == None:
			format_d = {}
		else:
			format_d = {'format': format}

		if isinstance(self.model, StrModel):
			v = StrEditor(self.model, self.frame, 'Text_Entry', **format_d)
		if isinstance(self.model, IntModel):
			v = IntEditor(self.model, self.frame, 'Text_Entry', **format_d)
		if isinstance(self.model, FloatModel):
			v = FloatEditor(self.model, self.frame, 'Text_Entry', **format_d)
		self.add_view(v)

# LazyXMLResource ###########################################################

class LazyXMLResource(object):
	def __init__(self, fname):
		self.fname = fname
		self._value = None
		
	def _get_value(self):
		if self._value == None:
			self._value = xrc.XmlResource(self.fname, 0)
			if self._value == None:
				log_error(u"Chyba pi nahrvn souboru s definic uivatelskho rozhran"+self.fname)
		return self._value
	value = property(_get_value)

# single_choice_dlg ########################################################

def single_choice_dlg(title, msg, items, parent = None):
		select_dlg = wx.SingleChoiceDialog(parent, msg, title, items)
		result = select_dlg.ShowModal()
		choice = select_dlg.GetSelection()
		select_dlg.Destroy()
		if result != wx.ID_OK:
			return None
		return choice

# ask_yes_no ################################################################

def ask_yes_no(title, msg, parent = None):
	dlg = wx.MessageDialog(parent, msg, title, wx.ICON_QUESTION | wx.YES_NO)
	result = dlg.ShowModal()
	dlg.Destroy()
	if result == wx.ID_YES:
		return True
	return False

# open_dialog ###############################################################

def open_dialog(dir, full_path, prompt, wildcard, parent = None):
	fd = FileDialogs(parent, dir, full_path, prompt, prompt, wildcard)
	path_fname = fd.open_dialog()
	if path_fname == None:
		return None
	return path_fname[0]

# save_dialog ###############################################################

def save_dialog(dir, full_path, prompt, wildcard, parent = None):
	fd = FileDialogs(parent, dir, full_path, prompt, prompt, wildcard)
	path_fname = fd.save_dialog()
	if path_fname == None:
		return None
	return path_fname[0]

# try_save_model ############################################################

def try_save_model(frame, model, prompt = "Vyberte soubor, kam se maj data uloit", 
                   dir = '', full_path = '', wildcard = ''):
	""" Otevre dialog 'ulozit' a pokusi se tam ulozit model.

	vraci: True - ulozeno, False - chyba, None - Zruseno uzivatelem
	"""
	fd = FileDialogs(frame, dir, full_path, prompt, prompt, wildcard)
	path_fname = fd.save_dialog()
	if not path_fname:
		return None
	try:
		model.xml_save(path_fname[0])
	except EnvironmentError, e:
		log_environment_error(e)
		return False
	return True

# popup_menu ################################################################

def popup_menu(window, definition):
	"""Creates a popup menu.

	window: wx.Window instance
	definition: list of (text, callback) items. if text is empty or None, a
	separator is placed instead of it.  
	"""
	menu = wx.Menu()
	for text, callback in definition:
		if not text:
			menu.AppendSeparator()
		else:
			id = wx.NewId()
			window.Bind(wx.EVT_MENU, callback, id = id)
			menu.AppendItem(wx.MenuItem(menu, id, text))
	window.PopupMenu(menu)
