#!/usr/bin/python
# -*- encoding: cp1250 -*-

"""
Modul pro generovani pdf dokumentu (obrazku, html reportu, textu) a
pro jejich tisk.
"""

import os
import re
import subprocess

import wx

from reportlab.platypus import *
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.fonts import addMapping
from reportlab.rl_config import defaultPageSize
from reportlab.lib.units import inch
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont 
from sys import maxint

import Image as PILImage

import misc
import registry
from mvc import log_environment_error

utf8 = misc.utf8

PAGE_WIDTH, PAGE_HEIGHT = defaultPageSize

_title = u""

def _register_font(fontname, fname):
	for suffix, b, i in (('', 0, 0), ('I', 0, 1), ('BD', 1, 0), ('BI', 1, 1)):
		realfontname = fontname + suffix
		name, ext = os.path.splitext(fname)
		path = os.path.join(registry.paths.fonts, name + suffix + ext)
		pdfmetrics.registerFont(TTFont(realfontname, path))
		addMapping(fontname, b, i, realfontname)

_register_font('Arial', 'arial.ttf')
_register_font('Courier', 'cour.ttf')
_register_font('Times', 'times.ttf')
#pdfmetrics.registerFont(TTFont('Symbol', os.path.join(registry.paths.fonts, 'symbol.ttf')))
#addMapping('Symbol', 0, 0, 'symbol.ttf')

def _myFirstPage(canvas, doc):
	canvas.saveState()
	canvas.setFont('ArialI',9)
	canvas.drawString(inch, PAGE_HEIGHT - inch + inch/20, _title)
	canvas.line(inch, PAGE_HEIGHT-inch, PAGE_WIDTH - inch, PAGE_HEIGHT-inch)
	canvas.drawString(inch, 0.75 * inch, u"Strana %i" % doc.page)
	canvas.restoreState()

def _doc_build(doc, elements):
	""" Vytvori xml soubor
	Pokud nastala nejaka chyba, hazi EnvironmentError
	"""
	try:
		doc.build(elements,onFirstPage=_myFirstPage, onLaterPages=_myFirstPage)
	except EnvironmentError, e:
		log_environment_error(e)
		log_error(u"Chyba pi vytven PDF dokumentu")
		return False
	return True

def write_pdf_report(fname, title, report, images = None, rotation = -90):
	"""Vygeneruje pdf soubor s reportem z mereni.

	fname: Jmeno vysledneho pdf souboru
	title: Hlavicka zpravy (na kazde strance v souboru)
	report: HTML verze reportu
	images: Seznam jmen souboru s obrazky, ktere se vlozi na zacatek reportu
		(pokud je None, nevlozi se nic)
	rotation: O kolik stupnu proti smeru hod. rucicek se ma obrazek obratit.
	Pokud nastala nejaka chyba, hazi EnvironmentError
	"""

	global _title

	_title = utf8(title)
	para_style = getSampleStyleSheet()['Normal']

	if False:
		def find_tags(txt, tag, start = 0, end = maxint):
			"""Nalezne v txt text uzavreny mezi <tag> a </tag>. Hleda od start do end.

			Vraci None, pokud nic nenasel. Jinak vraci tuple (txt, start, end), kde
			txt je text MEZI tagy, start je index znaku '<' kterym zacina oteviraci
			tag, a end je index znaku '>' kterym konci uzaviraci tag. **Rozlisuje**
			mala a velka pismena. 
			"""

			_start = txt.find(u'<%s>'%tag, start, end)
			if _start == -1:
				return None
			_end = txt.find(u'</%s>'%tag, _start, end)
			if _end == -1:
				print 'find_tags: No matching closing tag to <%s>!' % tag
				return None
			tag_len = len(tag)
			return (txt[_start+tag_len+2 : _end], _start, _end+tag_len+2)

		def html_table(elems, txt):
			"""Z <table> .... <tr> .. <td> </table> vytvori reportlabovou Table 

			je to jen rychla vec, moc to neumi..

			elems: Sem se hodi vysledna tabulka.
			vsechny tagy musi byt uzavrene (tj. za <tr> musi nasledovat </tr> atd..)
			neumi spans, v kazdem radku musi byt stejne dat. Je to pomerne pomaly :-)
			"""

	def p(elems, txt):
		paragraph_spacing = 0.5
		# Tohle je hack. Reportlab mi neumi zobrazit font Symbol, coz je muj
		# zpusob generovani reckych pismen (omega, sigma) v html. Na druhou
		# stranu ma reportlab specialni tag <greek>, kterym se recka pismena
		# uvozuji. Timhle vyrazem nahradim prislusne font tagy greek tagy.
		# Pokud se ovsem bude font Symbol pouzivat kvuli necemu jinemu nez jsou
		# recka pismena, samozrejme budu opet ve spatne situaci.
		replace_greek = re.compile(r'<font\ face\ *=\ *"Symbol">(.*?)</font>', re.VERBOSE | re.IGNORECASE)
		def split(txt, tag, begin, end):
			return [begin + t + end for t in txt.split(tag)]
		for p in txt.split('<p>'):
			para = replace_greek.sub(r'<greek>\1</greek>', p)
			elems.extend([Paragraph(utf8(l), para_style) for l in
			             split(para, '<br>', '<font face="Arial">', '</font>')])
			elems.append(Spacer(PAGE_WIDTH, paragraph_spacing))

	doc = SimpleDocTemplate(fname)

	w = PAGE_WIDTH - (doc.rightMargin + doc.leftMargin) - inch/10
	h = PAGE_HEIGHT- (doc.topMargin + doc.bottomMargin) - inch/5
	elements = []
	if images == None:
		images = []
	image_fnames = []
	for image in images:
		try:
			if rotation:
				img_fname = misc.tempnam('.png')
				image_fnames.append(img_fname)
				PILImage.open(image).rotate(rotation).save(img_fname)
			else:
				img_fname = image
			img = Image(img_fname, width = w, height = h)
			elements.append(img)
			elements.append(PageBreak())
		except EnvironmentError, e:
			log_environment_error(e)
			log_error(u"Chyba pi zpracovn obrzku")
			return False

	p(elements, report)
	retval = _doc_build(doc, elements)

	for i in image_fnames:
		misc.safe_remove(i)

	return retval

def write_pdf_ascii(fname, title, text_file):
	"""Vygeneruje pdf soubor z ASCII dat

	fname: Jmeno vysledneho pdf souboru
	title: Hlavicka zpravy (na kazde strance v souboru)
	text_file: soubor s ASCII textem ktery se ma prevest do pdf
	Pokud nastala nejaka chyba, hazi EnvironmentError
	"""
	global _title

	try:
		f = open(text_file, 'r')
		text = f.read()
		f.close()
	except EnvironmentError, e:
		log_environment_error(e)
		log_error(u"Chyba pi ten souboru s textem")
		return False

	_title = utf8(title)
	para_style = getSampleStyleSheet()['Code']
	doc = SimpleDocTemplate(fname)
	elems = [Preformatted(utf8(text), para_style)]
	return _doc_build(doc, elems)

def write_pdf_image(fname, title, image, rotation = -90):
	"""Vygeneruje pdf soubor s obrazkem.

	fname: Jmeno vysledneho pdf souboru
	title: Hlavicka zpravy
	image: Jmeno souboru s obrazkem, ktery se vlozi na prvni stranku
	rotation: O kolik stupnu proti smeru hod. rucicek se ma obrazek obratit.
	Pokud nastala nejaka chyba, hazi EnvironmentError
	"""
	global _title

	_title = utf8(title)

	doc = SimpleDocTemplate(fname)

	w = PAGE_WIDTH - (doc.rightMargin + doc.leftMargin) - inch/10
	h = PAGE_HEIGHT- (doc.topMargin + doc.bottomMargin) - inch/5
	elements = []
	try:
		if rotation:
			img_fname = misc.tempnam('.png')
			PILImage.open(image).rotate(rotation).save(img_fname)
		else:
			img_fname = image
		img = Image(img_fname, width = w, height = h)
		elements.append(img)
	except EnvironmentError, e:
		log_environment_error(e)
		log_error(u"Chyba pi vytven PDF dokumentu")
		return False

	retval = _doc_build(doc, elements)

	if image != img_fname:
		misc.safe_remove(img_fname)

	return retval

def pdf_print(fname):
	"""Vytiskne PDF dokument.

	pouziva acrobat reader s parametrem /p.
	cestu k readeru bere z Settings
	Pokud nastala nejaka chyba, hazi EnvironmentError
	"""
	s = registry.settings.value
	acrord = s.acrobat_reader.value

	if not os.path.exists(acrord):
		log_error(u"Neplatn cesta k programu Acrobat Reader. Zkontrolujte cestu v dialogu Nastaven.")
		return False
	if not os.path.exists(fname):
		log_error(u"soubor %s neexistuje." % fname)
		return False

	try:
		#sp = subprocess.Popen([acrord, fname]) # Netiskne, jen otevre Reader
		sp = subprocess.Popen([acrord, '/p', fname])
	except EnvironmentError, e:
		log_environment_error(e)
		log_error(u"Chyba pi spoutn programu Acrobat Reader.")
		return False
