#!/usr/bin/python
# -*- encoding: cp1250 -*-

import os
import wx
from wx import xrc
import mvc
import calibration as calibration_def
import settings as settings_module
from misc import Counter

# Zde je potreba nastavit cestu k datovemu adresari goniofotometru
#data_path = r'c:\projects\gofomo\app\current\data'
data_path = 'c:\gfm\data'

# Appname, version ##########################################################

version = u"0.4"
appname = u"Goniofotometr %s" % version
author = u"Tom Dvok"

# open_subwindows ###########################################################

class _OpenSubwindows(Counter):
	def all_closed(self):
		return self.is_zero()

open_subwindows = _OpenSubwindows()

# wildcards #################################################################

class _Wildcards(object):
	all = ''
	xml = "Soubory XML(*.xml)|*.xml"
	pdf = "Soubory PDF(*.pdf)|*.pdf"
	pickle = "Soubory Pickle(*.pkl)|*.pkl"
	exe = "Spustiteln soubory(*.exe)|*.exe"
	txt = "Textov soubory(*.txt)|*.txt"
	pictures = "Soubory PNG(*.png)|*.png"

wildcards = _Wildcards()

# paths #####################################################################

class _Paths(object):
	fonts = os.path.join(data_path, 'fonts')
	images = os.path.join(data_path, 'images')
	config = os.path.join(data_path, 'konfigurace')
	calibration_matrix = os.path.join(data_path, 'mereni', 'kalibracni_matice')
	logs = os.path.join(data_path, 'logs')

	def __init__(self):
		_typetest_dirs = (
				('def', 'definice'), 
				('data', 'mereni'), 
				('eval', 'vyhodnoceni'), 
				('protocols', 'protokoly'),
			)
		_typetest_types = (
				('directional', 'smerova'),
				('approach', 'sestupova'),
				('table', 'tabulkova'),
				('omni', 'vsesmerova'),
				('light', 'svitidla'),
				)

		for (var_dir, path_dir) in _typetest_dirs:
			for (var_type, path_type) in _typetest_types: 
				var = 'typetests_%s_%s' % (var_dir, var_type) # napr. 'typetests_def_table'
				path = os.path.join(data_path, path_dir, path_type)
				setattr(self, var, path)

				var = 'devel_%s' % var_type # napr. 'devel_table'
				path = os.path.join(data_path, 'vyvoj', path_type)
				setattr(self, var, path)
paths = _Paths()

# fnames ####################################################################

class _FNames(object):
	settings = os.path.join(paths.config, 'nastaveni.xml')
	runtime_settings = os.path.join(paths.config, 'behove_nastaveni.xml')
	calibration_table = os.path.join(paths.config, 'kalibrace-stul.xml')
	calibration_arm = os.path.join(paths.config, 'kalibrace-rameno.xml')
	luxmeters = os.path.join(paths.config, 'luxmetry.xml')
	img_logo = os.path.join(paths.images, 'logo.bmp')

fnames = _FNames()

# xresources ################################################################

class _XResources(object):
	calibration =      mvc.LazyXMLResource('calibration_gui.xrc')
	common =           mvc.LazyXMLResource('common_dialogs.xrc')
	eval =             mvc.LazyXMLResource('eval_typetest_gui.xrc')
	main =             mvc.LazyXMLResource('main_gui.xrc')
	measure =          mvc.LazyXMLResource('measure_gui.xrc')
	settings =         mvc.LazyXMLResource('settings_gui.xrc')
	typetests =        mvc.LazyXMLResource('typetests_gui.xrc')
	ckdm_interface =   mvc.LazyXMLResource('ckdm_interface_gui.xrc')
	viewdata =         mvc.LazyXMLResource('viewdata_gui.xrc')
	devel =            mvc.LazyXMLResource('devel_gui.xrc')
	meas_corrections = mvc.LazyXMLResource('meas_corrections_gui.xrc')
	measure_typetest = mvc.LazyXMLResource('measure_typetest_gui.xrc')
	logging =          mvc.LazyXMLResource('error_logging.xrc')

xresources = _XResources()

# calibration ###############################################################

class _LazyModel(object):
	def __init__(self, model, filename, description = ''):
		self.model = model
		self.filename, self.description = filename, description
		self.loaded = False
		
	def _get_value(self):
		if self.loaded:
			return self.model

		# Nejprve musim model nacist ze souboru
		try:
			self.model.xml_load(self.filename)
		except (mvc.XMLModelLoadError, EnvironmentError):
			log_error(u"%s: Chyba pi natn ze souboru %s" % (self.description, self.filename))
			return self.model

		self.model.validate()
		if not self.model.validation.valid:
			mvc.log_model_errors(self.model)
			return self.model

		self.loaded = True
		return self.model

	value = property(_get_value)

	def save(self):
		try:
			self.model.xml_save(self.filename)
		except EnvironmentError, e:
			log_environment_error(e)

calibration_table = _LazyModel(calibration_def.CalibrationTable(), fnames.calibration_table, u"Kalibrace stolu")
calibration_arm = _LazyModel(calibration_def.CalibrationArm(), fnames.calibration_arm, u"Kalibrace ramene")
luxmeters = _LazyModel(calibration_def.Luxmeters(), fnames.luxmeters, u"Seznam luxmetr")
settings = _LazyModel(settings_module.Settings(), fnames.settings, u"Nastaven programu")
runtime_settings = _LazyModel(settings_module.RuntimeSettings(), fnames.runtime_settings, u"Bhov nastaven programu")
