# -*- encoding: cp1250 -*-

from matplotlib import cm

from mvc import *

import debug

available_colormaps = cm.cmapnames

# LumiTheme #################################################################

class LumiTheme(CompositeModel):
	_models = { # Defaultni tema je kost.
		'name': StrModel('Kost'),
		'theme': StrModel('bone'),
		'reversed': BoolModel(False),
	}

	def get_colormap(self):
		if not self.reversed.value:
			cmapname = self.theme.value
		else:
			cmapname = self.theme.value + '_r'
		try:
			return getattr(cm, cmapname)
		except AttributeError:
			return None

	def validator(self):
		if self.name.value == '':
			self.validator_failure("Nzev tmatu nesm bt przdn", self.name)
			return

		if self.get_colormap() == None:
			self.validator_failure("Neplatn tma luminogramu.", self.theme)
			return

# Settings ##################################################################

class Settings(CompositeModel):
	"""Nastaveni provadena uzivatelem"""
	_models = {
		'version': VersionModel('1.0'),
		'lumi_themes': ArrayModel(LumiTheme()),
		'acrobat_reader': FilenameModel(''),
		'images_dpi': IntModel(200), # dots per inch pro grafy v tisku a exportu
		'ckdm_com': StrEnumModel(('COM1', 'COM2', 'COM3', 'COM4'), 'COM1'),
	}

	def validator(self):
		if len(self.lumi_themes) == 0:
			self.validator_failure("Seznam tmat luminogramu nesm bt przdn", self.lumi_themes)
		elif not 10 <= self.images_dpi.value <= 3000:
			self.validator_failure("Rozlien obrzk mus bt v rozsahu 10..3000 dpi", self.images_dpi)

# RuntimeSettings ###########################################################

class RuntimeSettings(CompositeModel):
	"""Nastaveni ktere se meni za behu programu
	
	Veci ktere se meni za behu programu a mely by pretrvat mezi jednotlivymi
	spustenimi. Uzivatel ani nemusi moc vedet, ze nejake takove jsou.
	"""
	_models = {
		'version': VersionModel('1.0'),
		# Posledni nulova poloha stolu
		'last_rot': FloatModel(0.0),
		'last_tilt': FloatModel(0.0),
	}
	
	def validate(self):
		from table_math import TableMath
		if abs(self.last_rot.value) > TableMath.ROT_LIMIT:
			self.last_rot.value = 0.0
		if abs(self.last_tilt.value) > TableMath.TILT_LIMIT:
			self.last_tilt.value = 0.0
		return CompositeModel.validate(self)
