# -*- encoding: cp1250 -*-

import string
import os

import wx

from mvc import *
import registry
import settings
import plot

resources = registry.xresources.settings

def preview_plot():
		p = plot.PlotModel()
		cols = [x*0.1 for x in xrange(11)]
		rows = [x*0.1 for x in xrange(11)]
		p.mesh.create_from_function(cols, rows, lambda o, s: s)
		p.extent.set_lbwh(0.0, 0.0, 1.0, 1.0)
		p.grid.set_grid_fixed(False, [], []) 
		p.title.value = p.xlabel.value = p.ylabel.value = u""
		p.lumi_on.value = True
		p.colorbar_on.value = False
		p.colorbar_title.value = u"Svtivost"
		return p

# LumiThemeDlg ##############################################################

class LumiThemeDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_LumiTheme', parent)

		m, f = self.model, self.frame

		self.plot = preview_plot()
		self.add_view(plot.PlotView(self.plot, f, resources, 'Plot_preview'))

		available_colormaps = ArrayModel(StrModel())
		for cm in settings.available_colormaps:
			available_colormaps.append(StrModel(cm))
		try:
			theme_idx = settings.available_colormaps.index(m.theme.value)
		except ValueError:
			theme_idx = 0
		focused_cm = IntModel(theme_idx)
		self.add_view(ChoiceEditor(available_colormaps, f, 'Choice_colormap',
		                         lambda cm: cm.value, focused_cm))
		self._copy_cm = ArrayItemCopier(available_colormaps, focused_cm, m.theme)

		self.add_view(StrEditor(m.name, f, 'Text_name'))
		self.add_view(CheckEditor(m.reversed, f, 'Check_reversed'))

		self.model.theme.add_change_observer(self.on_changed)
		self.model.reversed.add_change_observer(self.on_changed)

	def on_changed(self):
		self.plot.lumi_theme.copy_from(self.model)

	def on_destroy(self):
		self.model.theme.del_change_observer(self.on_changed)
		self.model.reversed.del_change_observer(self.on_changed)

# SettingsFrame #############################################################

class SettingsFrame(DocumentFrame):
	def __init__(self, model, parent = None):
		DocumentFrame.__init__(self, model, resources, 'Frame_Settings', parent)

		m, f = self.model, self.frame

		# Zalozka temata luminogramu
		display_defs = {
			'name': (u"nzev", 100, list_text('%s', 'name')),
			'theme': (u"tma", 100, list_text('%s', 'theme')),
			'reversed': (u"obrcen", 100, list_bool('reversed')),
		}
		focused_theme = IntModel(0)
		self.add_view(
			MulticolArrayEditor(m.lumi_themes, f, 'List_lumi_themes', 
			focused_theme, ('name', 'theme', 'reversed'), display_defs, 
			'But_up', 'But_down', 'But_add', 'But_del', 'But_edit', 
			LumiThemeDlg.run_dialog)
		)

		self.plot = preview_plot()
		self.add_view(plot.PlotView(self.plot, f, resources, 'Plot_preview'))
		self._copy_cm = ArrayItemCopier(m.lumi_themes, focused_theme, self.plot.lumi_theme)

		# Zalozka Export a tisk
		self.add_view(FilenameBrowseEditor(
			m.acrobat_reader, f, 'BrowseText_acrobat_reader', 'BrowseBut_acrobat_reader',
			dir = 'C:\Program Files', prompt = u"Cesta k programu Acrobat Reader", 
			wildcard = registry.wildcards.exe))
		self.add_view(IntEditor(m.images_dpi, f, 'Text_images_dpi'))

		# Zalozka komunikace
		self.add_view(RadioEditor(m.ckdm_com, f, 'Radio_ckdm_com'))

		# Menu
		self.set_menu('MB_Settings')
		self.bind_file_events(None, 'MI_Open', 'MI_Save', 'MI_SaveAs', 'MI_Quit')
		self.setup_file_dialogs(registry.paths.config, registry.fnames.settings, 
			u"Otevt soubor s nastavenm", "Uloit soubor s nastavenm", 
			registry.wildcards.xml)

# gui_settings ##############################################################

def gui_settings(parent):
	s = registry.settings.value
	if not s:
		s = settings.Settings()
	SettingsFrame(s, parent.frame).run()
