import unittest
import storage

from numpy import array

import misc_num

places = 5 #pro assertAlmostEqual
epsilon = 1.0/(10**places)
dtype = 'f'

class MeshTestCase(unittest.TestCase):
	def setUp(self):
		pass

	@staticmethod
	def mesh33():
		"""Vytvori a vrati mrizku

		123
		456
		789
		"""
		m = storage.Mesh()
		coords = (-1.0, 0.0, 1.0)
		m.create_from_function(coords, coords, lambda o,s: 3*(s+1) + o+2)
		return m

	@staticmethod
	def mesh44():
		"""Vytvori a vrati mrizku

		1111
		2222
		3333
		4444
		"""
		m = storage.Mesh()
		coords = (-1.0, 0.0, 1.0, 2.0)
		m.create_from_function(coords, coords, lambda o, s: s + 2)
		return m

	@staticmethod
	def mesh44_stephalf():
		"""Vytvori a vrati mrizku

		1111
		2222
		3333
		4444
		s krokem souradnic 0.5
		"""
		m = storage.Mesh()
		coords = (-0.5, 0.0, 0.5, 1.0)
		m.create_from_function(coords, coords, lambda o, s: s*2 + 2)
		return m


	def assertMeshEqual(self, m1, m2):
		for v1, v2 in zip(m1, m2):
			self.assertVecEqual(v1, v2)

	def assertVecEqual(self, v1, v2):
		"""Vraci true pokud jsou dva vektory (1d pole) stejne"""
		self.assertTrue(misc_num.vec_equals(v1,v2,epsilon))
	
	def test_multiply_bottom(self):
		def test(c, sigma, result):
			m = self.mesh33()
			m.multiply_bottom(c, sigma)
			self.assertMeshEqual(m.mesh.value, array(result, dtype))

		test(0.5, 0, (( (0.5, 1, 1.5), (3.75, 4.5, 5.25), (7, 8, 9))))
		test(0.5, 0.3, (( (0.5, 1, 1.5), (3.75, 4.5, 5.25), (7, 8, 9))))
		test(0.5, -0.3, (( (0.5, 1, 1.5), (3.75, 4.5, 5.25), (7, 8, 9))))
		test(0.5, 1, (( (0.5, 1, 1.5), (2, 2.5, 3), (7, 8, 9))))
		test(0.5, 10, (( (0.5, 1, 1.5), (2, 2.5, 3), (3.5, 4, 4.5))))
		test(0.5, -1, (((1, 2, 3), (4, 5, 6), (7, 8, 9))))
		test(0.5, -10, (((1, 2, 3), (4, 5, 6), (7, 8, 9))))

	def test_read_from_measdata(self):
		#jen velmi jednoduchy test, 3x3 pravidelne rozmistene body
		rd = storage.MeasData()
		test_m = self.mesh33()
		for ridx, r in enumerate(test_m.rows.value):
			for cidx, c in enumerate(test_m.cols.value):
				mp = storage.MeasuredPoint().set_osi(c, r,test_m.mesh.value[ridx][cidx])
				rd.points.append(mp)
		m = rd.as_mesh()
		self.assertVecEqual(m.rows.value, test_m.rows.value)
		self.assertVecEqual(m.cols.value, test_m.cols.value)
		self.assertMeshEqual(m.mesh.value, test_m.mesh.value)

	def test_get_val(self):
		m = self.mesh33()
		self.assertAlmostEqual(m.get_val(0, 0), 5, places)
		self.assertAlmostEqual(m.get_val(-1, -1), 1, places)
		self.assertAlmostEqual(m.get_val(-2, -2), 1, places)
		self.assertAlmostEqual(m.get_val(2, 2), 9, places)
		self.assertAlmostEqual(m.get_val(0.5, 0.5), 7, places)

	def test_find_symmetric_points(self):
		def assertSymetric(mesh, twins, sigma):
			self.assertEqual(set(mesh.find_symmetric_points(sigma)), set(twins))

		assertSymetric(self.mesh44(), [((i, 2), (i, 0)) for i in xrange(4)], 0)
		assertSymetric(self.mesh44(), [((i, 2), (i, 0)) for i in xrange(4)], 0.3)
		assertSymetric(self.mesh44(), [((i, 2), (i, 0)) for i in xrange(4)], -0.3)
		assertSymetric(self.mesh44(), [((i, 3), (i, 1)) for i in xrange(4)], 1)
		assertSymetric(self.mesh44_stephalf(), [((i, 2), (i, 0)) for i in xrange(4)], 0)

		assertSymetric(self.mesh33(), [((i, 2), (i, 0)) for i in xrange(3)], 0)
		assertSymetric(self.mesh33(), [], 0.6)
		assertSymetric(self.mesh33(), [], -0.6)
		assertSymetric(self.mesh33(), [], 100)
		assertSymetric(self.mesh33(), [], -100)

def suite():
	return unittest.makeSuite(MeshTestCase)

if __name__ == '__main__':
	unittest.main()
