# -*- encoding: cp1250 -*-

import wx

from mvc import *
import typetests
import wxPyPlot
import misc
import registry

resources = registry.xresources.typetests
##############################################################################
# Preview ####################################################################
##############################################################################

class Preview(View):
	"""Nahled izokandel typove zkousky"""
	def __init__(self, panel, typetest, focused_item, first_wider = False):
		self.typetest, self.focused_item = typetest, focused_item
		self.first_wider = first_wider
		self.focused_item.add_change_observer(self.update)

		canvas_sizer = wx.BoxSizer(wx.HORIZONTAL)
		panel.SetSizer(canvas_sizer)
		self.plot = wxPyPlot.PlotCanvas(panel)
		canvas_sizer.Add(self.plot, proportion = 1, flag = wx.EXPAND)
		self.plot.SetSize(panel.GetSize())

	def update(self):
		self.plot.Clear()
		polylines = []
		for idx, item in enumerate(self.typetest.isocandels):
			col = misc.ternary(self.focused_item.value == idx, 'cyan', 'blue')
			w = misc.ternary(idx == 0 and self.first_wider, 2, 1)
			for line in item.get_lines():
				l = ((line.x0, line.y0), (line.x1, line.y1))
				polylines.append(wxPyPlot.PolyLine(l, colour=col, width=w))
		if not len(polylines):
			return
		plot_graphics = wxPyPlot.PlotGraphics(polylines, u"Nhled", u"omega", u"sigma")
		self.plot.Draw(plot_graphics)

	def done(self):
		View.done(self)
		self.focused_item.del_change_observer(self.update)

# TypetestFrame #############################################################

class TypetestFrame(DocumentFrame):
	"""Predek framu definic typovych zkousek"""
	frame_xrcid = None
	def __init__(self, model, parent = None):
		DocumentFrame.__init__(self, model, resources, self.frame_xrcid, parent)

		self.attach_panel(resources, 'UNKN_typetest_header', resources, 'Panel_typetest_header')

		self.add_view(StrEditor(self.model.name, self.frame, 'Text_name'))
		self.add_view(StrEditor(self.model.description, self.frame, 'Text_description'))

		self.set_menu('MB_Typetest')
		self.bind_file_events(None, 'MI_Open', 'MI_Save', 'MI_SaveAs', 'MI_Quit')
		self.setup_file_dialogs(self.model.dir_def, '', 
			u"Vyberte soubor s typovou zkoukou",
			u"Vyberte soubor s typovou zkoukou", registry.wildcards.xml)

##############################################################################
# Izokandelove typove zkousky ################################################
##############################################################################

# IsocandelDlg ###############################################################

class IsocandelDlg(Dialog):
	"""Predek dialogu pro definici izokandel"""
	xrc_name = None
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, self.xrc_name, parent)

		m, f = self.model, self.frame
		self.add_view(FloatEditor(m.omega_min, f, 'Text_omega_min', '%.3f'))
		self.add_view(FloatEditor(m.omega_max, f, 'Text_omega_max', '%.3f'))
		self.add_view(FloatEditor(m.sigma_min, f, 'Text_sigma_min', '%.3f'))
		self.add_view(FloatEditor(m.sigma_max, f, 'Text_sigma_max', '%.3f'))

# IsocandelRectangleDlg #####################################################

class IsocandelRectangleDlg(IsocandelDlg):
	"""Dialog definice obdelnikove izokandely"""
	xrc_name = 'Dlg_IsocandelRectangle'
	def __init__(self, model, parent = None):
		IsocandelDlg.__init__(self, model, parent)
		m, f = self.model, self.frame
		self.add_view(FloatEditor(m.intensity, f, 'Text_intensity', '%.1f'))

# IsocandelEllipseDlg #######################################################

class IsocandelEllipseDlg(IsocandelDlg):
	"""Dialog definice elipsovite izokandely"""
	xrc_name = 'Dlg_IsocandelEllipse'
	def __init__(self, model, parent = None):
		IsocandelDlg.__init__(self, model, parent)
		m, f = self.model, self.frame
		self.add_view(FloatEditor(m.intensity, f, 'Text_intensity', '%.1f'))

# IsocandelOvalDlg ##########################################################

class IsocandelOvalDlg(IsocandelDlg):
	"""Dialog definice ovalne izokandely"""
	xrc_name = 'Dlg_IsocandelOval'
	def __init__(self, model, parent = None):
		IsocandelDlg.__init__(self, model, parent)
		m, f = self.model, self.frame
		self.add_view(FloatEditor(m.white_intensity, f, 'Text_white_intensity', '%.1f'))
		self.add_view(FloatEditor(m.red_intensity, f, 'Text_red_intensity', '%.1f'))

# IsocandelRectangleListDef ##################################################

# TODO mozna by to cely stalo za refaktor, jako potomek nejake tridy co se umi
# editnout, vytvorit novou polozku atd.. a pak to zabalit do dalsi tridy ktera
# by umela vybrat ze seznamu techto definic, vytvorit novou polozku atd...
# Koneckoncu, mohla by to bejt nepovinna soucast Modelu, spousta veci by se
# zjednodusila, sakra :-)

class IsocandelRectangleListDef(object):
	"""Definice zobrazovani obdelnikove izokandely pro ListMVCBuilder"""
	title = u"Obdlnk"
	template = typetests.IsocandelRectangle()
	edit_dlg = IsocandelRectangleDlg.run_dialog
	column_display_defs = {
		'type': (lambda li, m: li.SetText(u"Obdlnk")),
		'omega_min': list_text('%.3f', 'omega_min'),
		'omega_max': list_text('%.3f', 'omega_max'),
		'sigma_min': list_text('%.3f', 'sigma_min'),
		'sigma_max': list_text('%.3f', 'sigma_max'),
		'intensity': list_text('%.1f', 'intensity'),
	}

# IsocandelEllipseListDef ###################################################

class IsocandelEllipseListDef(object):
	"""Definice zobrazovani elipsovite izokandely pro ListMVCBuilder"""
	title = u"Elipsa"
	template = typetests.IsocandelEllipse()
	edit_dlg = IsocandelEllipseDlg.run_dialog
	column_display_defs = {
		'type': (lambda li, m: li.SetText(u"Elipsa")),
		'omega_min': list_text('%.3f', 'omega_min'),
		'omega_max': list_text('%.3f', 'omega_max'),
		'sigma_min': list_text('%.3f', 'sigma_min'),
		'sigma_max': list_text('%.3f', 'sigma_max'),
		'intensity': list_text('%.1f', 'intensity'),
	}

# IsocandelOvalListDef ######################################################

class IsocandelOvalListDef(object):
	"""Definice zobrazovani ovalne izokandely pro ListMVCBuilder"""
	title = u"Ovl"
	template = typetests.IsocandelOval()
	edit_dlg = IsocandelOvalDlg.run_dialog
	column_display_defs = {
		'type': (lambda li, m: li.SetText(u"Ovl")),
		'omega_min': list_text('%.3f', 'omega_min'),
		'omega_max': list_text('%.3f', 'omega_max'),
		'sigma_min': list_text('%.3f', 'sigma_min'),
		'sigma_max': list_text('%.3f', 'sigma_max'),
		'intensity': (lambda li, m: li.SetText(u"bl: %.1f, erven: %.1f" % \
		                  (m.white_intensity.value, m.red_intensity.value)))
	 }

# TypetestIsoFrame ##########################################################

class TypetestIsoFrame(TypetestFrame):
	"""Predek oken definic izokandelovych zkousek"""
	isocandel_defs = []
	first_wider = None

	@staticmethod
	def create_range_editors(iso_range_model):
		m, f = iso_range_model.model, iso_range_model.frame
		iso_range_model.add_view(FloatEditor(m.omega_min, f, 'Text_omega_min', '%.3f'))
		iso_range_model.add_view(FloatEditor(m.omega_max, f, 'Text_omega_max', '%.3f'))
		iso_range_model.add_view(FloatEditor(m.omega_step, f, 'Text_omega_step', '%.3f'))
		iso_range_model.add_view(FloatEditor(m.sigma_min, f, 'Text_sigma_min', '%.3f'))
		iso_range_model.add_view(FloatEditor(m.sigma_max, f, 'Text_sigma_max', '%.3f'))
		iso_range_model.add_view(FloatEditor(m.sigma_step, f, 'Text_sigma_step', '%.3f'))

	def __init__(self, model, parent = None):
		TypetestFrame.__init__(self, model, parent)

		m, f = self.model, self.frame
		self.attach_panel(resources, 'UNKN_iso_measure_range', resources, 'Panel_iso_measure_range')
		self.create_range_editors(self)

		focused_item = IntModel(-1)
		self.preview = Preview(self.xrcctrl('Panel_Preview'), m, focused_item,
		                       self.first_wider)
		ilist_mvc_builder = ListMVCBuilder(self.isocandel_defs)
		width_hdr = wx.LIST_AUTOSIZE_USEHEADER
		columns = (
				('type', u"Typ", 70),
				('omega_min', u"Omega od", width_hdr),
				('omega_max', u"Omega do", width_hdr),
				('sigma_min', u"Sigma od", width_hdr),
				('sigma_max', u"Sigma do", width_hdr),
				('intensity', u"Svtivost", width_hdr),
			)
		self.ilist_view = ilist_mvc_builder.build_view(
			m.isocandels, f, 'List_isocandels', columns, focused_item)
		self.ilist_ctrl = ilist_mvc_builder.build_controller(
			m.isocandels, self.ilist_view, 
			'But_isocandels_up', 'But_isocandels_down',
			'But_isocandels_add', 'But_isocandels_del', 'But_isocandels_edit',
			"Nov izokandela", "Vyberte typ izokandely")

# TypetestDirectionalFrame ##################################################

class TypetestDirectionalFrame(TypetestIsoFrame):
	"""Okno definice smerove zkousky"""
	frame_xrcid = 'Frame_Typetest_Directional'
	isocandel_defs = [IsocandelRectangleListDef, IsocandelEllipseListDef]
	first_wider = True
	def __init__(self, model, parent):
		TypetestIsoFrame.__init__(self, model, parent)
		TypetestIsoFrame.__init__(self, model, parent)
		m, f = self.model, self.frame
		self.add_view(FloatEditor(m.brightness, f, 'Text_brightness', '%.1f'))
		self.add_view(FloatEditor(m.uniformity, f, 'Text_uniformity', '%.1f'))
		self.add_view(CheckEditor(m.inset, f, 'Check_inset'))

# TypetestApproachFrame ##################################################

class TypetestApproachFrame(TypetestIsoFrame):
	"""Okno definice sestupove zkousky"""
	frame_xrcid = 'Frame_Typetest_Approach'
	isocandel_defs = [IsocandelOvalListDef]
	first_wider = False
	def __init__(self, model, parent):
		TypetestIsoFrame.__init__(self, model, parent)
		m, f = self.model, self.frame
		self.add_view(FloatEditor(m.sym_min, f, 'Text_sym_min', '%.1f'))
		self.add_view(FloatEditor(m.sym_max, f, 'Text_sym_max', '%.1f'))

#############################################################################
# Polarni typove zkousky (vsesmeraky, svitidla) #############################
#############################################################################

# OmegaSpanDlg ##############################################################
			
class OmegaSpanDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_PolarOmegaSpan', parent)

		m, f = self.model, self.frame
		self.add_view(FloatEditor(m.sigma, f, 'Text_sigma', '%.3f'))
		self.add_view(FloatEditor(m.omega_min, f, 'Text_omega_min', '%.3f'))
		self.add_view(FloatEditor(m.omega_max, f, 'Text_omega_max', '%.3f'))
		self.add_view(FloatEditor(m.omega_step, f, 'Text_omega_step', '%.3f'))
		self.add_view(FloatEditor(m.intensity, f, 'Text_intensity', '%.1f'))

# SigmaSpanDlg ##############################################################

class SigmaSpanDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_PolarSigmaSpan', parent)

		m, f = self.model, self.frame
		self.add_view(FloatEditor(m.omega, f, 'Text_omega', '%.3f'))
		self.add_view(FloatEditor(m.sigma_min, f, 'Text_sigma_min', '%.3f'))
		self.add_view(FloatEditor(m.sigma_max, f, 'Text_sigma_max', '%.3f'))
		self.add_view(FloatEditor(m.sigma_step, f, 'Text_sigma_step', '%.3f'))
		self.add_view(FloatEditor(m.intensity, f, 'Text_intensity', '%.1f'))
		self.add_view(RadioEditor(m.parameter, f, 'Radio_parameter'))

# TypetestPolarFrame #########################################################

class TypetestPolarFrame(TypetestFrame):
	frame_xrcid = 'Frame_Typetest_Polar'

	def __init__(self, model, parent = None):
		TypetestFrame.__init__(self, model, parent)
		m, f = self.model, self.frame
		self.attach_panel(resources, 'UNKN_omega_spans', resources, 'Panel_omega_spans')
		self.add_view(self.create_omega_spans_editor(f, 'List_omega_spans', m.omega_spans))
		self.attach_panel(resources, 'UNKN_sigma_spans', resources, 'Panel_sigma_spans')
		self.add_view(self.create_sigma_spans_editor(f, 'List_sigma_spans', m.sigma_spans))

	@staticmethod
	def create_omega_spans_editor(frame, list_id, spans):
		width_hdr = wx.LIST_AUTOSIZE_USEHEADER
		display_defs = {
			'sigma': (u"Sigma", width_hdr, list_text('%.3f', 'sigma')),
			'omega_min': (u"Od omega", width_hdr, list_text('%.3f', 'omega_min')),
			'omega_max': (u"Do omega", width_hdr, list_text('%.3f', 'omega_max')),
			'omega_step': (u"Krok omega", width_hdr, list_text('%.3f', 'omega_step')),
			'intensity': (u"Intenzita", width_hdr, list_text('%.1f', 'intensity')),
			}
		return \
			MulticolArrayEditor(spans, frame, list_id, 
			None, ('sigma', 'omega_min', 'omega_max', 'omega_step', 'intensity'), 
			display_defs, 
			'But_omega_spans_up', 'But_omega_spans_down', 'But_omega_spans_add',
			'But_omega_spans_del', 'But_omega_spans_edit', OmegaSpanDlg.run_dialog)

	@staticmethod
	def create_sigma_spans_editor(frame, list_id, spans):
		def display_parameter_omega(list_item, model):
			if model.parameter.value == 'value':
				txt = u"%.3f" % model.omega.value
			elif model.parameter.value == 'min':
				txt = u"Minimum"
			elif model.parameter.value == 'max':
				txt = u"Maximum"
			list_item.SetText(txt)

		width_hdr = wx.LIST_AUTOSIZE_USEHEADER
		display_defs = {
			'omega': (u"Omega", width_hdr, display_parameter_omega),
			'sigma_min': (u"Od sigma", width_hdr, list_text('%.3f', 'sigma_min')),
			'sigma_max': (u"Do sigma", width_hdr, list_text('%.3f', 'sigma_max')),
			'sigma_step': (u"Krok sigma", width_hdr, list_text('%.3f', 'sigma_step')),
			'intensity': (u"Intenzita", width_hdr, list_text('%.1f', 'intensity')),
			}
		return \
			MulticolArrayEditor(spans, frame, list_id,
			None, ('omega', 'sigma_min', 'sigma_max', 'sigma_step', 'intensity'), 
			display_defs, 
			'But_sigma_spans_up', 'But_sigma_spans_down', 'But_sigma_spans_add',
			'But_sigma_spans_del', 'But_sigma_spans_edit', SigmaSpanDlg.run_dialog)

# TypetestOmniFrame #########################################################

class TypetestOmniFrame(TypetestPolarFrame):
	frame_title = u"Definice typov zkouky - Typov zkouka vesmrovch nvstidel"

# TypetestLightFrame ########################################################

class TypetestLightFrame(TypetestPolarFrame):
	frame_title = u"Definice typov zkouky - Typov zkouka svtidel"

##############################################################################
# Tabulkove typove zkousky ###################################################
##############################################################################

# PointDlg ###################################################################

class PointDlg(Dialog):
	def __init__(self, model, parent = None):
		Dialog.__init__(self, model, resources, 'Dlg_TablePoint', parent)

		m, f = self.model, self.frame
		self.add_view(FloatEditor(m.omega, f, 'Text_omega', '%.3f'))
		self.add_view(FloatEditor(m.sigma, f, 'Text_sigma', '%.3f'))
		self.add_view(FloatEditor(m.intensity, f, 'Text_intensity', '%.1f'))

# TypetestTableFrame ########################################################

class TypetestTableFrame(TypetestFrame):
	frame_xrcid = 'Frame_Typetest_Table'

	def __init__(self, model, parent = None):
		TypetestFrame.__init__(self, model, parent)
		m, f = self.model, self.frame
		self.attach_panel(resources, 'UNKN_table_points', resources, 'Panel_table_points')
		self.add_view(self.create_points_editor(f, 'List_points', m.points))

	@staticmethod
	def create_points_editor(frame, list_id, points):
		width_hdr = wx.LIST_AUTOSIZE_USEHEADER
		display_defs = {
			'omega': ("Omega", width_hdr, list_text('%.3f', 'omega')),
			'sigma': ("Sigma", width_hdr, list_text('%.3f', 'sigma')),
			'intensity': ("Intenzita", width_hdr, list_text('%.1f', 'intensity')),
			}
		return \
			MulticolArrayEditor(points, frame, list_id,
			None, ('omega', 'sigma', 'intensity'), display_defs, 
			'But_points_up', 'But_points_down', 'But_points_add',
			'But_points_del', 'But_points_edit', PointDlg.run_dialog)

# funkce ####################################################################

def gui_def_typetests_directional(parent):
	"""Spusti dialog pro definici typove zkousky smer. navestidel"""
	TypetestDirectionalFrame(typetests.TypetestDirectional(), parent.frame).run()

def gui_def_typetests_approach(parent):
	"""Spusti dialog pro definici typove zkousky sest. navestidel"""
	TypetestApproachFrame(typetests.TypetestApproach(), parent.frame).run()

def gui_def_typetests_table(parent):
	"""Spusti dialog pro definici typove zkousky tabulkovych mereni"""
	TypetestTableFrame(typetests.TypetestTable(), parent.frame).run()

def gui_def_typetests_omni(parent):
	"""Spusti dialog pro definici typove zkousky vsesmerovych navestidel"""
	TypetestOmniFrame(typetests.TypetestOmni(), parent.frame).run()

def gui_def_typetests_light(parent):
	"""Spusti dialog pro definici typove zkousky vsesmerovych navestidel"""
	TypetestLightFrame(typetests.TypetestLight(), parent.frame).run()
